@@ -25,9 +25,9 @@ namespace Goals
 		{
 		}
 
-		virtual Goals::TGoalVec decompose() const override;
-		virtual std::string toString() const override;
-		virtual bool operator==(const BuildingBehavior & other) const override
+		Goals::TGoalVec decompose(const Nullkiller * ai) const override;
+		std::string toString() const override;
+		bool operator==(const BuildingBehavior & other) const override
 		{
 			return true;
 		}
