@@ -22,7 +22,7 @@ namespace Goals
 		SpellID spellID;
 
 	public:
-		AdventureSpellCast(HeroPtr hero, SpellID spellID)
+		AdventureSpellCast(const CGHeroInstance * hero, SpellID spellID)
 			: ElementarGoal(Goals::ADVENTURE_SPELL_CAST), spellID(spellID)
 		{
 			sethero(hero);
@@ -35,7 +35,7 @@ namespace Goals
 
 		void accept(AIGateway * ai) override;
 		std::string toString() const override;
-		virtual bool operator==(const AdventureSpellCast & other) const override;
+		bool operator==(const AdventureSpellCast & other) const override;
 	};
 }
 
