@@ -1,6 +1,3 @@
-namespace Nullkiller
-{
-
 /*
 * Build.h, part of VCMI engine
 *
@@ -14,6 +11,9 @@ namespace Nullkiller
 
 #include "CGoal.h"
 
+namespace Nullkiller
+{
+
 struct HeroPtr;
 class AIGateway;
 class FuzzyHelper;
@@ -32,7 +32,7 @@ namespace Goals
 		TSubgoal whatToDoToAchieve() override;
 		bool fulfillsMe(TSubgoal goal) override;
 
-		virtual bool operator==(const Build & other) const override
+		bool operator==(const Build & other) const override
 		{
 			return true;
 		}
