@@ -39,8 +39,8 @@ namespace Goals
 		}
 		BuildThis(BuildingID Bid, const CGTownInstance * tid);
 
-		virtual bool operator==(const BuildThis & other) const override;
-		virtual std::string toString() const override;
+		bool operator==(const BuildThis & other) const override;
+		std::string toString() const override;
 		void accept(AIGateway * ai) override;
 	};
 }
