@@ -30,8 +30,8 @@ namespace Goals
 		DefendTown(const CGTownInstance * town, const HitMapInfo & treat, const AIPath & defencePath, bool isCounterAttack = false);
 		DefendTown(const CGTownInstance * town, const HitMapInfo & treat, const CGHeroInstance * defender);
 
-		virtual bool operator==(const DefendTown & other) const override;
-		virtual std::string toString() const override;
+		bool operator==(const DefendTown & other) const override;
+		std::string toString() const override;
 
 		const HitMapInfo & getTreat() const { return treat; }
 
