@@ -14,8 +14,6 @@
 
 #include "../../lib/GameConstants.h"
 #include "../../lib/VCMI_Lib.h"
-#include "../../lib/CTownHandler.h"
-#include "../../lib/CBuildingHandler.h"
 #include "VCAI.h"
 
 struct DLL_EXPORT PotentialBuilding
