@@ -27,6 +27,6 @@ namespace Goals
 		TSubgoal whatToDoToAchieve() override;
 		bool fulfillsMe(TSubgoal goal) override;
 		std::string completeMessage() const override;
-		virtual bool operator==(const VisitObj & other) const override;
+		bool operator==(const VisitObj & other) const override;
 	};
 }
