@@ -25,6 +25,6 @@ namespace AIPathfinding
 		{
 		}
 
-		virtual Goals::TSubgoal whatToDo(const HeroPtr & hero) const override;
+		Goals::TSubgoal whatToDo(const HeroPtr & hero) const override;
 	};
 }
\ No newline at end of file
