@@ -13,6 +13,11 @@
 #include "../../AIUtility.h"
 #include "../../Goals/AbstractGoal.h"
 
+VCMI_LIB_NAMESPACE_BEGIN
+struct PathNodeInfo;
+struct CDestinationNodeInfo;
+VCMI_LIB_NAMESPACE_END
+
 struct AIPathNode;
 
 class ISpecialAction
@@ -29,4 +34,4 @@ public:
 		const AIPathNode * srcNode) const
 	{
 	}
-};
\ No newline at end of file
+};
