@@ -27,6 +27,6 @@ namespace AIPathfinding
 		{
 		}
 
-		virtual Goals::TSubgoal whatToDo(const HeroPtr & hero) const override;
+		Goals::TSubgoal whatToDo(const HeroPtr & hero) const override;
 	};
 }
