@@ -13,6 +13,7 @@
 #include "../gui/CIntObject.h"
 #include "../../lib/FunctionList.h"
 #include "../../lib/battle/BattleHex.h"
+#include "../../lib/texts/MetaString.h"
 #include "../windows/CWindowObject.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
@@ -21,6 +22,7 @@ class CGHeroInstance;
 struct BattleResult;
 struct InfoAboutHero;
 class CStack;
+class CPlayerBattleCallback;
 
 namespace battle
 {
@@ -42,6 +44,8 @@ class CAnimImage;
 class TransparentFilledRectangle;
 class CPlayerInterface;
 class BattleRenderer;
+class VideoWidget;
+class QuickSpellPanel;
 
 /// Class which shows the console at the bottom of the battle screen and manages the text of the console
 class BattleConsole : public CIntObject, public IStatusBar
@@ -70,7 +74,7 @@ private:
 	std::vector<std::string> splitText(const std::string &text);
 
 	/// select line(s) that will be visible in UI
-	std::vector<std::string> getVisibleText();
+	std::vector<std::string> getVisibleText() const;
 public:
 	BattleConsole(const BattleInterface & owner, std::shared_ptr<CPicture> backgroundSource, const Point & objectPos, const Point & imagePos, const Point &size);
 
@@ -145,6 +149,32 @@ public:
 	BattleHero(const BattleInterface & owner, const CGHeroInstance * hero, bool defender);
 };
 
+class QuickSpellPanel : public CIntObject
+{
+private:
+	std::shared_ptr<CFilledTexture> background;
+	std::shared_ptr<TransparentFilledRectangle> rect;
+	std::vector<std::shared_ptr<CButton>> buttons;
+	std::vector<std::shared_ptr<TransparentFilledRectangle>> buttonsIsAutoGenerated;
+	std::vector<std::shared_ptr<TransparentFilledRectangle>> buttonsDisabled;
+	std::vector<std::shared_ptr<CLabel>> labels;
+
+	BattleInterface & owner;
+public:
+	int QUICKSPELL_SLOTS = 12;
+
+	bool isEnabled; // isActive() is not working on multiple conditions, because of this we need a seperate flag
+
+	QuickSpellPanel(BattleInterface & owner);
+
+	void create();
+
+	std::vector<std::tuple<SpellID, bool>> getSpells() const;
+
+	void show(Canvas & to) override;
+	void inputModeChanged(InputMode modi) override;
+};
+
 class HeroInfoBasicPanel : public CIntObject //extracted from InfoWindow to fit better as non-popup embed element
 {
 private:
@@ -185,6 +215,14 @@ public:
 	HeroInfoWindow(const InfoAboutHero & hero, Point * position);
 };
 
+struct BattleResultResources
+{
+	VideoPath prologueVideo;
+	VideoPath loopedVideo;
+	AudioPath musicName;
+	MetaString resultText;
+};
+
 /// Class which is responsible for showing the battle result window
 class BattleResultWindow : public WindowBase
 {
@@ -195,25 +233,10 @@ private:
 	std::shared_ptr<CButton> repeat;
 	std::vector<std::shared_ptr<CAnimImage>> icons;
 	std::shared_ptr<CTextBox> description;
+	std::shared_ptr<VideoWidget> videoPlayer;
 	CPlayerInterface & owner;
 
-	enum BattleResultVideo
-	{
-		NONE,
-		WIN,
-		SURRENDER,
-		RETREAT,
-		RETREAT_LOOP,
-		DEFEAT,
-		DEFEAT_LOOP,
-		DEFEAT_SIEGE,
-		DEFEAT_SIEGE_LOOP,
-		WIN_SIEGE,
-		WIN_SIEGE_LOOP,
-	};
-	BattleResultVideo currentVideo;
-
-	void playVideo(bool startLoop = false);
+	BattleResultResources getResources(const BattleResult & br);
 	
 	void buttonPressed(int button); //internal function for button callbacks
 public:
@@ -224,7 +247,6 @@ public:
 	std::function<void(int result)> resultCallback; //callback receiving which button was pressed
 
 	void activate() override;
-	void show(Canvas & to) override;
 };
 
 /// Shows the stack queue
@@ -238,7 +260,8 @@ class StackQueue : public CIntObject
 		std::shared_ptr<CPicture> background;
 		std::shared_ptr<CAnimImage> icon;
 		std::shared_ptr<CLabel> amount;
-		std::shared_ptr<CAnimImage> stateIcon;
+		std::shared_ptr<CPicture> waitIcon;
+		std::shared_ptr<CPicture> defendIcon;
 		std::shared_ptr<CLabel> round;
 		std::shared_ptr<TransparentFilledRectangle> roundRect;
 
@@ -258,9 +281,6 @@ class StackQueue : public CIntObject
 	std::vector<std::shared_ptr<StackBox>> stackBoxes;
 	BattleInterface & owner;
 
-	std::shared_ptr<CAnimation> icons;
-	std::shared_ptr<CAnimation> stateIcons;
-
 	int32_t getSiegeShooterIconID();
 public:
 	const bool embedded;
