@@ -23,6 +23,13 @@ class InputSourceTouch;
 class InputSourceText;
 class InputSourceGameController;
 
+enum class InputMode
+{
+	KEYBOARD_AND_MOUSE,
+	TOUCH,
+	CONTROLLER
+};
+
 class InputHandler
 {
 	std::vector<SDL_Event> eventsQueue;
@@ -34,6 +41,9 @@ class InputHandler
 	const bool enableTouch;
 	const bool enableController;
 
+	InputMode currentInputMode;
+	void setCurrentInputMode(InputMode modi);
+
 	std::vector<SDL_Event> acquireEvents();
 
 	void preprocessEvent(const SDL_Event & event);
@@ -91,4 +101,8 @@ public:
 	bool isKeyboardCmdDown() const;
 	bool isKeyboardCtrlDown() const;
 	bool isKeyboardShiftDown() const;
+
+	InputMode getCurrentInputMode();
+
+	void copyToClipBoard(const std::string & text);
 };
