@@ -19,6 +19,7 @@
 
 #include "../../lib/CConfigHandler.h"
 #include "../../lib/Rect.h"
+#include "../eventsSDL/InputHandler.h"
 
 template<typename Functor>
 void EventDispatcher::processLists(ui16 activityFlag, const Functor & cb)
@@ -34,12 +35,14 @@ void EventDispatcher::processLists(ui16 activityFlag, const Functor & cb)
 	processList(AEventsReceiver::HOVER, hoverable);
 	processList(AEventsReceiver::MOVE, motioninterested);
 	processList(AEventsReceiver::DRAG, draginterested);
+	processList(AEventsReceiver::DRAG_POPUP, dragPopupInterested);
 	processList(AEventsReceiver::KEYBOARD, keyinterested);
 	processList(AEventsReceiver::TIME, timeinterested);
 	processList(AEventsReceiver::WHEEL, wheelInterested);
 	processList(AEventsReceiver::DOUBLECLICK, doubleClickInterested);
 	processList(AEventsReceiver::TEXTINPUT, textInterested);
 	processList(AEventsReceiver::GESTURE, panningInterested);
+	processList(AEventsReceiver::INPUT_MODE_CHANGE, inputModeChangeInterested);
 }
 
 void EventDispatcher::activateElement(AEventsReceiver * elem, ui16 activityFlag)
@@ -198,10 +201,20 @@ void EventDispatcher::dispatchShowPopup(const Point & position, int tolerance)
 
 void EventDispatcher::dispatchClosePopup(const Point & position)
 {
-	if (GH.windows().isTopWindowPopup())
-		GH.windows().popWindows(1);
+	bool popupOpen = GH.windows().isTopWindowPopup(); // popup can already be closed for mouse dragging with RMB
 
-	assert(!GH.windows().isTopWindowPopup());
+	auto hlp = rclickable;
+
+	for(auto & i : hlp)
+	{
+		if(!vstd::contains(rclickable, i))
+			continue;
+
+		i->closePopupWindow(!popupOpen);
+	}
+
+	if(popupOpen)
+		GH.windows().popWindows(1);
 }
 
 void EventDispatcher::handleLeftButtonClick(const Point & position, int tolerance, bool isPressed)
@@ -251,6 +264,10 @@ void EventDispatcher::handleLeftButtonClick(const Point & position, int toleranc
 				i->mouseClickedState = isPressed;
 				i->clickCancel(position);
 			}
+			else if(isPressed)
+			{
+				i->notFocusedClick();
+			}
 		}
 	}
 }
@@ -304,7 +321,7 @@ void EventDispatcher::dispatchTextInput(const std::string & text)
 {
 	for(auto it : textInterested)
 	{
-		it->textInputed(text);
+		it->textInputted(text);
 	}
 }
 
@@ -316,6 +333,14 @@ void EventDispatcher::dispatchTextEditing(const std::string & text)
 	}
 }
 
+void EventDispatcher::dispatchInputModeChanged(const InputMode & modi)
+{
+	for(auto it : inputModeChangeInterested)
+	{
+		it->inputModeChanged(modi);
+	}
+}
+
 void EventDispatcher::dispatchGesturePanningStarted(const Point & initialPosition)
 {
 	auto copied = panningInterested;
@@ -423,3 +448,10 @@ void EventDispatcher::dispatchMouseDragged(const Point & currentPosition, const
 			elem->mouseDragged(currentPosition, lastUpdateDistance);
 	}
 }
+
+void EventDispatcher::dispatchMouseDraggedPopup(const Point & currentPosition, const Point & lastUpdateDistance)
+{
+	EventReceiversList diCopy = dragPopupInterested;
+	for(auto & elem : diCopy)
+		elem->mouseDraggedPopup(currentPosition, lastUpdateDistance);
+}
