@@ -9,4 +9,17 @@
  */
 #pragma once
 
-enum class ETextAlignment {TOPLEFT, TOPCENTER, CENTER, BOTTOMRIGHT};
+enum class ETextAlignment
+{
+	TOPLEFT,
+	TOPCENTER,
+	TOPRIGHT,
+
+	CENTERLEFT,
+	CENTER,
+	CENTERRIGHT,
+
+	BOTTOMLEFT,
+	BOTTOMCENTER,
+	BOTTOMRIGHT
+};
