@@ -23,7 +23,6 @@ class CSelectionBase;
 class CSavingScreen : public CSelectionBase
 {
 public:
-	const StartInfo * localSi;
 	std::shared_ptr<CMapInfo> localMi;
 
 	CSavingScreen();
@@ -33,4 +32,7 @@ public:
 
 	const CMapInfo * getMapInfo() override;
 	const StartInfo * getStartInfo() override;
+	
+protected:
+	void close() override;
 };
