@@ -10,7 +10,7 @@
 #pragma once
 
 #include "../windows/CWindowObject.h"
-#include "../../lib/JsonNode.h"
+#include "../../lib/json/JsonNode.h"
 #include "../../lib/LoadProgress.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
@@ -24,18 +24,17 @@ class CGStatusBar;
 class CTextBox;
 class CTabbedInt;
 class CAnimImage;
-class CAnimation;
 class CButton;
 class CFilledTexture;
 class CLabel;
-
+class VideoWidget;
 
 // TODO: Find new location for these enums
-enum ESelectionScreen : ui8 {
+enum class ESelectionScreen : ui8 {
 	unknown = 0, newGame, loadGame, saveGame, scenarioInfo, campaignList
 };
 
-enum ELoadMode : ui8
+enum class ELoadMode : ui8
 {
 	NONE = 0, SINGLE, MULTI, CAMPAIGN, TUTORIAL
 };
@@ -48,6 +47,7 @@ class CMenuScreen : public CWindowObject
 	std::shared_ptr<CTabbedInt> tabs;
 
 	std::shared_ptr<CPicture> background;
+	std::shared_ptr<VideoWidget> videoPlayer;
 	std::vector<std::shared_ptr<CPicture>> images;
 
 	std::shared_ptr<CIntObject> createTab(size_t index);
@@ -57,9 +57,8 @@ public:
 
 	CMenuScreen(const JsonNode & configNode);
 
-	void show(Canvas & to) override;
 	void activate() override;
-	void deactivate() override;
+	void show(Canvas & to) override;
 
 	void switchToTab(size_t index);
 	void switchToTab(std::string name);
@@ -84,17 +83,22 @@ public:
 	ESelectionScreen screenType;
 	std::shared_ptr<CPicture> background;
 	std::shared_ptr<CPicture> picture;
+	std::shared_ptr<CTextBox> textTitle;
 	std::shared_ptr<CTextInput> playerName;
 	std::shared_ptr<CButton> buttonHotseat;
+	std::shared_ptr<CButton> buttonLobby;
 	std::shared_ptr<CButton> buttonHost;
 	std::shared_ptr<CButton> buttonJoin;
 	std::shared_ptr<CButton> buttonCancel;
 	std::shared_ptr<CGStatusBar> statusBar;
 
 	CMultiMode(ESelectionScreen ScreenType);
-	void hostTCP();
-	void joinTCP();
-	std::string getPlayerName();
+	void openLobby();
+	void hostTCP(EShortcut shortcut);
+	void joinTCP(EShortcut shortcut);
+
+	/// Get all configured player names. The first name would always be present and initialized to its default value.
+	std::vector<std::string> getPlayersNames();
 
 	void onNameChange(std::string newText);
 };
@@ -116,14 +120,14 @@ class CMultiPlayers : public WindowBase
 	void enterSelectionScreen();
 
 public:
-	CMultiPlayers(const std::string & firstPlayer, ESelectionScreen ScreenType, bool Host, ELoadMode LoadMode);
+	CMultiPlayers(const std::vector<std::string> & playerNames, ESelectionScreen ScreenType, bool Host, ELoadMode LoadMode, EShortcut shortcut);
 };
 
 /// Manages the configuration of pregame GUI elements like campaign screen, main menu, loading screen,...
 class CMainMenuConfig
 {
 public:
-	static CMainMenuConfig & get();
+	static const CMainMenuConfig & get();
 	const JsonNode & getConfig() const;
 	const JsonNode & getCampaigns() const;
 
@@ -139,6 +143,8 @@ class CMainMenu : public CIntObject, public IUpdateable, public std::enable_shar
 {
 	std::shared_ptr<CFilledTexture> backgroundAroundMenu;
 
+	std::vector<VideoPath> videoPlayList;
+
 	CMainMenu(); //Use CMainMenu::create
 
 public:
@@ -148,7 +154,7 @@ public:
 	void activate() override;
 	void onScreenResize() override;
 	void update() override;
-	static void openLobby(ESelectionScreen screenType, bool host, const std::vector<std::string> * names, ELoadMode loadMode);
+	static void openLobby(ESelectionScreen screenType, bool host, const std::vector<std::string> & names, ELoadMode loadMode);
 	static void openCampaignLobby(const std::string & campaignFileName, std::string campaignSet = "");
 	static void openCampaignLobby(std::shared_ptr<CampaignState> campaign);
 	static void startTutorial();
@@ -159,6 +165,8 @@ public:
 
 	static std::shared_ptr<CPicture> createPicture(const JsonNode & config);
 
+	void playIntroVideos();
+	void playMusic();
 };
 
 /// Simple window to enter the server's address.
@@ -175,8 +183,7 @@ class CSimpleJoinScreen : public WindowBase
 	void connectToServer();
 	void leaveScreen();
 	void onChange(const std::string & newText);
-	void startConnectThread(const std::string & addr = {}, ui16 port = 0);
-	void connectThread(const std::string & addr, ui16 port);
+	void startConnection(const std::string & addr = {}, ui16 port = 0);
 
 public:
 	CSimpleJoinScreen(bool host = true);
@@ -190,6 +197,7 @@ class CLoadingScreen : virtual public CWindowObject, virtual public Load::Progre
 
 public:	
 	CLoadingScreen();
+	CLoadingScreen(ImagePath background);
 	~CLoadingScreen();
 
 	void tick(uint32_t msPassed) override;
