@@ -34,7 +34,7 @@ MapViewActions::MapViewActions(MapView & owner, const std::shared_ptr<MapViewMod
 	pos.w = model->getPixelsVisibleDimensions().x;
 	pos.h = model->getPixelsVisibleDimensions().y;
 
-	addUsedEvents(LCLICK | SHOW_POPUP | DRAG | GESTURE | HOVER | MOVE | WHEEL);
+	addUsedEvents(LCLICK | SHOW_POPUP | DRAG | DRAG_POPUP | GESTURE | HOVER | MOVE | WHEEL);
 }
 
 void MapViewActions::setContext(const std::shared_ptr<IMapRendererContext> & context)
@@ -80,6 +80,12 @@ void MapViewActions::showPopupWindow(const Point & cursorPosition)
 		adventureInt->onTileRightClicked(tile);
 }
 
+void MapViewActions::closePopupWindow(bool alreadyClosed)
+{
+	if(alreadyClosed)
+		dragActive = false;
+}
+
 void MapViewActions::mouseMoved(const Point & cursorPosition, const Point & lastUpdateDistance)
 {
 	handleHover(cursorPosition);
@@ -101,6 +107,12 @@ void MapViewActions::mouseDragged(const Point & cursorPosition, const Point & la
 		owner.onMapSwiped(lastUpdateDistance);
 }
 
+void MapViewActions::mouseDraggedPopup(const Point & cursorPosition, const Point & lastUpdateDistance)
+{
+	dragActive = true;
+	owner.onMapSwiped(lastUpdateDistance);
+}
+
 void MapViewActions::gesturePanning(const Point & initialPosition, const Point & currentPosition, const Point & lastUpdateDistance)
 {
 	owner.onMapSwiped(lastUpdateDistance);
