@@ -11,9 +11,13 @@
 
 #include "../render/IImageLoader.h"
 
+class SDLImageShared;
+
 class SDLImageLoader : public IImageLoader
 {
-	SDLImage * image;
+	static constexpr int DEFAULT_PALETTE_COLORS = 256;
+
+	SDLImageShared * image;
 	ui8 * lineStart;
 	ui8 * position;
 public:
@@ -25,7 +29,7 @@ public:
 	//init image with these sizes and palette
 	void init(Point SpriteSize, Point Margins, Point FullSize, SDL_Color *pal);
 
-	SDLImageLoader(SDLImage * Img);
+	SDLImageLoader(SDLImageShared * Img);
 	~SDLImageLoader();
 };
 
