@@ -74,8 +74,7 @@ class CHeroWindow : public CStatusbarWindow, public IGarrisonHolder, public CWin
 	std::shared_ptr<CLabel> specName;
 	std::shared_ptr<MoraleLuckBox> morale;
 	std::shared_ptr<MoraleLuckBox> luck;
-	std::vector<std::shared_ptr<LRClickableAreaWTextComp>> secSkillAreas;
-	std::vector<std::shared_ptr<CAnimImage>> secSkillImages;
+	std::vector< std::shared_ptr<CSecSkillPlace>> secSkills;
 	std::vector<std::shared_ptr<CLabel>> secSkillNames;
 	std::vector<std::shared_ptr<CLabel>> secSkillValues;
 
@@ -100,9 +99,9 @@ public:
 
 	CHeroWindow(const CGHeroInstance * hero);
 
-	void update(const CGHeroInstance * hero, bool redrawNeeded = false); //sets main displayed hero
+	void update() override;
 
-	void dismissCurrent(); //dissmissed currently displayed hero (curHero)
+	void dismissCurrent(); //dismissed currently displayed hero (curHero)
 	void commanderWindow();
 	void switchHero(); //changes displayed hero
 	void updateGarrisons() override;
