@@ -10,19 +10,18 @@
 
 #pragma once
 
+#include "Entity.h"
 VCMI_LIB_NAMESPACE_BEGIN
 
 class PlayerColor;
 class TeamID;
 class IBonusBearer;
 
-class DLL_LINKAGE Player
+class DLL_LINKAGE Player : public EntityWithBonuses<PlayerColor>
 {
 public:
-	virtual PlayerColor getColor() const = 0;
 	virtual TeamID getTeam() const = 0;
 	virtual bool isHuman() const = 0;
-	virtual const IBonusBearer * accessBonuses() const = 0;
 	virtual int getResourceAmount(int type) const = 0;
 };
 
