@@ -44,6 +44,9 @@ public:
 	virtual bool isMagical() const = 0; //Should this spell considered as magical effect or as ability (like dendroid's bind)
 
 	virtual bool hasSchool(SpellSchool school) const = 0;
+	virtual bool canCastOnSelf() const = 0;
+	virtual bool canCastOnlyOnSelf() const = 0;
+	virtual bool canCastWithoutSkip() const = 0;
 	virtual void forEachSchool(const SchoolCallback & cb) const = 0;
 	virtual int32_t getCost(const int32_t skillLevel) const = 0;
 
