@@ -22,14 +22,14 @@ class AboutProjectView : public QWidget
 	Q_OBJECT
 
 	void changeEvent(QEvent *event) override;
+
+	/// Hides a widget and expands second widgets to take place of first widget in layout
+	void hideAndStretchWidget(QGridLayout * layout, QWidget * toHide, QWidget * toStretch);
+
 public:
 	explicit AboutProjectView(QWidget * parent = nullptr);
 
-public slots:
-
 private slots:
-
-
 	void on_updatesButton_clicked();
 
 	void on_openGameDataDir_clicked();
@@ -40,15 +40,14 @@ private slots:
 
 	void on_pushButtonDiscord_clicked();
 
-	void on_pushButtonSlack_clicked();
-
 	void on_pushButtonGithub_clicked();
 
 	void on_pushButtonHomepage_clicked();
 
 	void on_pushButtonBugreport_clicked();
 
+	void on_openConfigDir_clicked();
+
 private:
 	Ui::AboutProjectView * ui;
-
 };
