@@ -15,14 +15,22 @@
 namespace Ui
 {
 class MainWindow;
-const QString teamName = "VCMI Team";
-const QString appName = "VCMI Launcher";
+const QString teamName = "vcmi";
+const QString appName = "launcher";
 }
 
 class QTableWidgetItem;
 class CModList;
 class CModListView;
 
+enum class ETranslationStatus : int8_t
+{
+	NOT_AVAILABLE, // translation for this language was not found in mod list. Could also happen if player is offline or disabled repository checkout
+	NOT_INSTALLLED, // translation mod found, but it is not installed
+	DISABLED, // translation mod found, and installed, but toggled off
+	ACTIVE // translation mod active OR game is already in specified language (e.g. English H3 for players with English language)
+};
+
 class MainWindow : public QMainWindow
 {
 	Q_OBJECT
@@ -30,24 +38,23 @@ class MainWindow : public QMainWindow
 #ifdef ENABLE_QT_TRANSLATIONS
 	QTranslator translator;
 #endif
-private:
 	Ui::MainWindow * ui;
+
 	void load();
-	
+
 	enum TabRows
 	{
 		MODS = 0,
 		SETTINGS = 1,
 		SETUP = 2,
 		ABOUT = 3,
+		START = 4,
 	};
 
-	void changeEvent(QEvent *event) override;
 public:
 	explicit MainWindow(QWidget * parent = nullptr);
 	~MainWindow() override;
 
-	const CModList & getModList() const;
 	CModListView * getModView();
 
 	void updateTranslation();
@@ -55,8 +62,18 @@ public:
 	
 	void detectPreferredLanguage();
 	void enterSetup();
-	void exitSetup();
+	void exitSetup(bool goToMods);
 	void switchToModsTab();
+	void switchToStartTab();
+
+	void dragEnterEvent(QDragEnterEvent* event) override;
+	void dropEvent(QDropEvent *event) override;
+
+	void manualInstallFile(QString filePath);
+	ETranslationStatus getTranslationStatus();
+
+protected:
+	void changeEvent(QEvent * event) override;
 
 public slots:
 	void on_startGameButton_clicked();
@@ -64,6 +81,5 @@ public slots:
 private slots:
 	void on_modslistButton_clicked();
 	void on_settingsButton_clicked();
-	void on_startEditorButton_clicked();
 	void on_aboutButton_clicked();
 };
