@@ -17,23 +17,29 @@ namespace Ui
 class CModListView;
 }
 
-class CModManager;
+class ModStateController;
 class CModList;
-class CModListModel;
+class ModStateItemModel;
+class ModStateModel;
 class CModFilterModel;
 class CDownloadManager;
 class QTableWidgetItem;
 
-class CModEntry;
+class ModState;
 
 class CModListView : public QWidget
 {
 	Q_OBJECT
 
-	std::unique_ptr<CModManager> manager;
-	CModListModel * modModel;
+	std::shared_ptr<ModStateModel> modStateModel;
+	std::unique_ptr<ModStateController> manager;
+	ModStateItemModel * modModel;
 	CModFilterModel * filterModel;
 	CDownloadManager * dlManager;
+	JsonNode accumulatedRepositoryData;
+	QString activatingPreset;
+
+	QStringList enqueuedModDownloads;
 
 	void setupModModel();
 	void setupFilterModel();
@@ -42,31 +48,21 @@ class CModListView : public QWidget
 	void checkManagerErrors();
 
 	/// replace mod ID's with proper human-readable mod names
-	QStringList getModNames(QStringList input);
+	QStringList getModNames(QString queryingMod, QStringList input);
+
+	/// returns list of mods that are needed for install of this mod (potentially including this mod itself)
+	QStringList getModsToInstall(QString mod);
 
 	// find mods unknown to mod list (not present in repo and not installed)
-	QStringList findInvalidDependencies(QString mod);
-	// find mods that block enabling of this mod: conflicting with this mod or one of required mods
-	QStringList findBlockingMods(QString modUnderTest);
-	// find mods that depend on this one
-	QStringList findDependentMods(QString mod, bool excludeDisabled);
-
-	void manualInstallFile(QString filePath);
-	void downloadFile(QString file, QString url, QString description, qint64 size = 0);
-	void downloadFile(QString file, QUrl url, QString description, qint64 size = 0);
+	QStringList findUnavailableMods(QStringList candidates);
 
 	void installMods(QStringList archives);
 	void installMaps(QStringList maps);
-	void installFiles(QStringList mods);
 
-	QString genChangelogText(CModEntry & mod);
-	QString genModInfoText(CModEntry & mod);
+	QString genChangelogText(const ModState & mod);
+	QString genModInfoText(const ModState & mod);
 
 	void changeEvent(QEvent *event) override;
-	void dragEnterEvent(QDragEnterEvent* event) override;
-	void dropEvent(QDropEvent *event) override;
-signals:
-	void modsChanged();
 
 public:
 	explicit CModListView(QWidget * parent = nullptr);
@@ -75,26 +71,53 @@ public:
 	void loadScreenshots();
 	void loadRepositories();
 
+	void reload();
+
 	void disableModInfo();
 
 	void selectMod(const QModelIndex & index);
 
-	const CModList & getModList() const;
-	
 	// First Launch View interface
 
 	/// install mod by name
 	void doInstallMod(const QString & modName);
 
+	/// update mod by name
+	void doUpdateMod(const QString & modName);
+
 	/// returns true if mod is available in repository and can be installed
 	bool isModAvailable(const QString & modName);
 
 	/// finds translation mod for specified languages. Returns empty string on error
 	QString getTranslationModName(const QString & language);
 
+	/// finds all already imported Heroes Chronicles mods (if any)
+	QStringList getInstalledChronicles();
+
+	/// finds all mods that can be updated
+	QStringList getUpdateableMods();
+
+	void createNewPreset(const QString & presetName);
+	void deletePreset(const QString & presetName);
+	void activatePreset(const QString & presetName);
+	void renamePreset(const QString & oldPresetName, const QString & newPresetName);
+
+	QStringList getAllPresets() const;
+	QString getActivePreset() const;
+
+	JsonNode exportCurrentPreset() const;
+	void importPreset(const JsonNode & data);
+
 	/// returns true if mod is currently enabled
 	bool isModEnabled(const QString & modName);
 
+	/// returns true if mod is currently installed
+	bool isModInstalled(const QString & modName);
+
+	void downloadMod(const ModState & mod);
+	void downloadFile(QString file, QUrl url, QString description, qint64 sizeBytes = 0);
+	void installFiles(QStringList mods);
+
 public slots:
 	void enableModByName(QString modName);
 	void disableModByName(QString modName);
@@ -109,31 +132,17 @@ private slots:
 	void hideProgressBar();
 
 	void on_lineEdit_textChanged(const QString & arg1);
-
 	void on_comboBox_currentIndexChanged(int index);
-
 	void on_enableButton_clicked();
-
 	void on_disableButton_clicked();
-
 	void on_updateButton_clicked();
-
 	void on_uninstallButton_clicked();
-
 	void on_installButton_clicked();
-
-	void on_installFromFileButton_clicked();
-
 	void on_pushButton_clicked();
-
 	void on_refreshButton_clicked();
-
 	void on_allModsView_activated(const QModelIndex & index);
-
 	void on_tabWidget_currentChanged(int index);
-
 	void on_screenshotsList_clicked(const QModelIndex & index);
-
 	void on_allModsView_doubleClicked(const QModelIndex &index);
 
 private:
