@@ -14,10 +14,14 @@ namespace Ui
 class CSettingsView;
 }
 
+class MainWindow;
+
 class CSettingsView : public QWidget
 {
 	Q_OBJECT
 
+	MainWindow * getMainWindow();
+
 public:
 	explicit CSettingsView(QWidget * parent = nullptr);
 	~CSettingsView();
@@ -67,6 +71,7 @@ private slots:
 
 	void on_buttonIgnoreSslErrors_clicked(bool checked);
 	void on_comboBoxUpscalingFilter_currentIndexChanged(int index);
+	void on_comboBoxDownscalingFilter_currentIndexChanged(int index);
 	void on_sliderMusicVolume_valueChanged(int value);
 	void on_sliderSoundVolume_valueChanged(int value);
 	void on_buttonRelativeCursorMode_toggled(bool value);
@@ -78,10 +83,25 @@ private slots:
 	void on_sliderToleranceDistanceController_valueChanged(int value);
 	void on_lineEditGameLobbyHost_textChanged(const QString &arg1);
 	void on_spinBoxNetworkPortLobby_valueChanged(int arg1);
-
 	void on_sliderControllerSticksAcceleration_valueChanged(int value);
-
 	void on_sliderControllerSticksSensitivity_valueChanged(int value);
+	void on_sliderScalingFont_valueChanged(int value);
+	void on_buttonFontAuto_clicked(bool checked);
+	void on_buttonFontScalable_clicked(bool checked);
+	void on_buttonFontOriginal_clicked(bool checked);
+
+
+	void on_buttonValidationOff_clicked(bool checked);
+
+	void on_buttonValidationBasic_clicked(bool checked);
+
+	void on_buttonValidationFull_clicked(bool checked);
+
+	void on_sliderScalingCursor_valueChanged(int value);
+
+	void on_buttonScalingAuto_toggled(bool checked);
+
+	void on_buttonHandleBackRightMouseButton_toggled(bool checked);
 
 private:
 	Ui::CSettingsView * ui;
