@@ -9,7 +9,7 @@
  */
 #pragma once
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__OpenBSD__)
 	#include <sys/types.h>
 	#include <sys/time.h>
 	#include <sys/resource.h>
@@ -23,7 +23,9 @@ VCMI_LIB_NAMESPACE_BEGIN
 
 class CStopWatch
 {
-	si64 start, last, mem;
+	si64 start;
+	si64 last;
+	si64 mem;
 
 public:
 	CStopWatch()
@@ -55,7 +57,7 @@ public:
 private:
 	si64 clock() 
 	{
-	#ifdef __FreeBSD__ // TODO: enable also for Apple?
+	#if defined(__FreeBSD__) || defined(__OpenBSD__) // TODO: enable also for Apple?
 		struct rusage usage;
 		getrusage(RUSAGE_SELF, &usage);
 		return static_cast<si64>(usage.ru_utime.tv_sec + usage.ru_stime.tv_sec) * 1000000 + usage.ru_utime.tv_usec + usage.ru_stime.tv_usec;
