@@ -14,7 +14,8 @@
 	#include <windows.h>
 #elif defined(VCMI_HAIKU)
 	#include <OS.h>
-#elif !defined(VCMI_APPLE) && !defined(VCMI_FREEBSD) && !defined(VCMI_HURD)
+#elif !defined(VCMI_APPLE) && !defined(VCMI_FREEBSD) && \
+	!defined(VCMI_HURD) && !defined(VCMI_OPENBSD)
 	#include <sys/prctl.h>
 #endif
 
@@ -54,10 +55,25 @@ void CThreadHelper::processTasks()
 	}
 }
 
-// set name for this thread.
-// NOTE: on *nix string will be trimmed to 16 symbols
+static thread_local std::string threadNameForLogging;
+
+std::string getThreadName()
+{
+	if (!threadNameForLogging.empty())
+		return threadNameForLogging;
+
+	return boost::lexical_cast<std::string>(boost::this_thread::get_id());
+}
+
+void setThreadNameLoggingOnly(const std::string &name)
+{
+	threadNameForLogging = name;
+}
+
 void setThreadName(const std::string &name)
 {
+	threadNameForLogging = name;
+
 #ifdef VCMI_WINDOWS
 #ifndef __GNUC__
 	//follows http://msdn.microsoft.com/en-us/library/xcb2z8hs.aspx
@@ -89,12 +105,16 @@ void setThreadName(const std::string &name)
 //not supported
 #endif
 
-#elif defined(__linux__)
-	prctl(PR_SET_NAME, name.c_str(), 0, 0, 0);
 #elif defined(VCMI_APPLE)
 	pthread_setname_np(name.c_str());
+#elif defined(VCMI_FREEBSD)
+	pthread_setname_np(pthread_self(), name.c_str());
 #elif defined(VCMI_HAIKU)
 	rename_thread(find_thread(NULL), name.c_str());
+#elif defined(VCMI_UNIX)
+	prctl(PR_SET_NAME, name.c_str(), 0, 0, 0);
+#else
+	#error "Failed to find method to set thread name on this system. Please provide one (or disable this line if you just want code to compile)"
 #endif
 }
 
