@@ -22,7 +22,9 @@ public:
 	void run();
 private:
 	boost::mutex rtinm;
-	int currentTask, amount, threads;
+	int currentTask;
+	int amount;
+	int threads;
 	std::vector<Task> *tasks;
 
 
@@ -60,7 +62,9 @@ public:
 	}
 private:
 	boost::mutex rtinm;
-	size_t currentTask, amount, threads;
+	size_t currentTask;
+	size_t amount;
+	size_t threads;
 	Tasks * tasks;
 	std::vector<std::shared_ptr<Payload>> context;
 
@@ -81,7 +85,14 @@ private:
 	}
 };
 
-
+/// Sets thread name that will be used for both logs and debugger (if supported)
+/// WARNING: on Unix-like systems this method should not be used for main thread since it will also change name of the process
 void DLL_LINKAGE setThreadName(const std::string &name);
 
+/// Sets thread name for use in logging only
+void DLL_LINKAGE setThreadNameLoggingOnly(const std::string &name);
+
+/// Returns human-readable thread name that was set before, or string form of system-provided thread ID if no human-readable name was set
+std::string DLL_LINKAGE getThreadName();
+
 VCMI_LIB_NAMESPACE_END
