@@ -9,6 +9,8 @@
  */
 #pragma once
 
+#include "filesystem/ResourcePath.h"
+
 VCMI_LIB_NAMESPACE_BEGIN
 
 class IBonusBearer;
@@ -19,10 +21,10 @@ struct Bonus;
 class DLL_LINKAGE IBonusTypeHandler
 {
 public:
-	virtual ~IBonusTypeHandler(){};
+	virtual ~IBonusTypeHandler() = default;
 
 	virtual std::string bonusToString(const std::shared_ptr<Bonus> & bonus, const IBonusBearer * bearer, bool description) const = 0;
-	virtual std::string bonusToGraphics(const std::shared_ptr<Bonus> & bonus) const = 0;
+	virtual ImagePath bonusToGraphics(const std::shared_ptr<Bonus> & bonus) const = 0;
 };
 
 VCMI_LIB_NAMESPACE_END
