@@ -33,7 +33,7 @@ namespace vstd
 
 		explicit ScopeGuard(Func && f):
 			fire(true),
-			f(std::forward<Func>(f))
+			f(std::move(f))
 		{}
 		~ScopeGuard()
 		{
