@@ -225,7 +225,7 @@ std::vector<JsonNode> ScriptHandler::loadLegacyData()
 ScriptPtr ScriptHandler::loadFromJson(vstd::CLoggerBase * logger, const std::string & scope,
 	const JsonNode & json, const std::string & identifier) const
 {
-	ScriptPtr ret = std::make_shared<ScriptImpl>(this);
+	auto ret = std::make_shared<ScriptImpl>(this);
 
 	JsonDeserializer handler(nullptr, json);
 	ret->identifier = identifier;
@@ -279,7 +279,7 @@ void ScriptHandler::loadState(const JsonNode & state)
 
 		const JsonNode & scriptData = keyValue.second;
 
-		ScriptPtr script = std::make_shared<ScriptImpl>(this);
+		auto script = std::make_shared<ScriptImpl>(this);
 
 		JsonDeserializer handler(nullptr, scriptData);
 		script->serializeJsonState(handler);
