@@ -23,7 +23,7 @@ struct DLL_LINKAGE SiegeInfo
 	// return EWallState decreased by value of damage points
 	static EWallState applyDamage(EWallState state, unsigned int value);
 
-	template <typename Handler> void serialize(Handler &h, const int version)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & wallState;
 		h & gateState;
