@@ -12,7 +12,7 @@
 #include "Bonus.h"
 
 #include "../GameConstants.h"
-#include "../JsonNode.h"
+#include "../json/JsonNode.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
