@@ -10,7 +10,7 @@
 #pragma once
 
 #include "../filesystem/ResourcePath.h"
-#include "../MetaString.h"
+#include "../texts/MetaString.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -22,7 +22,7 @@ struct DLL_LINKAGE CampaignScenarioPrologEpilog
 	AudioPath prologVoice;
 	MetaString prologText;
 
-	template <typename Handler> void serialize(Handler &h, const int formatVersion)
+	template <typename Handler> void serialize(Handler &h)
 	{
 		h & hasPrologEpilog;
 		h & prologVideo;
