@@ -25,33 +25,13 @@ namespace BuildingSubID
 	{
 		DEFAULT = -50,
 		NONE = -1,
-		STABLES,
-		BROTHERHOOD_OF_SWORD,
 		CASTLE_GATE,
-		CREATURE_TRANSFORMER,
 		MYSTIC_POND,
-		FOUNTAIN_OF_FORTUNE,
-		ARTIFACT_MERCHANT,
-		LOOKOUT_TOWER,
 		LIBRARY,
-		MANA_VORTEX,
 		PORTAL_OF_SUMMONING,
 		ESCAPE_TUNNEL,
-		FREELANCERS_GUILD,
-		BALLISTA_YARD,
-		ATTACK_VISITING_BONUS,
-		MAGIC_UNIVERSITY,
-		SPELL_POWER_GARRISON_BONUS,
-		ATTACK_GARRISON_BONUS,
-		DEFENSE_GARRISON_BONUS,
-		DEFENSE_VISITING_BONUS,
-		SPELL_POWER_VISITING_BONUS,
-		KNOWLEDGE_VISITING_BONUS,
-		EXPERIENCE_VISITING_BONUS,
-		LIGHTHOUSE,
 		TREASURY,
-		CUSTOM_VISITING_BONUS,
-		CUSTOM_VISITING_REWARD
+		BANK
 	};
 }
 
@@ -59,7 +39,7 @@ enum class EMarketMode : int8_t
 {
 	RESOURCE_RESOURCE, RESOURCE_PLAYER, CREATURE_RESOURCE, RESOURCE_ARTIFACT,
 	ARTIFACT_RESOURCE, ARTIFACT_EXP, CREATURE_EXP, CREATURE_UNDEAD, RESOURCE_SKILL,
-	MARTKET_AFTER_LAST_PLACEHOLDER
+	MARKET_AFTER_LAST_PLACEHOLDER
 };
 
 enum class EAiTactic : int8_t
@@ -262,4 +242,21 @@ enum class EMovementMode : int8_t
 	TOWN_PORTAL,
 };
 
+enum class EMapLevel : int8_t
+{
+	ANY = -1,
+	SURFACE = 0,
+	UNDERGROUND = 1
+};
+
+enum class EWeekType : int8_t
+{
+	FIRST_WEEK,
+	NORMAL,
+	DOUBLE_GROWTH,
+	BONUS_GROWTH,
+	DEITYOFFIRE,
+	PLAGUE
+};
+
 VCMI_LIB_NAMESPACE_END
