@@ -19,11 +19,6 @@ VCMI_LIB_NAMESPACE_BEGIN
 class DLL_LINKAGE COutputStream : public virtual CStream
 {
 public:
-	/**
-	 * D-tor.
-	 */
-	virtual ~COutputStream() {}
-
 	/**
 	 * Write n bytes from the stream into the data buffer.
 	 *
