@@ -17,10 +17,34 @@
 #include "../filesystem/Filesystem.h"
 #include "../modding/ModScope.h"
 #include "../modding/CModHandler.h"
+#include "../texts/TextOperations.h"
 #include "../ScopeGuard.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
+/// Searches for keys similar to 'target' in 'candidates' map
+/// Returns closest match or empty string if no suitable candidates are found
+static std::string findClosestMatch(const JsonMap & candidates, const std::string & target)
+{
+	// Maximum distance at which we can consider strings to be similar
+	// If strings have more different symbols than this number then it is not a typo, but a completely different word
+	static constexpr int maxDistance = 5;
+	int bestDistance = maxDistance;
+	std::string bestMatch;
+
+	for (auto const & candidate : candidates)
+	{
+		int newDistance = TextOperations::getLevenshteinDistance(candidate.first, target);
+
+		if (newDistance < bestDistance)
+		{
+			bestDistance = newDistance;
+			bestMatch = candidate.first;
+		}
+	}
+	return bestMatch;
+}
+
 static std::string emptyCheck(JsonValidator & validator, const JsonNode & baseSchema, const JsonNode & schema, const JsonNode & data)
 {
 	// check is not needed - e.g. incorporated into another check
@@ -348,7 +372,7 @@ static std::string requiredCheck(JsonValidator & validator, const JsonNode & bas
 	std::string errors;
 	for(const auto & required : schema.Vector())
 	{
-		if (data[required.String()].isNull())
+		if (data[required.String()].isNull() && data.getModScope() != "core")
 			errors += validator.makeErrorMessage("Required entry " + required.String() + " is missing");
 	}
 	return errors;
@@ -417,7 +441,13 @@ static std::string additionalPropertiesCheck(JsonValidator & validator, const Js
 
 			// or, additionalItems field can be bool which indicates if such items are allowed
 			else if(!schema.isNull() && !schema.Bool()) // present and set to false - error
-				errors += validator.makeErrorMessage("Unknown entry found: " + entry.first);
+			{
+				std::string bestCandidate = findClosestMatch(baseSchema["properties"].Struct(), entry.first);
+				if (!bestCandidate.empty())
+					errors += validator.makeErrorMessage("Unknown entry found: '" + entry.first + "'. Perhaps you meant '" + bestCandidate + "'?");
+				else
+					errors += validator.makeErrorMessage("Unknown entry found: " + entry.first);
+			}
 		}
 	}
 	return errors;
@@ -496,6 +526,7 @@ static std::string imageFile(const JsonNode & node)
 static std::string videoFile(const JsonNode & node)
 {
 	TEST_FILE(node.getModScope(), "Video/", node.String(), EResType::VIDEO);
+	TEST_FILE(node.getModScope(), "Video/", node.String(), EResType::VIDEO_LOW_QUALITY);
 	return "Video file \"" + node.String() + "\" was not found";
 }
 #undef TEST_FILE
