@@ -14,9 +14,8 @@
 #include <vcmi/spells/Spell.h>
 #include <vcmi/spells/Service.h>
 
-#include "../CGeneralTextHandler.h"
-#include "../CSoundBase.h"
-#include "../GameSettings.h"
+#include "../texts/CGeneralTextHandler.h"
+#include "../IGameSettings.h"
 #include "../CPlayerState.h"
 #include "../mapObjectConstructors/CObjectClassesHandler.h"
 #include "../mapObjectConstructors/CBankInstanceConstructor.h"
@@ -24,7 +23,6 @@
 #include "../networkPacks/Component.h"
 #include "../networkPacks/PacksForClient.h"
 #include "../networkPacks/PacksForClientBattle.h"
-#include "../MetaString.h"
 #include "../IGameCallback.h"
 #include "../gameState/CGameState.h"
 
@@ -37,12 +35,14 @@ static std::string visitedTxt(const bool visited)
 	return VLC->generaltexth->allTexts[id];
 }
 
-//must be instantiated in .cpp file for access to complete types of all member fields
-CBank::CBank() = default;
+CBank::CBank(IGameCallback *cb)
+	: CArmedInstance(cb)
+{}
+
 //must be instantiated in .cpp file for access to complete types of all member fields
 CBank::~CBank() = default;
 
-void CBank::initObj(CRandomGenerator & rand)
+void CBank::initObj(vstd::RNG & rand)
 {
 	daycounter = 0;
 	resetDuration = 0;
@@ -59,7 +59,7 @@ std::string CBank::getHoverText(PlayerColor player) const
 	if (!wasVisited(player))
 		return getObjectName();
 
-	return getObjectName() + "\n" + visitedTxt(bc == nullptr);
+	return getObjectName() + "\n" + visitedTxt(bankConfig == nullptr);
 }
 
 std::vector<Component> CBank::getPopupComponents(PlayerColor player) const
@@ -67,10 +67,10 @@ std::vector<Component> CBank::getPopupComponents(PlayerColor player) const
 	if (!wasVisited(player))
 		return {};
 
-	if (!VLC->settings()->getBoolean(EGameSettings::BANKS_SHOW_GUARDS_COMPOSITION))
+	if (!cb->getSettings().getBoolean(EGameSettings::BANKS_SHOW_GUARDS_COMPOSITION))
 		return {};
 
-	if (bc == nullptr)
+	if (bankConfig == nullptr)
 		return {};
 
 	std::map<CreatureID, int> guardsAmounts;
@@ -90,11 +90,13 @@ std::vector<Component> CBank::getPopupComponents(PlayerColor player) const
 
 void CBank::setConfig(const BankConfig & config)
 {
-	bc = std::make_unique<BankConfig>(config);
+	bankConfig = std::make_unique<BankConfig>(config);
 	clearSlots(); // remove all stacks, if any
 
 	for(const auto & stack : config.guards)
-		setCreature (SlotID(stacksCount()), stack.type->getId(), stack.count);
+		setCreature (SlotID(stacksCount()), stack.getId(), stack.count);
+
+	daycounter = 1; //yes, 1 since "today" daycounter won't be incremented
 }
 
 void CBank::setPropertyDer (ObjProperty what, ObjPropertyID identifier)
@@ -104,25 +106,24 @@ void CBank::setPropertyDer (ObjProperty what, ObjPropertyID identifier)
 		case ObjProperty::BANK_DAYCOUNTER: //daycounter
 				daycounter+= identifier.getNum();
 			break;
-		case ObjProperty::BANK_RESET:
-			// FIXME: Object reset must be done by separate netpack from server
-			initObj(cb->gameState()->getRandomGenerator());
-			daycounter = 1; //yes, 1 since "today" daycounter won't be incremented
-			break;
 		case ObjProperty::BANK_CLEAR:
-			bc.reset();
+			bankConfig.reset();
 			break;
 	}
 }
 
-void CBank::newTurn(CRandomGenerator & rand) const
+void CBank::newTurn(vstd::RNG & rand) const
 {
-	if (bc == nullptr)
+	if (bankConfig == nullptr)
 	{
 		if (resetDuration != 0)
 		{
 			if (daycounter >= resetDuration)
-				cb->setObjPropertyValue(id, ObjProperty::BANK_RESET); //daycounter 0
+			{
+				auto handler = std::dynamic_pointer_cast<CBankInstanceConstructor>(getObjectHandler());
+				auto config = handler->generateConfiguration(cb, rand, ID);
+				cb->setBankObjectConfiguration(id, config);
+			}
 			else
 				cb->setObjPropertyValue(id, ObjProperty::BANK_DAYCOUNTER, 1); //daycounter++
 		}
@@ -136,169 +137,65 @@ bool CBank::wasVisited (PlayerColor player) const
 
 void CBank::onHeroVisit(const CGHeroInstance * h) const
 {
-	ChangeObjectVisitors cov(ChangeObjectVisitors::VISITOR_ADD_TEAM, id, h->id);
-	cb->sendAndApply(&cov);
+	ChangeObjectVisitors cov(ChangeObjectVisitors::VISITOR_ADD_PLAYER, id, h->id);
+	cb->sendAndApply(cov);
 
-	int banktext = 0;
-	switch (ID.toEnum())
-	{
-	case Obj::DERELICT_SHIP:
-		banktext = 41;
-		break;
-	case Obj::DRAGON_UTOPIA:
-		banktext = 47;
-		break;
-	case Obj::CRYPT:
-		banktext = 119;
-		break;
-	case Obj::SHIPWRECK:
-		banktext = 122;
-		break;
-	case Obj::PYRAMID:
-		banktext = 105;
-		break;
-	case Obj::CREATURE_BANK:
-	default:
-		banktext = 32;
-		break;
-	}
 	BlockingDialog bd(true, false);
 	bd.player = h->getOwner();
-	bd.soundID = soundBase::invalid; // Sound is handled in json files, else two sounds are played
-	bd.text.appendLocalString(EMetaText::ADVOB_TXT, banktext);
+	bd.text.appendLocalString(EMetaText::ADVOB_TXT, 32);
 	bd.components = getPopupComponents(h->getOwner());
-	if (banktext == 32)
-		bd.text.replaceRawString(getObjectName());
-
-	cb->showBlockingDialog(&bd);
+	bd.text.replaceTextID(getObjectHandler()->getNameTextID());
+	cb->showBlockingDialog(this, &bd);
 }
 
 void CBank::doVisit(const CGHeroInstance * hero) const
 {
-	int textID = -1;
 	InfoWindow iw;
 	iw.type = EInfoWindowMode::AUTO;
 	iw.player = hero->getOwner();
 	MetaString loot;
 
-	if (bc)
+	if (!bankConfig)
 	{
-		switch (ID.toEnum())
-		{
-		case Obj::DERELICT_SHIP:
-			textID = 43;
-			break;
-		case Obj::CRYPT:
-			textID = 121;
-			break;
-		case Obj::SHIPWRECK:
-			textID = 124;
-			break;
-		case Obj::PYRAMID:
-			textID = 106;
-			break;
-		case Obj::CREATURE_BANK:
-		case Obj::DRAGON_UTOPIA:
-		default:
-			textID = 34;
-			break;
-		}
-	}
-	else
-	{
-		switch (ID.toEnum())
-		{
-		case Obj::SHIPWRECK:
-		case Obj::DERELICT_SHIP:
-		case Obj::CRYPT:
-		{
-			GiveBonus gbonus;
-			gbonus.id = hero->id;
-			gbonus.bonus.duration = BonusDuration::ONE_BATTLE;
-			gbonus.bonus.source = BonusSource::OBJECT_TYPE;
-			gbonus.bonus.sid = BonusSourceID(ID);
-			gbonus.bonus.type = BonusType::MORALE;
-			gbonus.bonus.val = -1;
-			switch (ID.toEnum())
-			{
-			case Obj::SHIPWRECK:
-				textID = 123;
-				gbonus.bdescr.appendRawString(VLC->generaltexth->arraytxt[99]);
-				break;
-			case Obj::DERELICT_SHIP:
-				textID = 42;
-				gbonus.bdescr.appendRawString(VLC->generaltexth->arraytxt[101]);
-				break;
-			case Obj::CRYPT:
-				textID = 120;
-				gbonus.bdescr.appendRawString(VLC->generaltexth->arraytxt[98]);
-				break;
-			}
-			cb->giveHeroBonus(&gbonus);
-			iw.components.emplace_back(ComponentType::MORALE, -1);
-			iw.soundID = soundBase::GRAVEYARD;
-			break;
-		}
-		case Obj::PYRAMID:
-		{
-			GiveBonus gb;
-			gb.bonus = Bonus(BonusDuration::ONE_BATTLE, BonusType::LUCK, BonusSource::OBJECT_INSTANCE, -2, BonusSourceID(id), VLC->generaltexth->arraytxt[70]);
-			gb.id = hero->id;
-			cb->giveHeroBonus(&gb);
-			textID = 107;
-			iw.components.emplace_back(ComponentType::LUCK, -2);
-			break;
-		}
-		case Obj::CREATURE_BANK:
-		case Obj::DRAGON_UTOPIA:
-		default:
-			iw.text.appendRawString(VLC->generaltexth->advobtxt[33]);// This was X, now is completely empty
-			iw.text.replaceRawString(getObjectName());
-		}
-		if(textID != -1)
-		{
-			iw.text.appendLocalString(EMetaText::ADVOB_TXT, textID);
-		}
+		iw.text.appendRawString(VLC->generaltexth->advobtxt[33]);// This was X, now is completely empty
+		iw.text.replaceTextID(getObjectHandler()->getNameTextID());
 		cb->showInfoDialog(&iw);
 	}
 
-
 	//grant resources
-	if (bc)
+	if (bankConfig)
 	{
 		for (GameResID it : GameResID::ALL_RESOURCES())
 		{
-			if (bc->resources[it] != 0)
+			if (bankConfig->resources[it] != 0)
 			{
-				iw.components.emplace_back(ComponentType::RESOURCE, it, bc->resources[it]);
+				iw.components.emplace_back(ComponentType::RESOURCE, it, bankConfig->resources[it]);
 				loot.appendRawString("%d %s");
-				loot.replaceNumber(bc->resources[it]);
+				loot.replaceNumber(bankConfig->resources[it]);
 				loot.replaceName(it);
-				cb->giveResource(hero->getOwner(), it, bc->resources[it]);
+				cb->giveResource(hero->getOwner(), it, bankConfig->resources[it]);
 			}
 		}
 		//grant artifacts
-		for (auto & elem : bc->artifacts)
+		for (auto & elem : bankConfig->artifacts)
 		{
 			iw.components.emplace_back(ComponentType::ARTIFACT, elem);
 			loot.appendRawString("%s");
 			loot.replaceName(elem);
-			cb->giveHeroNewArtifact(hero, elem.toArtifact(), ArtifactPosition::FIRST_AVAILABLE);
+			cb->giveHeroNewArtifact(hero, elem, ArtifactPosition::FIRST_AVAILABLE);
 		}
 		//display loot
 		if (!iw.components.empty())
 		{
-			iw.text.appendLocalString(EMetaText::ADVOB_TXT, textID);
-			if (textID == 34)
+			iw.text.appendLocalString(EMetaText::ADVOB_TXT, 34);
+			const auto * strongest = boost::range::max_element(bankConfig->guards, [](const CStackBasicDescriptor & a, const CStackBasicDescriptor & b)
 			{
-				const auto * strongest = boost::range::max_element(bc->guards, [](const CStackBasicDescriptor & a, const CStackBasicDescriptor & b)
-				{
-					return a.type->getFightValue() < b.type->getFightValue();
-				})->type;
+				return a.getType()->getFightValue() < b.getType()->getFightValue();
+			})->getType();
+
+			iw.text.replaceNamePlural(strongest->getId());
+			iw.text.replaceRawString(loot.buildList());
 
-				iw.text.replaceNamePlural(strongest->getId());
-				iw.text.replaceRawString(loot.buildList());
-			}
 			cb->showInfoDialog(&iw);
 		}
 
@@ -306,16 +203,13 @@ void CBank::doVisit(const CGHeroInstance * hero) const
 		iw.components.clear();
 		iw.text.clear();
 
-		if (!bc->spells.empty())
+		if (!bankConfig->spells.empty())
 		{
 			std::set<SpellID> spells;
 
 			bool noWisdom = false;
-			if(textID == 106)
-			{
-				iw.text.appendLocalString(EMetaText::ADVOB_TXT, textID); //pyramid
-			}
-			for(const SpellID & spellId : bc->spells)
+
+			for(const SpellID & spellId : bankConfig->spells)
 			{
 				const auto * spell = spellId.toEntity(VLC);
 				iw.text.appendName(spellId);
@@ -348,9 +242,9 @@ void CBank::doVisit(const CGHeroInstance * hero) const
 
 		//grant creatures
 		CCreatureSet ourArmy;
-		for(const auto & slot : bc->creatures)
+		for(const auto & slot : bankConfig->creatures)
 		{
-			ourArmy.addToSlot(ourArmy.getSlotFor(slot.type->getId()), slot.type->getId(), slot.count);
+			ourArmy.addToSlot(ourArmy.getSlotFor(slot.getId()), slot.getId(), slot.count);
 		}
 
 		for(const auto & elem : ourArmy.Slots())
@@ -378,18 +272,18 @@ void CBank::doVisit(const CGHeroInstance * hero) const
 
 void CBank::battleFinished(const CGHeroInstance *hero, const BattleResult &result) const
 {
-	if (result.winner == 0)
+	if (result.winner == BattleSide::ATTACKER)
 	{
 		doVisit(hero);
 	}
 }
 
-void CBank::blockingDialogAnswered(const CGHeroInstance *hero, ui32 answer) const
+void CBank::blockingDialogAnswered(const CGHeroInstance *hero, int32_t answer) const
 {
 	if (answer)
 	{
-		if (bc) // not looted bank
-			cb->startBattleI(hero, this, true);
+		if (bankConfig) // not looted bank
+			cb->startBattle(hero, this);
 		else
 			doVisit(hero);
 	}
