@@ -12,8 +12,8 @@
 #include "MapIdentifiersH3M.h"
 
 #include "../VCMI_Lib.h"
-#include "../CTownHandler.h"
-#include "../CHeroHandler.h"
+#include "../entities/faction/CFaction.h"
+#include "../entities/faction/CTownHandler.h"
 #include "../filesystem/Filesystem.h"
 #include "../mapObjectConstructors/AObjectTypeHandler.h"
 #include "../mapObjectConstructors/CObjectClassesHandler.h"
@@ -116,6 +116,22 @@ void MapIdentifiersH3M::remapTemplate(ObjectTemplate & objectTemplate)
 		objectTemplate.subid = mappedType.subID;
 	}
 
+	if (VLC->objtypeh->knownObjects().count(objectTemplate.id) == 0)
+	{
+		logGlobal->warn("Unknown object found: %d | %d", objectTemplate.id, objectTemplate.subid);
+
+		objectTemplate.id = Obj::NOTHING;
+		objectTemplate.subid = {};
+	}
+	else
+	{
+		if (VLC->objtypeh->knownSubObjects(objectTemplate.id).count(objectTemplate.subid) == 0)
+		{
+			logGlobal->warn("Unknown subobject found: %d | %d", objectTemplate.id, objectTemplate.subid);
+			objectTemplate.subid = {};
+		}
+	}
+
 	if (objectTemplate.id == Obj::TOWN || objectTemplate.id == Obj::RANDOM_DWELLING_FACTION)
 		objectTemplate.subid = remap(FactionID(objectTemplate.subid));
 
