@@ -11,7 +11,7 @@
 #include "ActiveModsInSaveList.h"
 
 #include "../VCMI_Lib.h"
-#include "CModInfo.h"
+#include "ModDescription.h"
 #include "CModHandler.h"
 #include "ModIncompatibility.h"
 
@@ -21,13 +21,13 @@ std::vector<TModID> ActiveModsInSaveList::getActiveGameplayAffectingMods()
 {
 	std::vector<TModID> result;
 	for (auto const & entry : VLC->modh->getActiveMods())
-		if (VLC->modh->getModInfo(entry).checkModGameplayAffecting())
+		if (VLC->modh->getModInfo(entry).affectsGameplay())
 			result.push_back(entry);
 
 	return result;
 }
 
-const ModVerificationInfo & ActiveModsInSaveList::getVerificationInfo(TModID mod)
+ModVerificationInfo ActiveModsInSaveList::getVerificationInfo(TModID mod)
 {
 	return VLC->modh->getModInfo(mod).getVerificationInfo();
 }
@@ -44,10 +44,10 @@ void ActiveModsInSaveList::verifyActiveMods(const std::map<TModID, ModVerificati
 			missingMods.push_back(modList.at(compared.first).name);
 
 		if (compared.second == ModVerificationStatus::DISABLED)
-			missingMods.push_back(VLC->modh->getModInfo(compared.first).getVerificationInfo().name);
+			missingMods.push_back(VLC->modh->getModInfo(compared.first).getName());
 
 		if (compared.second == ModVerificationStatus::EXCESSIVE)
-			excessiveMods.push_back(VLC->modh->getModInfo(compared.first).getVerificationInfo().name);
+			excessiveMods.push_back(VLC->modh->getModInfo(compared.first).getName());
 	}
 
 	if(!missingMods.empty() || !excessiveMods.empty())
