@@ -15,7 +15,11 @@
 
 VCMI_LIB_NAMESPACE_BEGIN
 
+#if BOOST_VERSION >= 108700
+using NetworkContext = boost::asio::io_context;
+#else
 using NetworkContext = boost::asio::io_service;
+#endif
 using NetworkSocket = boost::asio::ip::tcp::socket;
 using NetworkAcceptor = boost::asio::ip::tcp::acceptor;
 using NetworkBuffer = boost::asio::streambuf;
