@@ -18,12 +18,13 @@
 #include "ObjProperty.h"
 
 #include "../CCreatureSet.h"
-#include "../MetaString.h"
 #include "../ResourceSet.h"
 #include "../TurnTimerInfo.h"
 #include "../gameState/EVictoryLossCheckResult.h"
+#include "../gameState/RumorState.h"
 #include "../gameState/QuestInfo.h"
 #include "../gameState/TavernSlot.h"
+#include "../gameState/GameStatistics.h"
 #include "../int3.h"
 #include "../mapping/CMapDefines.h"
 #include "../spells/ViewSpellInt.h"
@@ -56,6 +57,7 @@ struct DLL_LINKAGE PackageApplied : public CPackForClient
 	{
 	}
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState *gs) override {}
 
 	ui8 result = 0; //0 - something went wrong, request hasn't been realized; 1 - OK
 	ui32 packType = 0; //type id of applied package
@@ -80,6 +82,7 @@ struct DLL_LINKAGE SystemMessage : public CPackForClient
 	SystemMessage() = default;
 
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState *gs) override {}
 
 	MetaString text;
 
@@ -99,6 +102,7 @@ struct DLL_LINKAGE PlayerBlocked : public CPackForClient
 	PlayerColor player;
 
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState *gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -110,7 +114,7 @@ struct DLL_LINKAGE PlayerBlocked : public CPackForClient
 
 struct DLL_LINKAGE PlayerCheated : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	PlayerColor player;
 	bool losingCheatCode = false;
@@ -128,7 +132,7 @@ struct DLL_LINKAGE PlayerCheated : public CPackForClient
 
 struct DLL_LINKAGE TurnTimeUpdate : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 	
 	PlayerColor player;
 	TurnTimerInfo turnTimer;
@@ -142,7 +146,7 @@ struct DLL_LINKAGE TurnTimeUpdate : public CPackForClient
 
 struct DLL_LINKAGE PlayerStartsTurn : public Query
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	PlayerColor player;
 
@@ -157,7 +161,7 @@ struct DLL_LINKAGE PlayerStartsTurn : public Query
 
 struct DLL_LINKAGE DaysWithoutTown : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	PlayerColor player;
 	std::optional<int32_t> daysWithoutCastle;
@@ -175,7 +179,7 @@ struct DLL_LINKAGE EntitiesChanged : public CPackForClient
 {
 	std::vector<EntityChanges> changes;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -187,7 +191,7 @@ struct DLL_LINKAGE EntitiesChanged : public CPackForClient
 
 struct DLL_LINKAGE SetResources : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -205,7 +209,7 @@ struct DLL_LINKAGE SetResources : public CPackForClient
 
 struct DLL_LINKAGE SetPrimSkill : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -225,7 +229,7 @@ struct DLL_LINKAGE SetPrimSkill : public CPackForClient
 
 struct DLL_LINKAGE SetSecSkill : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -245,7 +249,7 @@ struct DLL_LINKAGE SetSecSkill : public CPackForClient
 
 struct DLL_LINKAGE HeroVisitCastle : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -268,7 +272,7 @@ struct DLL_LINKAGE HeroVisitCastle : public CPackForClient
 
 struct DLL_LINKAGE ChangeSpells : public CPackForClient
 {
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -284,12 +288,39 @@ struct DLL_LINKAGE ChangeSpells : public CPackForClient
 	}
 };
 
-struct DLL_LINKAGE SetMana : public CPackForClient
+struct DLL_LINKAGE SetResearchedSpells : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
+	ui8 level = 0;
+	ObjectInstanceID tid;
+	std::vector<SpellID> spells;
+	bool accepted;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & level;
+		h & tid;
+		h & spells;
+		h & accepted;
+	}
+};
+
+struct DLL_LINKAGE SetMana : public CPackForClient
+{
+	void applyGs(CGameState * gs) override;
+
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	SetMana() = default;
+	SetMana(ObjectInstanceID hid, si32 val, bool absolute)
+		: hid(hid)
+		, val(val)
+		, absolute(absolute)
+	{}
+
 	ObjectInstanceID hid;
 	si32 val = 0;
 	bool absolute = true;
@@ -304,7 +335,14 @@ struct DLL_LINKAGE SetMana : public CPackForClient
 
 struct DLL_LINKAGE SetMovePoints : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
+
+	SetMovePoints() = default;
+	SetMovePoints(ObjectInstanceID hid, si32 val, bool absolute)
+		: hid(hid)
+		, val(val)
+		, absolute(absolute)
+	{}
 
 	ObjectInstanceID hid;
 	si32 val = 0;
@@ -322,7 +360,7 @@ struct DLL_LINKAGE SetMovePoints : public CPackForClient
 
 struct DLL_LINKAGE FoWChange : public CPackForClient
 {
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	std::unordered_set<int3> tiles;
 	PlayerColor player;
@@ -346,7 +384,7 @@ struct DLL_LINKAGE SetAvailableHero : public CPackForClient
 	{
 		army.clearSlots();
 	}
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	TavernHeroSlot slotID;
 	TavernSlotRole roleID;
@@ -377,7 +415,7 @@ struct DLL_LINKAGE GiveBonus : public CPackForClient
 	{
 	}
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	ETarget who = ETarget::OBJECT;
 	VariantIdentifier<ObjectInstanceID, PlayerColor, BattleID> id;
@@ -396,7 +434,7 @@ struct DLL_LINKAGE GiveBonus : public CPackForClient
 
 struct DLL_LINKAGE ChangeObjPos : public CPackForClient
 {
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	/// Object to move
 	ObjectInstanceID objid;
@@ -417,7 +455,7 @@ struct DLL_LINKAGE ChangeObjPos : public CPackForClient
 
 struct DLL_LINKAGE PlayerEndsTurn : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	PlayerColor player;
 
@@ -431,10 +469,11 @@ struct DLL_LINKAGE PlayerEndsTurn : public CPackForClient
 
 struct DLL_LINKAGE PlayerEndsGame : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	PlayerColor player;
 	EVictoryLossCheckResult victoryLossCheckResult;
+	StatisticDataSet statistic;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -442,12 +481,14 @@ struct DLL_LINKAGE PlayerEndsGame : public CPackForClient
 	{
 		h & player;
 		h & victoryLossCheckResult;
+		if (h.version >= Handler::Version::STATISTICS_SCREEN)
+			h & statistic;
 	}
 };
 
 struct DLL_LINKAGE PlayerReinitInterface : public CPackForClient
 {
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	std::vector<PlayerColor> players;
 	ui8 playerConnectionId; //PLAYER_AI for AI player
@@ -468,7 +509,7 @@ struct DLL_LINKAGE RemoveBonus : public CPackForClient
 	{
 	}
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	GiveBonus::ETarget who; //who receives bonus
 	VariantIdentifier<HeroTypeID, PlayerColor, BattleID, ObjectInstanceID> whoID;
@@ -495,7 +536,7 @@ struct DLL_LINKAGE SetCommanderProperty : public CPackForClient
 {
 	enum ECommanderProperty { ALIVE, BONUS, SECONDARY_SKILL, EXPERIENCE, SPECIAL_SKILL };
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	ObjectInstanceID heroid;
 
@@ -518,7 +559,7 @@ struct DLL_LINKAGE SetCommanderProperty : public CPackForClient
 
 struct DLL_LINKAGE AddQuest : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	PlayerColor player;
 	QuestInfo quest;
@@ -536,7 +577,7 @@ struct DLL_LINKAGE UpdateArtHandlerLists : public CPackForClient
 {
 	std::map<ArtifactID, int> allocatedArtifacts;
 
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -545,40 +586,12 @@ struct DLL_LINKAGE UpdateArtHandlerLists : public CPackForClient
 	}
 };
 
-struct DLL_LINKAGE UpdateMapEvents : public CPackForClient
-{
-	std::list<CMapEvent> events;
-
-	void applyGs(CGameState * gs) const;
-	void visitTyped(ICPackVisitor & visitor) override;
-
-	template <typename Handler> void serialize(Handler & h)
-	{
-		h & events;
-	}
-};
-
-struct DLL_LINKAGE UpdateCastleEvents : public CPackForClient
-{
-	ObjectInstanceID town;
-	std::list<CCastleEvent> events;
-
-	void applyGs(CGameState * gs) const;
-	void visitTyped(ICPackVisitor & visitor) override;
-
-	template <typename Handler> void serialize(Handler & h)
-	{
-		h & town;
-		h & events;
-	}
-};
-
 struct DLL_LINKAGE ChangeFormation : public CPackForClient
 {
 	ObjectInstanceID hid;
 	EArmyFormation formation{};
 
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -597,7 +610,7 @@ struct DLL_LINKAGE RemoveObject : public CPackForClient
 	{
 	}
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	/// ID of removed object
@@ -615,7 +628,7 @@ struct DLL_LINKAGE RemoveObject : public CPackForClient
 
 struct DLL_LINKAGE TryMoveHero : public CPackForClient
 {
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	enum EResult
 	{
@@ -656,11 +669,11 @@ struct DLL_LINKAGE TryMoveHero : public CPackForClient
 
 struct DLL_LINKAGE NewStructures : public CPackForClient
 {
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	ObjectInstanceID tid;
 	std::set<BuildingID> bid;
-	si16 builded = 0;
+	si16 built = 0;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -668,13 +681,13 @@ struct DLL_LINKAGE NewStructures : public CPackForClient
 	{
 		h & tid;
 		h & bid;
-		h & builded;
+		h & built;
 	}
 };
 
 struct DLL_LINKAGE RazeStructures : public CPackForClient
 {
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	ObjectInstanceID tid;
 	std::set<BuildingID> bid;
@@ -692,7 +705,7 @@ struct DLL_LINKAGE RazeStructures : public CPackForClient
 
 struct DLL_LINKAGE SetAvailableCreatures : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	ObjectInstanceID tid;
 	std::vector<std::pair<ui32, std::vector<CreatureID> > > creatures;
@@ -708,7 +721,7 @@ struct DLL_LINKAGE SetAvailableCreatures : public CPackForClient
 
 struct DLL_LINKAGE SetHeroesInTown : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	ObjectInstanceID tid; //id of town
 	ObjectInstanceID visiting; //id of visiting hero
@@ -726,7 +739,7 @@ struct DLL_LINKAGE SetHeroesInTown : public CPackForClient
 
 struct DLL_LINKAGE HeroRecruited : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	HeroTypeID hid; //subID of hero
 	ObjectInstanceID tid;
@@ -748,7 +761,7 @@ struct DLL_LINKAGE HeroRecruited : public CPackForClient
 
 struct DLL_LINKAGE GiveHero : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	ObjectInstanceID id; //object id
 	ObjectInstanceID boatId;
@@ -771,6 +784,7 @@ struct DLL_LINKAGE OpenWindow : public Query
 	ObjectInstanceID visitor;
 
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState *gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -783,37 +797,29 @@ struct DLL_LINKAGE OpenWindow : public Query
 
 struct DLL_LINKAGE NewObject : public CPackForClient
 {
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	/// Object ID to create
-	MapObjectID ID;
-	/// Object secondary ID to create
-	MapObjectSubID subID;
-	/// Position of visitable tile of created object
-	int3 targetPos;
+	CGObjectInstance * newObject;
 	/// Which player initiated creation of this object
 	PlayerColor initiator;
 
-	ObjectInstanceID createdObjectID; //used locally, filled during applyGs
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
-		h & ID;
-		subID.serializeIdentifier(h, ID);
-		h & targetPos;
+		h & newObject;
 		h & initiator;
 	}
 };
 
 struct DLL_LINKAGE SetAvailableArtifacts : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	//two variants: id < 0: set artifact pool for Artifact Merchants in towns; id >= 0: set pool for adv. map Black Market (id is the id of Black Market instance then)
 	ObjectInstanceID id;
-	std::vector<const CArtifact *> arts;
+	std::vector<ArtifactID> arts;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -835,7 +841,7 @@ struct DLL_LINKAGE ChangeStackCount : CGarrisonOperationPack
 	TQuantity count;
 	bool absoluteValue; //if not -> count will be added (or subtracted if negative)
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -854,7 +860,7 @@ struct DLL_LINKAGE SetStackType : CGarrisonOperationPack
 	SlotID slot;
 	CreatureID type;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -871,7 +877,7 @@ struct DLL_LINKAGE EraseStack : CGarrisonOperationPack
 	ObjectInstanceID army;
 	SlotID slot;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -888,7 +894,7 @@ struct DLL_LINKAGE SwapStacks : CGarrisonOperationPack
 	SlotID srcSlot;
 	SlotID dstSlot;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -907,7 +913,7 @@ struct DLL_LINKAGE InsertNewStack : CGarrisonOperationPack
 	CreatureID type;
 	TQuantity count = 0;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -929,7 +935,7 @@ struct DLL_LINKAGE RebalanceStacks : CGarrisonOperationPack
 
 	TQuantity count;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -946,7 +952,7 @@ struct DLL_LINKAGE BulkRebalanceStacks : CGarrisonOperationPack
 {
 	std::vector<RebalanceStacks> moves;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler>
@@ -961,7 +967,7 @@ struct DLL_LINKAGE BulkSmartRebalanceStacks : CGarrisonOperationPack
 	std::vector<RebalanceStacks> moves;
 	std::vector<ChangeStackCount> changes;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler>
@@ -979,73 +985,59 @@ struct DLL_LINKAGE CArtifactOperationPack : CPackForClient
 struct DLL_LINKAGE PutArtifact : CArtifactOperationPack
 {
 	PutArtifact() = default;
-	explicit PutArtifact(ArtifactLocation & dst, bool askAssemble = true)
-		: al(dst), askAssemble(askAssemble)
+	explicit PutArtifact(const ArtifactInstanceID & id, const ArtifactLocation & dst, bool askAssemble = true)
+		: al(dst), askAssemble(askAssemble), id(id)
 	{
 	}
 
 	ArtifactLocation al;
 	bool askAssemble;
-	ConstTransitivePtr<CArtifactInstance> art;
+	ArtifactInstanceID id;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
 		h & al;
 		h & askAssemble;
-		h & art;
+		h & id;
 	}
 };
 
 struct DLL_LINKAGE NewArtifact : public CArtifactOperationPack
 {
-	ConstTransitivePtr<CArtifactInstance> art;
+	ObjectInstanceID artHolder;
+	ArtifactID artId;
+	SpellID spellId;
+	ArtifactPosition pos;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
-		h & art;
+		h & artHolder;
+		h & artId;
+		h & spellId;
+		h & pos;
 	}
 };
 
-struct DLL_LINKAGE EraseArtifact : CArtifactOperationPack
+struct DLL_LINKAGE BulkEraseArtifacts : CArtifactOperationPack
 {
-	ArtifactLocation al;
+	ObjectInstanceID artHolder;
+	std::vector<ArtifactPosition> posPack;
+	std::optional<SlotID> creature;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
-		h & al;
-	}
-};
-
-struct DLL_LINKAGE MoveArtifact : CArtifactOperationPack
-{
-	MoveArtifact() = default;
-	MoveArtifact(const PlayerColor & interfaceOwner, const ArtifactLocation & src, const ArtifactLocation & dst, bool askAssemble = true)
-		: interfaceOwner(interfaceOwner), src(src), dst(dst), askAssemble(askAssemble)
-	{
-	}
-	PlayerColor interfaceOwner;
-	ArtifactLocation src;
-	ArtifactLocation dst;
-	bool askAssemble = true;
-
-	void applyGs(CGameState * gs);
-	void visitTyped(ICPackVisitor & visitor) override;
-
-	template <typename Handler> void serialize(Handler & h)
-	{
-		h & interfaceOwner;
-		h & src;
-		h & dst;
-		h & askAssemble;
+		h & artHolder;
+		h & posPack;
+		h & creature;
 	}
 };
 
@@ -1055,17 +1047,20 @@ struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 	{
 		ArtifactPosition srcPos;
 		ArtifactPosition dstPos;
+		bool askAssemble;
 
 		LinkedSlots() = default;
-		LinkedSlots(const ArtifactPosition & srcPos, const ArtifactPosition & dstPos)
+		LinkedSlots(const ArtifactPosition & srcPos, const ArtifactPosition & dstPos, bool askAssemble = false)
 			: srcPos(srcPos)
 			, dstPos(dstPos)
+			, askAssemble(askAssemble)
 		{
 		}
 		template <typename Handler> void serialize(Handler & h)
 		{
 			h & srcPos;
 			h & dstPos;
+			h & askAssemble;
 		}
 	};
 
@@ -1079,8 +1074,6 @@ struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 		: interfaceOwner(PlayerColor::NEUTRAL)
 		, srcArtHolder(ObjectInstanceID::NONE)
 		, dstArtHolder(ObjectInstanceID::NONE)
-		, swap(false)
-		, askAssemble(false)
 		, srcCreature(std::nullopt)
 		, dstCreature(std::nullopt)
 	{
@@ -1089,19 +1082,15 @@ struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 		: interfaceOwner(interfaceOwner)
 		, srcArtHolder(srcArtHolder)
 		, dstArtHolder(dstArtHolder)
-		, swap(swap)
-		, askAssemble(false)
 		, srcCreature(std::nullopt)
 		, dstCreature(std::nullopt)
 	{
 	}
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	std::vector<LinkedSlots> artsPack0;
 	std::vector<LinkedSlots> artsPack1;
-	bool swap;
-	bool askAssemble;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -1114,24 +1103,22 @@ struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 		h & dstArtHolder;
 		h & srcCreature;
 		h & dstCreature;
-		h & swap;
-		h & askAssemble;
 	}
 };
 
 struct DLL_LINKAGE AssembledArtifact : CArtifactOperationPack
 {
-	ArtifactLocation al; //where assembly will be put
-	const CArtifact * builtArt;
+	ArtifactLocation al;
+	ArtifactID artId;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
 		h & al;
-		h & builtArt;
+		h & artId;
 	}
 };
 
@@ -1139,7 +1126,7 @@ struct DLL_LINKAGE DisassembledArtifact : CArtifactOperationPack
 {
 	ArtifactLocation al;
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -1157,7 +1144,7 @@ struct DLL_LINKAGE HeroVisit : public CPackForClient
 
 	bool starting; //false -> ending
 
-	void applyGs(CGameState * gs);
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -1170,48 +1157,6 @@ struct DLL_LINKAGE HeroVisit : public CPackForClient
 	}
 };
 
-struct DLL_LINKAGE NewTurn : public CPackForClient
-{
-	enum weekType { NORMAL, DOUBLE_GROWTH, BONUS_GROWTH, DEITYOFFIRE, PLAGUE, NO_ACTION };
-
-	void applyGs(CGameState * gs);
-
-	void visitTyped(ICPackVisitor & visitor) override;
-
-	struct Hero
-	{
-		ObjectInstanceID id; //id is a general serial id
-		ui32 move;
-		ui32 mana;
-		template <typename Handler> void serialize(Handler & h)
-		{
-			h & id;
-			h & move;
-			h & mana;
-		}
-		bool operator<(const Hero & h)const { return id < h.id; }
-	};
-
-	std::set<Hero> heroes; //updates movement and mana points
-	std::map<PlayerColor, ResourceSet> res; //player ID => resource value[res_id]
-	std::map<ObjectInstanceID, SetAvailableCreatures> cres;//creatures to be placed in towns
-	ui32 day = 0;
-	ui8 specialWeek = 0; //weekType
-	CreatureID creatureid; //for creature weeks
-
-	NewTurn() = default;
-
-	template <typename Handler> void serialize(Handler & h)
-	{
-		h & heroes;
-		h & cres;
-		h & res;
-		h & day;
-		h & specialWeek;
-		h & creatureid;
-	}
-};
-
 struct DLL_LINKAGE InfoWindow : public CPackForClient //103  - displays simple info window
 {
 	EInfoWindowMode type = EInfoWindowMode::MODAL;
@@ -1221,6 +1166,7 @@ struct DLL_LINKAGE InfoWindow : public CPackForClient //103  - displays simple i
 	ui16 soundID = 0;
 
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1233,9 +1179,42 @@ struct DLL_LINKAGE InfoWindow : public CPackForClient //103  - displays simple i
 	InfoWindow() = default;
 };
 
+struct DLL_LINKAGE NewTurn : public CPackForClient
+{
+	void applyGs(CGameState * gs) override;
+
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	ui32 day = 0;
+	CreatureID creatureid; //for creature weeks
+	EWeekType specialWeek = EWeekType::NORMAL;
+
+	std::vector<SetMovePoints> heroesMovement;
+	std::vector<SetMana> heroesMana;
+	std::vector<SetAvailableCreatures> availableCreatures;
+	std::map<PlayerColor, ResourceSet> playerIncome;
+	std::optional<RumorState> newRumor; // only on new weeks
+	std::optional<InfoWindow> newWeekNotification; // only on new week
+
+	NewTurn() = default;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & day;
+		h & creatureid;
+		h & specialWeek;
+		h & heroesMovement;
+		h & heroesMana;
+		h & availableCreatures;
+		h & playerIncome;
+		h & newRumor;
+		h & newWeekNotification;
+	}
+};
+
 struct DLL_LINKAGE SetObjectProperty : public CPackForClient
 {
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 	ObjectInstanceID id;
 	ObjProperty what{};
 
@@ -1257,17 +1236,17 @@ struct DLL_LINKAGE ChangeObjectVisitors : public CPackForClient
 {
 	enum VisitMode
 	{
-		VISITOR_ADD,      // mark hero as one that have visited this object
-		VISITOR_ADD_TEAM, // mark team as one that have visited this object
-		VISITOR_GLOBAL,   // mark player as one that have visited object of this type
-		VISITOR_REMOVE,   // unmark visitor, reversed to ADD
-		VISITOR_CLEAR     // clear all visitors from this object (object reset)
+		VISITOR_ADD_HERO,   // mark hero as one that have visited this object
+		VISITOR_ADD_PLAYER, // mark player as one that have visited this object instance
+		VISITOR_GLOBAL,     // mark player as one that have visited object of this type
+		VISITOR_SCOUTED,    // marks targeted team as having scouted this object
+		VISITOR_CLEAR,      // clear all visitors from this object (object reset)
 	};
 	VisitMode mode = VISITOR_CLEAR; // uses VisitMode enum
 	ObjectInstanceID object;
 	ObjectInstanceID hero; // note: hero owner will be also marked as "visited" this object
 
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -1294,7 +1273,7 @@ struct DLL_LINKAGE ChangeArtifactsCostume : public CPackForClient
 	uint32_t costumeIdx = 0;
 	const PlayerColor player = PlayerColor::NEUTRAL;
 
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	ChangeArtifactsCostume() = default;
@@ -1320,7 +1299,7 @@ struct DLL_LINKAGE HeroLevelUp : public Query
 	PrimarySkill primskill = PrimarySkill::ATTACK;
 	std::vector<SecondarySkill> skills;
 
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -1341,7 +1320,7 @@ struct DLL_LINKAGE CommanderLevelUp : public Query
 
 	std::vector<ui32> skills; //0-5 - secondary skills, val-100 - special skill
 
-	void applyGs(CGameState * gs) const;
+	void applyGs(CGameState * gs) override;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -1388,6 +1367,7 @@ struct DLL_LINKAGE BlockingDialog : public Query
 	BlockingDialog() = default;
 
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1407,6 +1387,7 @@ struct DLL_LINKAGE GarrisonDialog : public Query
 	bool removableUnits = false;
 
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1425,6 +1406,7 @@ struct DLL_LINKAGE ExchangeDialog : public Query
 	ObjectInstanceID hero2;
 
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1450,6 +1432,7 @@ struct DLL_LINKAGE TeleportDialog : public Query
 	bool impassable = false;
 
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1470,6 +1453,7 @@ struct DLL_LINKAGE MapObjectSelectDialog : public Query
 	std::vector<ObjectInstanceID> objects;
 
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1494,6 +1478,7 @@ struct DLL_LINKAGE AdvmapSpellCast : public CPackForClient
 
 protected:
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState * gs) override {}
 };
 
 struct DLL_LINKAGE ShowWorldViewEx : public CPackForClient
@@ -1503,6 +1488,8 @@ struct DLL_LINKAGE ShowWorldViewEx : public CPackForClient
 
 	std::vector<ObjectPosInfo> objectPositions;
 
+	void applyGs(CGameState * gs) override {}
+
 	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
@@ -1523,6 +1510,7 @@ struct DLL_LINKAGE PlayerMessageClient : public CPackForClient
 	{
 	}
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState * gs) override {}
 
 	PlayerColor player;
 	std::string text;
@@ -1541,6 +1529,7 @@ struct DLL_LINKAGE CenterView : public CPackForClient
 	ui32 focusTime = 0; //ms
 
 	void visitTyped(ICPackVisitor & visitor) override;
+	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
