@@ -11,9 +11,9 @@
 #pragma once
 
 #include "Limiter.h"
-#include "MetaString.h"
 #include "Reward.h"
 #include "../networkPacks/EInfoWindowMode.h"
+#include "../texts/MetaString.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -35,7 +35,7 @@ enum ESelectMode
 {
 	SELECT_FIRST,  // first reward that matches limiters
 	SELECT_PLAYER, // player can select from all allowed rewards
-	SELECT_RANDOM, // one random reward from all mathing limiters
+	SELECT_RANDOM, // one random reward from all matching limiters
 	SELECT_ALL // grant all rewards that match limiters
 };
 
@@ -44,7 +44,8 @@ enum class EEventType
 	EVENT_INVALID = 0,
 	EVENT_FIRST_VISIT,
 	EVENT_ALREADY_VISITED,
-	EVENT_NOT_AVAILABLE
+	EVENT_NOT_AVAILABLE,
+	EVENT_GUARDED
 };
 
 constexpr std::array<std::string_view, 4> SelectModeString{"selectFirst", "selectPlayer", "selectRandom", "selectAll"};
@@ -143,7 +144,7 @@ struct DLL_LINKAGE Configuration
 	/// how reward will be selected, uses ESelectMode enum
 	ui8 selectMode = Rewardable::SELECT_FIRST;
 
-	/// contols who can visit an object, uses EVisitMode enum
+	/// controls who can visit an object, uses EVisitMode enum
 	ui8 visitMode = Rewardable::VISIT_UNLIMITED;
 
 	/// how and when should the object be reset
@@ -155,12 +156,16 @@ struct DLL_LINKAGE Configuration
 	/// Limiter that will be used to determine that object is visited. Only if visit mode is set to "limiter"
 	Rewardable::Limiter visitLimiter;
 
+	std::string guardsLayout;
+
 	/// if true - player can refuse visiting an object (e.g. Tomb)
 	bool canRefuse = false;
 
 	/// if true - right-clicking object will show preview of object rewards
 	bool showScoutedPreview = false;
 
+	bool coastVisitable = false;
+
 	/// if true - object info will shown in infobox (like resource pickup)
 	EInfoWindowMode infoWindowType = EInfoWindowMode::AUTO;
 	
@@ -189,6 +194,13 @@ struct DLL_LINKAGE Configuration
 		h & canRefuse;
 		h & showScoutedPreview;
 		h & infoWindowType;
+		if (h.version >= Handler::Version::REWARDABLE_BANKS)
+		{
+			h & coastVisitable;
+			h & guardsLayout;
+		}
+		else
+			coastVisitable = false;
 	}
 };
 
