@@ -10,15 +10,12 @@
 
 #pragma once
 
-#include "../GameConstants.h"
-#include "../CRandomGenerator.h"
 #include "CMapGenOptions.h"
-#include "../int3.h"
-#include "CRmgTemplate.h"
 #include "../LoadProgress.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
+class MetaString;
 class CRmgTemplate;
 class CMapGenOptions;
 class JsonNode;
@@ -26,6 +23,7 @@ class RmgMap;
 class CMap;
 class Zone;
 class CZonePlacer;
+class IGameCallback;
 
 using JsonVector = std::vector<JsonNode>;
 
@@ -42,15 +40,21 @@ public:
 		std::string defaultRoadType;
 		std::string secondaryRoadType;
 		int treasureValueLimit;
-		std::vector<int> prisonExperience, prisonValues;
+		std::vector<int> prisonExperience;
+		std::vector<int> prisonValues;
 		std::vector<int> scrollValues;
-		int pandoraMultiplierGold, pandoraMultiplierExperience, pandoraMultiplierSpells, pandoraSpellSchool, pandoraSpell60;
+		int pandoraMultiplierGold;
+		int pandoraMultiplierExperience;
+		int pandoraMultiplierSpells;
+		int pandoraSpellSchool;
+		int pandoraSpell60;
 		std::vector<int> pandoraCreatureValues;
-		std::vector<int> questValues, questRewardValues;
+		std::vector<int> questValues;
+		std::vector<int> questRewardValues;
 		bool singleThread;
 	};
 	
-	explicit CMapGenerator(CMapGenOptions& mapGenOptions, int RandomSeed = std::time(nullptr));
+	explicit CMapGenerator(CMapGenOptions& mapGenOptions, IGameCallback * cb, int RandomSeed);
 	~CMapGenerator(); // required due to std::unique_ptr
 	
 	const Config & getConfig() const;
@@ -60,20 +64,19 @@ public:
 	std::unique_ptr<CMap> generate();
 
 	int getNextMonlithIndex();
-	int getPrisonsRemaning() const;
+	int getPrisonsRemaining() const;
 	std::shared_ptr<CZonePlacer> getZonePlacer() const;
 	const std::vector<ArtifactID> & getAllPossibleQuestArtifacts() const;
 	const std::vector<HeroTypeID> getAllPossibleHeroes() const;
 	void banQuestArt(const ArtifactID & id);
-	void banHero(const HeroTypeID& id);
-
+	void unbanQuestArt(const ArtifactID & id);
 	Zone * getZoneWater() const;
 	void addWaterTreasuresInfo();
 
 	int getRandomSeed() const;
 	
 private:
-	CRandomGenerator rand;
+	std::unique_ptr<vstd::RNG> rand;
 	int randomSeed;
 	CMapGenOptions& mapGenOptions;
 	Config config;
@@ -82,14 +85,13 @@ private:
 	
 	std::vector<rmg::ZoneConnection> connectionsLeft;
 	
-	int allowedPrisons;
 	int monolithIndex;
 	std::vector<ArtifactID> questArtifacts;
 
 	/// Generation methods
 	void loadConfig();
 	
-	std::string getMapDescription() const;
+	MetaString getMapDescription() const;
 
 	void initPrisonsRemaining();
 	void initQuestArtsRemaining();
