@@ -17,7 +17,9 @@
 #include "../../VCMI_Lib.h"
 #include "../../mapObjectConstructors/AObjectTypeHandler.h"
 #include "../../mapObjectConstructors/CObjectClassesHandler.h"
-#include "../../mapObjects/MapObjects.h" 
+#include "../../mapObjects/MapObjects.h"
+
+#include <vstd/RNG.h>
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -64,6 +66,22 @@ std::vector<CGObjectInstance*> QuestArtifactPlacer::getPossibleArtifactsToReplac
 	return artifactsToReplace;
 }
 
+CGObjectInstance * QuestArtifactPlacer::drawObjectToReplace()
+{
+	RecursiveLock lock(externalAccessMutex);
+
+	if (artifactsToReplace.empty())
+	{
+		return nullptr;
+	}
+	else
+	{
+		auto ret = *RandomGeneratorUtil::nextItem(artifactsToReplace, zone.getRand());
+		vstd::erase_if_present(artifactsToReplace, ret);
+		return ret;
+	}
+}
+
 void QuestArtifactPlacer::findZonesForQuestArts()
 {
 	const auto& distances = generator.getZonePlacer()->getDistanceMap().at(zone.getId());
@@ -79,7 +97,7 @@ void QuestArtifactPlacer::findZonesForQuestArts()
 	logGlobal->trace("Number of nearby zones suitable for quest artifacts: %d", questArtZones.size());
 }
 
-void QuestArtifactPlacer::placeQuestArtifacts(CRandomGenerator & rand)
+void QuestArtifactPlacer::placeQuestArtifacts(vstd::RNG & rand)
 {
 	for (const auto & artifactToPlace : questArtifactsToPlace)
 	{
@@ -87,14 +105,14 @@ void QuestArtifactPlacer::placeQuestArtifacts(CRandomGenerator & rand)
 		for (auto zone : questArtZones)
 		{
 			auto* qap = zone->getModificator<QuestArtifactPlacer>();
-			std::vector<CGObjectInstance *> artifactsToReplace = qap->getPossibleArtifactsToReplace();
-			if (artifactsToReplace.empty())
+			
+			auto objectToReplace = qap->drawObjectToReplace();
+			if (!objectToReplace)
 				continue;
 
-			auto artifactToReplace = *RandomGeneratorUtil::nextItem(artifactsToReplace, rand);
 			logGlobal->trace("Replacing %s at %s with the quest artifact %s",
-				artifactToReplace->getObjectName(),
-				artifactToReplace->getPosition().toString(),
+				objectToReplace->getObjectName(),
+				objectToReplace->anchorPos().toString(),
 				VLC->artifacts()->getById(artifactToPlace)->getNameTranslated());
 
 			//Update appearance. Terrain is irrelevant.
@@ -103,24 +121,15 @@ void QuestArtifactPlacer::placeQuestArtifacts(CRandomGenerator & rand)
 			auto templates = handler->getTemplates();
 			//artifactToReplace->appearance = templates.front();
 			newObj->appearance  = templates.front();
-			newObj->pos = artifactToReplace->pos;
+			newObj->setAnchorPos(objectToReplace->anchorPos());
 			mapProxy->insertObject(newObj);
-
-			for (auto z : map.getZones())
-			{
-				//Every qap has its OWN collection of artifacts
-				auto * localQap = zone->getModificator<QuestArtifactPlacer>();
-				if (localQap)
-				{
-					localQap->dropReplacedArtifact(artifactToReplace);
-				}
-			}
-			mapProxy->removeObject(artifactToReplace);
+			mapProxy->removeObject(objectToReplace);
 			break;
 		}
 	}
 }
 
+// TODO: Unused?
 void QuestArtifactPlacer::dropReplacedArtifact(CGObjectInstance* obj)
 {
 	RecursiveLock lock(externalAccessMutex);
