@@ -28,9 +28,9 @@ public:
 	void insertObjects(std::set<CGObjectInstance*>& objects);
 	void removeObject(CGObjectInstance* obj);
 
-	void drawTerrain(CRandomGenerator & generator, std::vector<int3> & tiles, TerrainId terrain);
-	void drawRivers(CRandomGenerator & generator, std::vector<int3> & tiles, TerrainId terrain);
-	void drawRoads(CRandomGenerator & generator, std::vector<int3> & tiles, RoadId roadType);
+	void drawTerrain(vstd::RNG & generator, std::vector<int3> & tiles, TerrainId terrain);
+	void drawRivers(vstd::RNG & generator, std::vector<int3> & tiles, TerrainId terrain);
+	void drawRoads(vstd::RNG & generator, std::vector<int3> & tiles, RoadId roadType);
 
 private:
 	mutable boost::shared_mutex mx;
