@@ -12,27 +12,27 @@
 
 VCMI_LIB_NAMESPACE_BEGIN
 
-int CMemorySerializer::read(void * data, unsigned size)
+int CMemorySerializer::read(std::byte * data, unsigned size)
 {
 	if(buffer.size() < readPos + size)
 		throw std::runtime_error(boost::str(boost::format("Cannot read past the buffer (accessing index %d, while size is %d)!") % (readPos + size - 1) % buffer.size()));
 
-	std::memcpy(data, buffer.data() + readPos, size);
+	std::copy_n(buffer.data() + readPos, size, data);
 	readPos += size;
 	return size;
 }
 
-int CMemorySerializer::write(const void * data, unsigned size)
+int CMemorySerializer::write(const std::byte * data, unsigned size)
 {
 	auto oldSize = buffer.size(); //and the pos to write from
 	buffer.resize(oldSize + size);
-	std::memcpy(buffer.data() + oldSize, data, size);
+	std::copy_n(data, size, buffer.data() + oldSize);
 	return size;
 }
 
 CMemorySerializer::CMemorySerializer(): iser(this), oser(this), readPos(0)
 {
-	iser.fileVersion = SERIALIZATION_VERSION;
+	iser.version = ESerializationVersion::CURRENT;
 }
 
 VCMI_LIB_NAMESPACE_END
