@@ -18,64 +18,79 @@ class CGTownInstance;
 
 enum class ESpellCastResult
 {
-	OK,
-	CANCEL,//cast failed but it is not an error
+	OK, // cast successful
+	CANCEL, // cast failed but it is not an error, no mana has been spent
 	PENDING,
-	ERROR//internal error occurred
+	ERROR// error occurred, for example invalid request from player
 };
 
-class DLL_LINKAGE AdventureSpellMechanics : public IAdventureSpellMechanics
+class AdventureSpellMechanics : public IAdventureSpellMechanics
 {
 public:
 	AdventureSpellMechanics(const CSpell * s);
 
+	bool canBeCast(spells::Problem & problem, const CGameInfoCallback * cb, const spells::Caster * caster) const final;
+	bool canBeCastAt(spells::Problem & problem, const CGameInfoCallback * cb, const spells::Caster * caster, const int3 & pos) const final;
+
 	bool adventureCast(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const override final;
 protected:
 	///actual adventure cast implementation
 	virtual ESpellCastResult applyAdventureEffects(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const;
 	virtual ESpellCastResult beginCast(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const;
+	virtual void endCast(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const;
+	virtual bool canBeCastImpl(spells::Problem & problem, const CGameInfoCallback * cb, const spells::Caster * caster) const;
+	virtual bool canBeCastAtImpl(spells::Problem & problem, const CGameInfoCallback * cb, const spells::Caster * caster, const int3 & pos) const;
+
 	void performCast(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const;
-	void endCast(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters, const ESpellCastResult result) const;
 };
 
-class DLL_LINKAGE SummonBoatMechanics : public AdventureSpellMechanics
+class SummonBoatMechanics final : public AdventureSpellMechanics
 {
 public:
 	SummonBoatMechanics(const CSpell * s);
 protected:
+	bool canBeCastImpl(spells::Problem & problem, const CGameInfoCallback * cb, const spells::Caster * caster) const override;
+
 	ESpellCastResult applyAdventureEffects(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const override;
 };
 
-class DLL_LINKAGE ScuttleBoatMechanics : public AdventureSpellMechanics
+class ScuttleBoatMechanics final : public AdventureSpellMechanics
 {
 public:
 	ScuttleBoatMechanics(const CSpell * s);
 protected:
+	bool canBeCastAtImpl(spells::Problem & problem, const CGameInfoCallback * cb, const spells::Caster * caster, const int3 & pos) const override;
+
 	ESpellCastResult applyAdventureEffects(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const override;
 };
 
-class DLL_LINKAGE DimensionDoorMechanics : public AdventureSpellMechanics
+class DimensionDoorMechanics final : public AdventureSpellMechanics
 {
 public:
 	DimensionDoorMechanics(const CSpell * s);
 protected:
+	bool canBeCastImpl(spells::Problem & problem, const CGameInfoCallback * cb, const spells::Caster * caster) const override;
+	bool canBeCastAtImpl(spells::Problem & problem, const CGameInfoCallback * cb, const spells::Caster * caster, const int3 & pos) const override;
+
 	ESpellCastResult applyAdventureEffects(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const override;
+	void endCast(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const override;
 };
 
-class DLL_LINKAGE TownPortalMechanics : public AdventureSpellMechanics
+class TownPortalMechanics final : public AdventureSpellMechanics
 {
 public:
 	TownPortalMechanics(const CSpell * s);
 protected:
 	ESpellCastResult applyAdventureEffects(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const override;
 	ESpellCastResult beginCast(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const override;
+	void endCast(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const override;
 private:
 	const CGTownInstance * findNearestTown(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters, const std::vector <const CGTownInstance*> & pool) const;
 	int32_t movementCost(const AdventureSpellCastParameters & parameters) const;
 	std::vector <const CGTownInstance*> getPossibleTowns(SpellCastEnvironment * env, const AdventureSpellCastParameters & parameters) const;
 };
 
-class DLL_LINKAGE ViewMechanics : public AdventureSpellMechanics
+class ViewMechanics : public AdventureSpellMechanics
 {
 public:
 	ViewMechanics(const CSpell * s);
@@ -85,7 +100,7 @@ protected:
 	virtual bool showTerrain(const int32_t spellLevel) const = 0;
 };
 
-class DLL_LINKAGE ViewAirMechanics : public ViewMechanics
+class ViewAirMechanics final : public ViewMechanics
 {
 public:
 	ViewAirMechanics(const CSpell * s);
@@ -94,7 +109,7 @@ protected:
 	bool showTerrain(const int32_t spellLevel) const override;
 };
 
-class DLL_LINKAGE ViewEarthMechanics : public ViewMechanics
+class ViewEarthMechanics final : public ViewMechanics
 {
 public:
 	ViewEarthMechanics(const CSpell * s);
