@@ -11,7 +11,6 @@
 #include "StdInc.h"
 #include "ISpellMechanics.h"
 
-#include "../CRandomGenerator.h"
 #include "../VCMI_Lib.h"
 
 #include "../bonuses/Bonus.h"
@@ -37,10 +36,11 @@
 
 #include "CSpellHandler.h"
 
-#include "../CHeroHandler.h"//todo: remove
 #include "../IGameCallback.h"//todo: remove
 #include "../BattleFieldHandler.h"
 
+#include <vstd/RNG.h>
+
 VCMI_LIB_NAMESPACE_BEGIN
 
 namespace spells
@@ -48,7 +48,7 @@ namespace spells
 
 static std::shared_ptr<TargetCondition> makeCondition(const CSpell * s)
 {
-	std::shared_ptr<TargetCondition> res = std::make_shared<TargetCondition>();
+	auto res = std::make_shared<TargetCondition>();
 
 	JsonDeserializer deser(nullptr, s->targetCondition);
 	res->serializeJson(deser, TargetConditionItemFactory::getDefault());
@@ -139,7 +139,6 @@ public:
 BattleCast::BattleCast(const CBattleInfoCallback * cb_, const Caster * caster_, const Mode mode_, const CSpell * spell_):
 	spell(spell_),
 	cb(cb_),
-	gameCb(IObjectInterface::cb), //FIXME: pass player callback (problem is that BattleAI do not have one)
 	caster(caster_),
 	mode(mode_),
 	smart(boost::logic::indeterminate),
@@ -150,7 +149,6 @@ BattleCast::BattleCast(const CBattleInfoCallback * cb_, const Caster * caster_,
 BattleCast::BattleCast(const BattleCast & orig, const Caster * caster_)
 	: spell(orig.spell),
 	cb(orig.cb),
-	gameCb(orig.gameCb),
 	caster(caster_),
 	mode(Mode::MAGIC_MIRROR),
 	magicSkillLevel(orig.magicSkillLevel),
@@ -184,11 +182,6 @@ const CBattleInfoCallback * BattleCast::getBattle() const
 	return cb;
 }
 
-const IGameInfoCallback * BattleCast::getGame() const
-{
-	return gameCb;
-}
-
 BattleCast::OptionalValue BattleCast::getSpellLevel() const
 {
 	return magicSkillLevel;
@@ -275,11 +268,9 @@ void BattleCast::cast(ServerCallback * server, Target target)
 		const std::string magicMirrorCacheStr = "type_MAGIC_MIRROR";
 		static const auto magicMirrorSelector = Selector::type()(BonusType::MAGIC_MIRROR);
 
-		auto rangeGen = server->getRNG()->getInt64Range(0, 99);
-
 		const int mirrorChance = mainTarget->valOfBonuses(magicMirrorSelector, magicMirrorCacheStr);
 
-		if(rangeGen() < mirrorChance)
+		if(server->getRNG()->nextInt(0, 99) < mirrorChance)
 		{
 			auto mirrorTargets = cb->battleGetUnitsIf([this](const battle::Unit * unit)
 			{
@@ -405,7 +396,7 @@ std::unique_ptr<ISpellMechanicsFactory> ISpellMechanicsFactory::get(const CSpell
 ///Mechanics
 Mechanics::Mechanics()
 	: caster(nullptr),
-	casterSide(0)
+	casterSide(BattleSide::NONE)
 {
 
 }
@@ -417,13 +408,11 @@ BaseMechanics::BaseMechanics(const IBattleCast * event):
 	mode(event->getMode()),
 	smart(event->isSmart()),
 	massive(event->isMassive()),
-	cb(event->getBattle()),
-	gameCb(event->getGame())
+	cb(event->getBattle())
 {
 	caster = event->getCaster();
 
-	//FIXME: do not crash on invalid side
-	casterSide = cb->playerToSide(caster->getCasterOwner()).value();
+	casterSide = cb->playerToSide(caster->getCasterOwner());
 
 	{
 		auto value = event->getSpellLevel();
@@ -483,7 +472,7 @@ bool BaseMechanics::adaptProblem(ESpellCastProblem source, Problem & target) con
 				return adaptGenericProblem(target);
 
 			//Recanter's Cloak or similar effect. Try to retrieve bonus
-			const auto b = hero->getBonusLocalFirst(Selector::type()(BonusType::BLOCK_MAGIC_ABOVE));
+			const auto b = hero->getFirstBonus(Selector::type()(BonusType::BLOCK_MAGIC_ABOVE));
 			//TODO what about other values and non-artifact sources?
 			if(b && b->val == 2 && b->source == BonusSource::ARTIFACT)
 			{
@@ -696,11 +685,6 @@ const Service * BaseMechanics::spells() const
 	return VLC->spells(); //todo: redirect
 }
 
-const IGameInfoCallback * BaseMechanics::game() const
-{
-	return gameCb;
-}
-
 const CBattleInfoCallback * BaseMechanics::battle() const
 {
 	return cb;
