@@ -16,7 +16,6 @@
 
 #include "../ISpellMechanics.h"
 
-#include "../../MetaString.h"
 #include "../../battle/IBattleState.h"
 #include "../../battle/CBattleInfoCallback.h"
 #include "../../battle/Unit.h"
@@ -66,10 +65,10 @@ void Dispel::apply(ServerCallback * server, const Mechanics * m, const EffectTar
 	}
 
 	if(!sse.toRemove.empty())
-		server->apply(&sse);
+		server->apply(sse);
 
 	if(describe && !blm.lines.empty())
-		server->apply(&blm);
+		server->apply(blm);
 }
 
 bool Dispel::isValidTarget(const Mechanics * m, const battle::Unit * unit) const
@@ -97,7 +96,7 @@ std::shared_ptr<const BonusList> Dispel::getBonuses(const Mechanics * m, const b
 			if(!sourceSpell)
 				return false;//error
 
-			//Special case: DISRUPTING_RAY and ACID_BREATH_DEFENSE are "immune" to dispell
+			//Special case: DISRUPTING_RAY and ACID_BREATH_DEFENSE are "immune" to dispel
 			//Other even PERMANENT effects can be removed (f.e. BIND)
 			if(sourceSpell->getIndex() == SpellID::DISRUPTING_RAY || sourceSpell->getIndex() == SpellID::ACID_BREATH_DEFENSE)
 				return false;
