@@ -30,10 +30,9 @@ public:
 
 	EffectData data;
 
-	Effects();
-	virtual ~Effects();
+	virtual ~Effects() = default;
 
-	void add(const std::string & name, std::shared_ptr<Effect> effect, const int level);
+	void add(const std::string & name, const std::shared_ptr<Effect>& effect, const int level);
 
 	bool applicable(Problem & problem, const Mechanics * m) const;
 	bool applicable(Problem & problem, const Mechanics * m, const Target & aimPoint, const Target & spellTarget) const;
