@@ -12,13 +12,6 @@
 
 #include "Effect.h"
 
-#define VCMI_REGISTER_SPELL_EFFECT(Type, Name) \
-namespace\
-{\
-RegisterEffect<Type> register ## Type(Name);\
-}\
-\
-
 VCMI_LIB_NAMESPACE_BEGIN
 
 namespace spells
@@ -29,7 +22,6 @@ namespace effects
 class DLL_LINKAGE IEffectFactory
 {
 public:
-	IEffectFactory() = default;
 	virtual ~IEffectFactory() = default;
 
 	virtual Effect * create() const = 0;
@@ -40,15 +32,13 @@ class DLL_LINKAGE Registry
 public:
 	using FactoryPtr = std::shared_ptr<IEffectFactory>;
 
-	Registry();
-	virtual ~Registry();
+	virtual ~Registry() = default; //Required for child classes
 	virtual const IEffectFactory * find(const std::string & name) const = 0;
 	virtual void add(const std::string & name, FactoryPtr item) = 0;
 };
 
 class DLL_LINKAGE GlobalRegistry
 {
-	GlobalRegistry() = default;
 public:
     static Registry * get();
 };
@@ -57,26 +47,12 @@ template<typename E>
 class EffectFactory : public IEffectFactory
 {
 public:
-	EffectFactory() = default;
-	virtual ~EffectFactory() = default;
-
 	Effect * create() const override
 	{
 		return new E();
 	}
 };
 
-template<typename E>
-class RegisterEffect
-{
-public:
-	RegisterEffect(const std::string & name)
-	{
-		auto f = std::make_shared<EffectFactory<E>>();
-		GlobalRegistry::get()->add(name, f);
-	}
-};
-
 }
 }
 
