@@ -19,11 +19,11 @@ GeneratorProgress::GeneratorProgress(Load::Progress & source, QWidget *parent) :
 	source(source)
 {
 	ui->setupUi(this);
+	
+	setWindowFlags(Qt::Dialog | Qt::WindowTitleHint | Qt::CustomizeWindowHint);
 
 	setAttribute(Qt::WA_DeleteOnClose);
 
-	setWindowFlags(Qt::Window);
-
 	show();
 }
 
