@@ -11,34 +11,32 @@
 #include "inspector.h"
 #include "../lib/ArtifactUtils.h"
 #include "../lib/CArtHandler.h"
+#include "../lib/entities/hero/CHeroClass.h"
+#include "../lib/entities/hero/CHeroHandler.h"
 #include "../lib/spells/CSpellHandler.h"
-#include "../lib/CHeroHandler.h"
 #include "../lib/CRandomGenerator.h"
 #include "../lib/mapObjectConstructors/AObjectTypeHandler.h"
 #include "../lib/mapObjectConstructors/CObjectClassesHandler.h"
 #include "../lib/mapObjects/ObjectTemplate.h"
 #include "../lib/mapping/CMap.h"
 #include "../lib/constants/StringConstants.h"
+#include "../lib/CPlayerState.h"
+#include "../lib/texts/CGeneralTextHandler.h"
 
-#include "townbulidingswidget.h"
+#include "townbuildingswidget.h"
+#include "towneventswidget.h"
+#include "townspellswidget.h"
 #include "armywidget.h"
 #include "messagewidget.h"
 #include "rewardswidget.h"
 #include "questwidget.h"
+#include "heroartifactswidget.h"
 #include "heroskillswidget.h"
+#include "herospellwidget.h"
 #include "portraitwidget.h"
 #include "PickObjectDelegate.h"
 #include "../mapcontroller.h"
 
-static QList<std::pair<QString, QVariant>> CharacterIdentifiers
-{
-	{QObject::tr("Compliant"), QVariant::fromValue(int(CGCreature::Character::COMPLIANT))},
-	{QObject::tr("Friendly"), QVariant::fromValue(int(CGCreature::Character::FRIENDLY))},
-	{QObject::tr("Aggressive"), QVariant::fromValue(int(CGCreature::Character::AGGRESSIVE))},
-	{QObject::tr("Hostile"), QVariant::fromValue(int(CGCreature::Character::HOSTILE))},
-	{QObject::tr("Savage"), QVariant::fromValue(int(CGCreature::Character::SAVAGE))},
-};
-
 //===============IMPLEMENT OBJECT INITIALIZATION FUNCTIONS================
 Initializer::Initializer(CGObjectInstance * o, const PlayerColor & pl) : defaultPlayer(pl)
 {
@@ -56,7 +54,7 @@ Initializer::Initializer(CGObjectInstance * o, const PlayerColor & pl) : default
 	INIT_OBJ_TYPE(CGHeroPlaceholder);
 	INIT_OBJ_TYPE(CGHeroInstance);
 	INIT_OBJ_TYPE(CGSignBottle);
-	INIT_OBJ_TYPE(CGLighthouse);
+	INIT_OBJ_TYPE(FlaggableMapObject);
 	//INIT_OBJ_TYPE(CRewardableObject);
 	//INIT_OBJ_TYPE(CGPandoraBox);
 	//INIT_OBJ_TYPE(CGEvent);
@@ -87,6 +85,20 @@ void Initializer::initialize(CGDwelling * o)
 	if(!o) return;
 	
 	o->tempOwner = defaultPlayer;
+
+	if(o->ID == Obj::RANDOM_DWELLING || o->ID == Obj::RANDOM_DWELLING_LVL || o->ID == Obj::RANDOM_DWELLING_FACTION)
+	{
+		o->randomizationInfo = CGDwellingRandomizationInfo();
+		if(o->ID == Obj::RANDOM_DWELLING_LVL)
+		{
+			o->randomizationInfo->minLevel = o->subID;
+			o->randomizationInfo->maxLevel = o->subID;
+		}
+		if(o->ID == Obj::RANDOM_DWELLING_FACTION)
+		{
+			o->randomizationInfo->allowedFactions.insert(FactionID(o->subID));
+		}
+	}
 }
 
 void Initializer::initialize(CGGarrison * o)
@@ -104,7 +116,7 @@ void Initializer::initialize(CGShipyard * o)
 	o->tempOwner = defaultPlayer;
 }
 
-void Initializer::initialize(CGLighthouse * o)
+void Initializer::initialize(FlaggableMapObject * o)
 {
 	if(!o) return;
 	
@@ -135,25 +147,23 @@ void Initializer::initialize(CGHeroInstance * o)
 		o->subID = 0;
 		o->tempOwner = PlayerColor::NEUTRAL;
 	}
-	
+
 	if(o->ID == Obj::HERO)
 	{
-		for(auto t : VLC->heroh->objects)
+		for(auto const & t : VLC->heroh->objects)
 		{
 			if(t->heroClass->getId() == HeroClassID(o->subID))
 			{
-				o->type = t;
+				o->subID = t->getId();
 				break;
 			}
 		}
 	}
-	
-	if(o->type)
+
+	if(o->getHeroTypeID().hasValue())
 	{
-		//	o->type = VLC->heroh->objects.at(o->subID);
-		
-		o->gender = o->type->gender;
-		o->randomizeArmy(o->type->heroClass->faction);
+		o->gender = o->getHeroType()->gender;
+		o->randomizeArmy(o->getFactionID());
 	}
 }
 
@@ -163,17 +173,19 @@ void Initializer::initialize(CGTownInstance * o)
 
 	const std::vector<std::string> castleLevels{"village", "fort", "citadel", "castle", "capitol"};
 	int lvl = vstd::find_pos(castleLevels, o->appearance->stringID);
-	o->builtBuildings.insert(BuildingID::DEFAULT);
-	if(lvl > -1) o->builtBuildings.insert(BuildingID::TAVERN);
-	if(lvl > 0) o->builtBuildings.insert(BuildingID::FORT);
-	if(lvl > 1) o->builtBuildings.insert(BuildingID::CITADEL);
-	if(lvl > 2) o->builtBuildings.insert(BuildingID::CASTLE);
-	if(lvl > 3) o->builtBuildings.insert(BuildingID::CAPITOL);
+	o->addBuilding(BuildingID::DEFAULT);
+	if(lvl > -1) o->addBuilding(BuildingID::TAVERN);
+	if(lvl > 0) o->addBuilding(BuildingID::FORT);
+	if(lvl > 1) o->addBuilding(BuildingID::CITADEL);
+	if(lvl > 2) o->addBuilding(BuildingID::CASTLE);
+	if(lvl > 3) o->addBuilding(BuildingID::CAPITOL);
 
-	for(auto spell : VLC->spellh->objects) //add all regular spells to town
+	if(o->possibleSpells.empty())
 	{
-		if(!spell->isSpecial() && !spell->isCreatureAbility())
-			o->possibleSpells.push_back(spell->id);
+		for(auto const & spellId : VLC->spellh->getDefaultAllowed()) //add all regular spells to town
+		{
+			o->possibleSpells.push_back(spellId);
+		}
 	}
 }
 
@@ -184,7 +196,7 @@ void Initializer::initialize(CGArtifact * o)
 	if(o->ID == Obj::SPELL_SCROLL)
 	{
 		std::vector<SpellID> out;
-		for(auto spell : VLC->spellh->objects) //spellh size appears to be greater (?)
+		for(auto const & spell : VLC->spellh->objects) //spellh size appears to be greater (?)
 		{
 			if(VLC->spellh->getDefaultAllowed().count(spell->id) != 0)
 			{
@@ -213,62 +225,55 @@ void Initializer::initialize(CGMine * o)
 	}
 }
 
-void Initializer::initialize(CGResource * o)
-{
-	if(!o) return;
-	
-	o->amount = CGResource::RANDOM_AMOUNT;
-}
-
 //===============IMPLEMENT PROPERTIES SETUP===============================
 void Inspector::updateProperties(CArmedInstance * o)
 {
 	if(!o) return;
 	
 	auto * delegate = new ArmyDelegate(*o);
-	addProperty("Army", PropertyEditorPlaceholder(), delegate, false);
+	addProperty(QObject::tr("Army"), PropertyEditorPlaceholder(), delegate, false);
 }
 
 void Inspector::updateProperties(CGDwelling * o)
 {
 	if(!o) return;
 	
-	addProperty("Owner", o->tempOwner, false);
+	addProperty(QObject::tr("Owner"), o->tempOwner, new OwnerDelegate(controller), false);
 	
 	if (o->ID == Obj::RANDOM_DWELLING || o->ID == Obj::RANDOM_DWELLING_LVL)
 	{
 		auto * delegate = new PickObjectDelegate(controller, PickObjectDelegate::typedFilter<CGTownInstance>);
-		addProperty("Same as town", PropertyEditorPlaceholder(), delegate, false);
+		addProperty(QObject::tr("Same as town"), o->randomizationInfo, delegate, false);
 	}
 }
 
-void Inspector::updateProperties(CGLighthouse * o)
+void Inspector::updateProperties(FlaggableMapObject * o)
 {
 	if(!o) return;
-	
-	addProperty("Owner", o->tempOwner, false);
+
+	addProperty(QObject::tr("Owner"), o->tempOwner, new OwnerDelegate(controller), false);
 }
 
 void Inspector::updateProperties(CGGarrison * o)
 {
 	if(!o) return;
 	
-	addProperty("Owner", o->tempOwner, false);
-	addProperty("Removable units", o->removableUnits, false);
+	addProperty(QObject::tr("Owner"), o->tempOwner, new OwnerDelegate(controller), false);
+	addProperty(QObject::tr("Removable units"), o->removableUnits, false);
 }
 
 void Inspector::updateProperties(CGShipyard * o)
 {
 	if(!o) return;
 	
-	addProperty("Owner", o->tempOwner, false);
+	addProperty(QObject::tr("Owner"), o->tempOwner, new OwnerDelegate(controller), false);
 }
 
 void Inspector::updateProperties(CGHeroPlaceholder * o)
 {
 	if(!o) return;
 	
-	addProperty("Owner", o->tempOwner, false);
+	addProperty(QObject::tr("Owner"), o->tempOwner, new OwnerDelegate(controller), false);
 	
 	bool type = false;
 	if(o->heroType.has_value())
@@ -278,11 +283,11 @@ void Inspector::updateProperties(CGHeroPlaceholder * o)
 	
 	{
 		auto * delegate = new InspectorDelegate;
-		delegate->options = {{"POWER RANK", QVariant::fromValue(false)}, {"HERO TYPE", QVariant::fromValue(true)}};
-		addProperty("Placeholder type", delegate->options[type].first, delegate, false);
+		delegate->options = {{QObject::tr("POWER RANK"), QVariant::fromValue(false)}, {QObject::tr("HERO TYPE"), QVariant::fromValue(true)}};
+		addProperty(QObject::tr("Placeholder type"), delegate->options[type].first, delegate, false);
 	}
 	
-	addProperty("Power rank", o->powerRank.has_value() ? o->powerRank.value() : 0, type);
+	addProperty(QObject::tr("Power rank"), o->powerRank.has_value() ? o->powerRank.value() : 0, type);
 	
 	{
 		auto * delegate = new InspectorDelegate;
@@ -290,7 +295,7 @@ void Inspector::updateProperties(CGHeroPlaceholder * o)
 		{
 			delegate->options.push_back({QObject::tr(VLC->heroh->objects[i]->getNameTranslated().c_str()), QVariant::fromValue(VLC->heroh->objects[i]->getId().getNum())});
 		}
-		addProperty("Hero type", o->heroType.has_value() ? VLC->heroh->getById(o->heroType.value())->getNameTranslated() : "", delegate, !type);
+		addProperty(QObject::tr("Hero type"), o->heroType.has_value() ? VLC->heroh->getById(o->heroType.value())->getNameTranslated() : "", delegate, !type);
 	}
 }
 
@@ -298,36 +303,48 @@ void Inspector::updateProperties(CGHeroInstance * o)
 {
 	if(!o) return;
 	
-	addProperty("Owner", o->tempOwner, o->ID == Obj::PRISON); //field is not editable for prison
-	addProperty<int>("Experience", o->exp, false);
-	addProperty("Hero class", o->type ? o->type->heroClass->getNameTranslated() : "", true);
+	auto isPrison = o->ID == Obj::PRISON;
+	addProperty(QObject::tr("Owner"), o->tempOwner, new OwnerDelegate(controller, isPrison), isPrison); //field is not editable for prison
+	addProperty<int>(QObject::tr("Experience"), o->exp, false);
+	addProperty(QObject::tr("Hero class"), o->getHeroClassID().hasValue() ? o->getHeroClass()->getNameTranslated() : "", true);
 	
 	{ //Gender
 		auto * delegate = new InspectorDelegate;
-		delegate->options = {{"MALE", QVariant::fromValue(int(EHeroGender::MALE))}, {"FEMALE", QVariant::fromValue(int(EHeroGender::FEMALE))}};
-		addProperty<std::string>("Gender", (o->gender == EHeroGender::FEMALE ? "FEMALE" : "MALE"), delegate , false);
+		delegate->options = {{QObject::tr("MALE"), QVariant::fromValue(int(EHeroGender::MALE))}, {QObject::tr("FEMALE"), QVariant::fromValue(int(EHeroGender::FEMALE))}};
+		addProperty<std::string>(QObject::tr("Gender"), (o->gender == EHeroGender::FEMALE ? QObject::tr("FEMALE") : QObject::tr("MALE")).toStdString(), delegate , false);
 	}
-	addProperty("Name", o->getNameTranslated(), false);
-	addProperty("Biography", o->getBiographyTranslated(), new MessageDelegate, false);
-	addProperty("Portrait", PropertyEditorPlaceholder(), new PortraitDelegate(*o), false);
+	addProperty(QObject::tr("Name"), o->getNameTranslated(), false);
+	addProperty(QObject::tr("Biography"), o->getBiographyTranslated(), new MessageDelegate, false);
+	addProperty(QObject::tr("Portrait"), PropertyEditorPlaceholder(), new PortraitDelegate(*o), false);
 	
 	auto * delegate = new HeroSkillsDelegate(*o);
-	addProperty("Skills", PropertyEditorPlaceholder(), delegate, false);
+	addProperty(QObject::tr("Skills"), PropertyEditorPlaceholder(), delegate, false);
+	addProperty(QObject::tr("Spells"), PropertyEditorPlaceholder(), new HeroSpellDelegate(*o), false);
+	addProperty(QObject::tr("Artifacts"), PropertyEditorPlaceholder(), new HeroArtifactsDelegate(*o), false);
 	
-	if(o->type)
+	if(o->getHeroTypeID().hasValue() || o->ID == Obj::PRISON)
 	{ //Hero type
 		auto * delegate = new InspectorDelegate;
-		for(int i = 0; i < VLC->heroh->objects.size(); ++i)
+		for(const auto & heroPtr : VLC->heroh->objects)
 		{
-			if(controller.map()->allowedHeroes.count(HeroTypeID(i)) != 0)
+			if(controller.map()->allowedHeroes.count(heroPtr->getId()) != 0)
 			{
-				if(o->ID == Obj::PRISON || (o->type && VLC->heroh->objects[i]->heroClass->getIndex() == o->type->heroClass->getIndex()))
+				if(o->ID == Obj::PRISON || heroPtr->heroClass->getIndex() == o->getHeroClassID())
 				{
-					delegate->options.push_back({QObject::tr(VLC->heroh->objects[i]->getNameTranslated().c_str()), QVariant::fromValue(VLC->heroh->objects[i]->getId().getNum())});
+					delegate->options.push_back({QObject::tr(heroPtr->getNameTranslated().c_str()), QVariant::fromValue(heroPtr->getIndex())});
 				}
 			}
 		}
-		addProperty("Hero type", o->type->getNameTranslated(), delegate, false);
+		addProperty(QObject::tr("Hero type"), o->getHeroTypeID().hasValue() ? o->getHeroType()->getNameTranslated() : "", delegate, false);
+	}
+	{
+		const int maxRadius = 60;
+		auto * patrolDelegate = new InspectorDelegate;
+		patrolDelegate->options = { {QObject::tr("No patrol"), QVariant::fromValue(CGHeroInstance::NO_PATROLLING)} };
+		for(int i = 0; i <= maxRadius; ++i)
+			patrolDelegate->options.push_back({ QObject::tr("%n tile(s)", "", i), QVariant::fromValue(i)});
+		auto patrolRadiusText = o->patrol.patrolling ? QObject::tr("%n tile(s)", "", o->patrol.patrolRadius) : QObject::tr("No patrol");
+		addProperty(QObject::tr("Patrol radius"), patrolRadiusText, patrolDelegate, false);
 	}
 }
 
@@ -335,17 +352,21 @@ void Inspector::updateProperties(CGTownInstance * o)
 {
 	if(!o) return;
 	
-	addProperty("Town name", o->getNameTranslated(), false);
+	addProperty(QObject::tr("Town name"), o->getNameTranslated(), false);
 	
 	auto * delegate = new TownBuildingsDelegate(*o);
-	addProperty("Buildings", PropertyEditorPlaceholder(), delegate, false);
+	addProperty(QObject::tr("Buildings"), PropertyEditorPlaceholder(), delegate, false);
+	addProperty(QObject::tr("Spells"), PropertyEditorPlaceholder(), new TownSpellsDelegate(*o), false);
+	addProperty(QObject::tr("Events"), PropertyEditorPlaceholder(), new TownEventsDelegate(*o, controller), false);
+	if(o->ID == Obj::RANDOM_TOWN)
+		addProperty(QObject::tr("Same as player"), o->alignmentToPlayer, new OwnerDelegate(controller), false);
 }
 
 void Inspector::updateProperties(CGArtifact * o)
 {
 	if(!o) return;
 	
-	addProperty("Message", o->message, false);
+	addProperty(QObject::tr("Message"), o->message, false);
 	
 	CArtifactInstance * instance = o->storedArtifact;
 	if(instance)
@@ -354,12 +375,12 @@ void Inspector::updateProperties(CGArtifact * o)
 		if(spellId != SpellID::NONE)
 		{
 			auto * delegate = new InspectorDelegate;
-			for(auto spell : VLC->spellh->objects)
+			for(auto const & spell : VLC->spellh->objects)
 			{
 				if(controller.map()->allowedSpells.count(spell->id) != 0)
 					delegate->options.push_back({QObject::tr(spell->getNameTranslated().c_str()), QVariant::fromValue(int(spell->getId()))});
 			}
-			addProperty("Spell", VLC->spellh->getById(spellId)->getNameTranslated(), delegate, false);
+			addProperty(QObject::tr("Spell"), VLC->spellh->getById(spellId)->getNameTranslated(), delegate, false);
 		}
 	}
 }
@@ -368,42 +389,42 @@ void Inspector::updateProperties(CGMine * o)
 {
 	if(!o) return;
 	
-	addProperty("Owner", o->tempOwner, false);
-	addProperty("Resource", o->producedResource);
-	addProperty("Productivity", o->producedQuantity, false);
+	addProperty(QObject::tr("Owner"), o->tempOwner, new OwnerDelegate(controller), false);
+	addProperty(QObject::tr("Resource"), o->producedResource);
+	addProperty(QObject::tr("Productivity"), o->producedQuantity);
 }
 
 void Inspector::updateProperties(CGResource * o)
 {
 	if(!o) return;
 	
-	addProperty("Amount", o->amount, false);
-	addProperty("Message", o->message, false);
+	addProperty(QObject::tr("Amount"), o->amount, false);
+	addProperty(QObject::tr("Message"), o->message, false);
 }
 
 void Inspector::updateProperties(CGSignBottle * o)
 {
 	if(!o) return;
 	
-	addProperty("Message", o->message, new MessageDelegate, false);
+	addProperty(QObject::tr("Message"), o->message, new MessageDelegate, false);
 }
 
 void Inspector::updateProperties(CGCreature * o)
 {
 	if(!o) return;
 	
-	addProperty("Message", o->message, false);
+	addProperty(QObject::tr("Message"), o->message, false);
 	{ //Character
 		auto * delegate = new InspectorDelegate;
-		delegate->options = CharacterIdentifiers;
+		delegate->options = characterIdentifiers;
 		addProperty<CGCreature::Character>("Character", (CGCreature::Character)o->character, delegate, false);
 	}
-	addProperty("Never flees", o->neverFlees, false);
-	addProperty("Not growing", o->notGrowingTeam, false);
-	addProperty("Artifact reward", o->gainedArtifact); //TODO: implement in setProperty
-	addProperty("Army", PropertyEditorPlaceholder(), true);
-	addProperty("Amount", o->stacks[SlotID(0)]->count, false);
-	//addProperty("Resources reward", o->resources); //TODO: implement in setProperty
+	addProperty(QObject::tr("Never flees"), o->neverFlees, false);
+	addProperty(QObject::tr("Not growing"), o->notGrowingTeam, false);
+	addProperty(QObject::tr("Artifact reward"), o->gainedArtifact); //TODO: implement in setProperty
+	addProperty(QObject::tr("Army"), PropertyEditorPlaceholder(), true);
+	addProperty(QObject::tr("Amount"), o->stacks[SlotID(0)]->count, false);
+	//addProperty(QObject::tr("Resources reward"), o->resources); //TODO: implement in setProperty
 }
 
 void Inspector::updateProperties(CRewardableObject * o)
@@ -411,23 +432,23 @@ void Inspector::updateProperties(CRewardableObject * o)
 	if(!o) return;
 		
 	auto * delegate = new RewardsDelegate(*controller.map(), *o);
-	addProperty("Reward", PropertyEditorPlaceholder(), delegate, false);
+	addProperty(QObject::tr("Reward"), PropertyEditorPlaceholder(), delegate, false);
 }
 
 void Inspector::updateProperties(CGPandoraBox * o)
 {
 	if(!o) return;
 	
-	addProperty("Message", o->message, new MessageDelegate, false);
+	addProperty(QObject::tr("Message"), o->message, new MessageDelegate, false);
 }
 
 void Inspector::updateProperties(CGEvent * o)
 {
 	if(!o) return;
 	
-	addProperty("Remove after", o->removeAfterVisit, false);
-	addProperty("Human trigger", o->humanActivate, false);
-	addProperty("Cpu trigger", o->computerActivate, false);
+	addProperty(QObject::tr("Remove after"), o->removeAfterVisit, false);
+	addProperty(QObject::tr("Human trigger"), o->humanActivate, false);
+	addProperty(QObject::tr("Cpu trigger"), o->computerActivate, false);
 	//ui8 availableFor; //players whom this event is available for
 }
 
@@ -435,15 +456,15 @@ void Inspector::updateProperties(CGSeerHut * o)
 {
 	if(!o || !o->quest) return;
 	
-	addProperty("First visit text", o->quest->firstVisitText, new MessageDelegate, false);
-	addProperty("Next visit text", o->quest->nextVisitText, new MessageDelegate, false);
-	addProperty("Completed text", o->quest->completedText, new MessageDelegate, false);
-	addProperty("Repeat quest", o->quest->repeatedQuest, false);
-	addProperty("Time limit", o->quest->lastDay, false);
+	addProperty(QObject::tr("First visit text"), o->quest->firstVisitText, new MessageDelegate, false);
+	addProperty(QObject::tr("Next visit text"), o->quest->nextVisitText, new MessageDelegate, false);
+	addProperty(QObject::tr("Completed text"), o->quest->completedText, new MessageDelegate, false);
+	addProperty(QObject::tr("Repeat quest"), o->quest->repeatedQuest, false);
+	addProperty(QObject::tr("Time limit"), o->quest->lastDay, false);
 	
 	{ //Quest
 		auto * delegate = new QuestDelegate(controller, *o->quest);
-		addProperty("Quest", PropertyEditorPlaceholder(), delegate, false);
+		addProperty(QObject::tr("Quest"), PropertyEditorPlaceholder(), delegate, false);
 	}
 }
 
@@ -451,8 +472,8 @@ void Inspector::updateProperties(CGQuestGuard * o)
 {
 	if(!o || !o->quest) return;
 	
-	addProperty("Reward", PropertyEditorPlaceholder(), nullptr, true);
-	addProperty("Repeat quest", o->quest->repeatedQuest, true);
+	addProperty(QObject::tr("Reward"), PropertyEditorPlaceholder(), nullptr, true);
+	addProperty(QObject::tr("Repeat quest"), o->quest->repeatedQuest, true);
 }
 
 void Inspector::updateProperties()
@@ -461,25 +482,18 @@ void Inspector::updateProperties()
 		return;
 	table->setRowCount(0); //cleanup table
 	
-	addProperty("Indentifier", obj);
-	addProperty("ID", obj->ID.getNum());
-	addProperty("SubID", obj->subID);
-	addProperty("InstanceName", obj->instanceName);
-	addProperty("TypeName", obj->typeName);
-	addProperty("SubTypeName", obj->subTypeName);
+	addProperty(QObject::tr("Identifier"), obj);
+	addProperty(QObject::tr("ID"), obj->ID.getNum());
+	addProperty(QObject::tr("SubID"), obj->subID);
+	addProperty(QObject::tr("InstanceName"), obj->instanceName);
 	
-	if(!dynamic_cast<CGHeroInstance*>(obj))
+	if(obj->ID != Obj::HERO_PLACEHOLDER && !dynamic_cast<CGHeroInstance*>(obj))
 	{
 		auto factory = VLC->objtypeh->getHandlerFor(obj->ID, obj->subID);
-		addProperty("IsStatic", factory->isStaticObject());
+		addProperty(QObject::tr("IsStatic"), factory->isStaticObject());
 	}
 	
-	auto * delegate = new InspectorDelegate();
-	delegate->options.push_back({QObject::tr("neutral"), QVariant::fromValue(PlayerColor::NEUTRAL.getNum())});
-	for(int p = 0; p < controller.map()->players.size(); ++p)
-		if(controller.map()->players[p].canAnyonePlay())
-			delegate->options.push_back({QString::fromStdString(GameConstants::PLAYER_COLOR_NAMES[p]), QVariant::fromValue(PlayerColor(p).getNum())});
-	addProperty("Owner", obj->tempOwner, delegate, true);
+	addProperty(QObject::tr("Owner"), obj->tempOwner, new OwnerDelegate(controller), true);
 	
 	UPDATE_OBJ_PROPERTIES(CArmedInstance);
 	UPDATE_OBJ_PROPERTIES(CGResource);
@@ -493,7 +507,7 @@ void Inspector::updateProperties()
 	UPDATE_OBJ_PROPERTIES(CGHeroPlaceholder);
 	UPDATE_OBJ_PROPERTIES(CGHeroInstance);
 	UPDATE_OBJ_PROPERTIES(CGSignBottle);
-	UPDATE_OBJ_PROPERTIES(CGLighthouse);
+	UPDATE_OBJ_PROPERTIES(FlaggableMapObject);
 	UPDATE_OBJ_PROPERTIES(CRewardableObject);
 	UPDATE_OBJ_PROPERTIES(CGPandoraBox);
 	UPDATE_OBJ_PROPERTIES(CGEvent);
@@ -526,7 +540,7 @@ void Inspector::setProperty(const QString & key, const QVariant & value)
 	if(!obj)
 		return;
 	
-	if(key == "Owner")
+	if(key == QObject::tr("Owner"))
 		obj->tempOwner = PlayerColor(value.toInt());
 	
 	SET_PROPERTIES(CArmedInstance);
@@ -541,7 +555,7 @@ void Inspector::setProperty(const QString & key, const QVariant & value)
 	SET_PROPERTIES(CGHeroInstance);
 	SET_PROPERTIES(CGShipyard);
 	SET_PROPERTIES(CGSignBottle);
-	SET_PROPERTIES(CGLighthouse);
+	SET_PROPERTIES(FlaggableMapObject);
 	SET_PROPERTIES(CRewardableObject);
 	SET_PROPERTIES(CGPandoraBox);
 	SET_PROPERTIES(CGEvent);
@@ -554,7 +568,7 @@ void Inspector::setProperty(CArmedInstance * o, const QString & key, const QVari
 	if(!o) return;
 }
 
-void Inspector::setProperty(CGLighthouse * o, const QString & key, const QVariant & value)
+void Inspector::setProperty(FlaggableMapObject * o, const QString & key, const QVariant & value)
 {
 	if(!o) return;
 }
@@ -568,7 +582,7 @@ void Inspector::setProperty(CGPandoraBox * o, const QString & key, const QVarian
 {
 	if(!o) return;
 	
-	if(key == "Message")
+	if(key == QObject::tr("Message"))
 		o->message = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
 			TextIdentifier("guards", o->instanceName, "message"), value.toString().toStdString()));
 }
@@ -577,13 +591,13 @@ void Inspector::setProperty(CGEvent * o, const QString & key, const QVariant & v
 {
 	if(!o) return;
 	
-	if(key == "Remove after")
+	if(key == QObject::tr("Remove after"))
 		o->removeAfterVisit = value.toBool();
 	
-	if(key == "Human trigger")
+	if(key == QObject::tr("Human trigger"))
 		o->humanActivate = value.toBool();
 	
-	if(key == "Cpu trigger")
+	if(key == QObject::tr("Cpu trigger"))
 		o->computerActivate = value.toBool();
 }
 
@@ -591,16 +605,19 @@ void Inspector::setProperty(CGTownInstance * o, const QString & key, const QVari
 {
 	if(!o) return;
 	
-	if(key == "Town name")
+	if(key == QObject::tr("Town name"))
 		o->setNameTextId(mapRegisterLocalizedString("map", *controller.map(),
 			TextIdentifier("town", o->instanceName, "name"), value.toString().toStdString()));
+
+	if(key == QObject::tr("Same as player"))
+		o->alignmentToPlayer = PlayerColor(value.toInt());
 }
 
 void Inspector::setProperty(CGSignBottle * o, const QString & key, const QVariant & value)
 {
 	if(!o) return;
 	
-	if(key == "Message")
+	if(key == QObject::tr("Message"))
 		o->message = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
 			TextIdentifier("sign", o->instanceName, "message"), value.toString().toStdString()));
 }
@@ -609,7 +626,7 @@ void Inspector::setProperty(CGMine * o, const QString & key, const QVariant & va
 {
 	if(!o) return;
 	
-	if(key == "Productivity")
+	if(key == QObject::tr("Productivity"))
 		o->producedQuantity = value.toString().toInt();
 }
 
@@ -617,11 +634,11 @@ void Inspector::setProperty(CGArtifact * o, const QString & key, const QVariant
 {
 	if(!o) return;
 	
-	if(key == "Message")
+	if(key == QObject::tr("Message"))
 		o->message = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
 			TextIdentifier("guards", o->instanceName, "message"), value.toString().toStdString()));
 	
-	if(o->storedArtifact && key == "Spell")
+	if(o->storedArtifact && key == QObject::tr("Spell"))
 	{
 		o->storedArtifact = ArtifactUtils::createScroll(SpellID(value.toInt()));
 	}
@@ -631,7 +648,7 @@ void Inspector::setProperty(CGDwelling * o, const QString & key, const QVariant
 {
 	if(!o) return;
 	
-	if(key == "Same as town")
+	if(key == QObject::tr("Same as town"))
 	{
 		if (!o->randomizationInfo.has_value())
 			o->randomizationInfo = CGDwellingRandomizationInfo();
@@ -646,7 +663,7 @@ void Inspector::setProperty(CGGarrison * o, const QString & key, const QVariant
 {
 	if(!o) return;
 	
-	if(key == "Removable units")
+	if(key == QObject::tr("Removable units"))
 		o->removableUnits = value.toBool();
 }
 
@@ -654,7 +671,7 @@ void Inspector::setProperty(CGHeroPlaceholder * o, const QString & key, const QV
 {
 	if(!o) return;
 
-	if(key == "Placeholder type")
+	if(key == QObject::tr("Placeholder type"))
 	{
 		if(value.toBool())
 		{
@@ -672,10 +689,10 @@ void Inspector::setProperty(CGHeroPlaceholder * o, const QString & key, const QV
 		updateProperties();
 	}
 	
-	if(key == "Power rank")
+	if(key == QObject::tr("Power rank"))
 		o->powerRank = value.toInt();
 	
-	if(key == "Hero type")
+	if(key == QObject::tr("Hero type"))
 	{
 		o->heroType = HeroTypeID(value.toInt());
 	}
@@ -685,34 +702,38 @@ void Inspector::setProperty(CGHeroInstance * o, const QString & key, const QVari
 {
 	if(!o) return;
 	
-	if(key == "Gender")
+	if(key == QObject::tr("Gender"))
 		o->gender = EHeroGender(value.toInt());
 	
-	if(key == "Name")
+	if(key == QObject::tr("Name"))
 		o->nameCustomTextId = mapRegisterLocalizedString("map", *controller.map(),
 			TextIdentifier("hero", o->instanceName, "name"), value.toString().toStdString());
 	
-	if(key == "Biography")
+	if(key == QObject::tr("Biography"))
 		o->biographyCustomTextId = mapRegisterLocalizedString("map", *controller.map(),
 			TextIdentifier("hero", o->instanceName, "biography"), value.toString().toStdString());
 	
-	if(key == "Experience")
+	if(key == QObject::tr("Experience"))
 		o->exp = value.toString().toInt();
 	
-	if(key == "Hero type")
+	if(key == QObject::tr("Hero type"))
 	{
-		for(auto t : VLC->heroh->objects)
+		for(auto const & t : VLC->heroh->objects)
 		{
 			if(t->getId() == value.toInt())
-			{
 				o->subID = value.toInt();
-				o->type = t.get();
-			}
 		}
-		o->gender = o->type->gender;
-		o->randomizeArmy(o->type->heroClass->faction);
+		o->gender = o->getHeroType()->gender;
+		o->randomizeArmy(o->getHeroType()->heroClass->faction);
 		updateProperties(); //updating other properties after change
 	}
+
+	if(key == QObject::tr("Patrol radius"))
+	{
+		auto radius = value.toInt();
+		o->patrol.patrolRadius = radius;
+		o->patrol.patrolling = radius != CGHeroInstance::NO_PATROLLING;
+	}
 }
 
 void Inspector::setProperty(CGShipyard * o, const QString & key, const QVariant & value)
@@ -724,7 +745,7 @@ void Inspector::setProperty(CGResource * o, const QString & key, const QVariant
 {
 	if(!o) return;
 	
-	if(key == "Amount")
+	if(key == QObject::tr("Amount"))
 		o->amount = value.toString().toInt();
 }
 
@@ -732,16 +753,16 @@ void Inspector::setProperty(CGCreature * o, const QString & key, const QVariant
 {
 	if(!o) return;
 	
-	if(key == "Message")
+	if(key == QObject::tr("Message"))
 		o->message = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
 			TextIdentifier("monster", o->instanceName, "message"), value.toString().toStdString()));
-	if(key == "Character")
+	if(key == QObject::tr("Character"))
 		o->character = CGCreature::Character(value.toInt());
-	if(key == "Never flees")
+	if(key == QObject::tr("Never flees"))
 		o->neverFlees = value.toBool();
-	if(key == "Not growing")
+	if(key == QObject::tr("Not growing"))
 		o->notGrowingTeam = value.toBool();
-	if(key == "Amount")
+	if(key == QObject::tr("Amount"))
 		o->stacks[SlotID(0)]->count = value.toString().toInt();
 }
 
@@ -749,18 +770,18 @@ void Inspector::setProperty(CGSeerHut * o, const QString & key, const QVariant &
 {
 	if(!o) return;
 	
-	if(key == "First visit text")
+	if(key == QObject::tr("First visit text"))
 		o->quest->firstVisitText = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
 			TextIdentifier("quest", o->instanceName, "firstVisit"), value.toString().toStdString()));
-	if(key == "Next visit text")
+	if(key == QObject::tr("Next visit text"))
 		o->quest->nextVisitText = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
 			TextIdentifier("quest", o->instanceName, "nextVisit"), value.toString().toStdString()));
-	if(key == "Completed text")
+	if(key == QObject::tr("Completed text"))
 		o->quest->completedText = MetaString::createFromTextID(mapRegisterLocalizedString("map", *controller.map(),
 			TextIdentifier("quest", o->instanceName, "completed"), value.toString().toStdString()));
-	if(key == "Repeat quest")
+	if(key == QObject::tr("Repeat quest"))
 		o->quest->repeatedQuest = value.toBool();
-	if(key == "Time limit")
+	if(key == QObject::tr("Time limit"))
 		o->quest->lastDay = value.toString().toInt();
 }
 
@@ -828,6 +849,7 @@ QTableWidgetItem * Inspector::addProperty(const QString & value)
 {
 	auto * item = new QTableWidgetItem(value);
 	item->setFlags(Qt::NoItemFlags);
+	item->setToolTip(value);
 	return item;
 }
 
@@ -844,8 +866,10 @@ QTableWidgetItem * Inspector::addProperty(const PlayerColor & value)
 	if(value == PlayerColor::NEUTRAL)
 		str = QObject::tr("neutral");
 	
+	MetaString playerStr;
+	playerStr.appendName(value);
 	if(value.isValidPlayer())
-		str = QString::fromStdString(GameConstants::PLAYER_COLOR_NAMES[value]);
+		str = QString::fromStdString(playerStr.toString());
 	
 	auto * item = new QTableWidgetItem(str);
 	item->setFlags(Qt::NoItemFlags);
@@ -855,7 +879,9 @@ QTableWidgetItem * Inspector::addProperty(const PlayerColor & value)
 
 QTableWidgetItem * Inspector::addProperty(const GameResID & value)
 {
-	auto * item = new QTableWidgetItem(QString::fromStdString(GameConstants::RESOURCE_NAMES[value.toEnum()]));
+	MetaString str;
+	str.appendName(value);
+	auto * item = new QTableWidgetItem(QString::fromStdString(str.toString()));
 	item->setFlags(Qt::NoItemFlags);
 	item->setData(Qt::UserRole, QVariant::fromValue(value.getNum()));
 	return item;
@@ -867,7 +893,7 @@ QTableWidgetItem * Inspector::addProperty(CGCreature::Character value)
 	item->setFlags(Qt::NoItemFlags);
 	item->setData(Qt::UserRole, QVariant::fromValue(int(value)));
 	
-	for(auto & i : CharacterIdentifiers)
+	for(auto & i : characterIdentifiers)
 	{
 		if(i.second.toInt() == value)
 		{
@@ -879,10 +905,28 @@ QTableWidgetItem * Inspector::addProperty(CGCreature::Character value)
 	return item;
 }
 
+QTableWidgetItem * Inspector::addProperty(const std::optional<CGDwellingRandomizationInfo> & value)
+{
+	QString text = QObject::tr("Select town");
+	if(value.has_value() && !value->instanceId.empty())
+		text = QString::fromStdString(value->instanceId);
+
+	auto * item = new QTableWidgetItem(text);
+	item->setFlags(Qt::NoItemFlags);
+	return item;
+}
+
 //========================================================================
 
 Inspector::Inspector(MapController & c, CGObjectInstance * o, QTableWidget * t): obj(o), table(t), controller(c)
 {
+	characterIdentifiers = {
+		{ QObject::tr("Compliant"), QVariant::fromValue(int(CGCreature::Character::COMPLIANT)) },
+		{ QObject::tr("Friendly"), QVariant::fromValue(int(CGCreature::Character::FRIENDLY)) },
+		{ QObject::tr("Aggressive"), QVariant::fromValue(int(CGCreature::Character::AGGRESSIVE)) },
+		{ QObject::tr("Hostile"), QVariant::fromValue(int(CGCreature::Character::HOSTILE)) },
+		{ QObject::tr("Savage"), QVariant::fromValue(int(CGCreature::Character::SAVAGE)) },
+	};
 }
 
 /*
@@ -927,3 +971,16 @@ void InspectorDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
 		QStyledItemDelegate::setModelData(editor, model, index);
 	}
 }
+
+OwnerDelegate::OwnerDelegate(MapController & controller, bool addNeutral)
+{
+	if(addNeutral)
+		options.push_back({QObject::tr("neutral"), QVariant::fromValue(PlayerColor::NEUTRAL.getNum()) });
+	for(int p = 0; p < controller.map()->players.size(); ++p)
+		if(controller.map()->players[p].canAnyonePlay())
+		{
+			MetaString str;
+			str.appendName(PlayerColor(p));
+			options.push_back({QString::fromStdString(str.toString()), QVariant::fromValue(PlayerColor(p).getNum()) });
+		}
+}
