@@ -13,7 +13,6 @@
 #include "../lib/VCMI_Lib.h"
 #include "../lib/CSkillHandler.h"
 #include "../lib/spells/CSpellHandler.h"
-#include "../lib/CHeroHandler.h"
 #include "../lib/CArtHandler.h"
 #include "../lib/CCreatureHandler.h"
 #include "../lib/constants/StringConstants.h"
@@ -24,6 +23,11 @@
 #include "../lib/mapObjects/CGPandoraBox.h"
 #include "../lib/mapObjects/CQuest.h"
 
+#include <vcmi/HeroTypeService.h>
+#include <vcmi/HeroType.h>
+#include <vcmi/HeroClassService.h>
+#include <vcmi/HeroClass.h>
+
 RewardsWidget::RewardsWidget(CMap & m, CRewardableObject & p, QWidget *parent) :
 	QDialog(parent),
 	map(m),
@@ -34,10 +38,10 @@ RewardsWidget::RewardsWidget(CMap & m, CRewardableObject & p, QWidget *parent) :
 	
 	//fill core elements
 	for(const auto & s : Rewardable::VisitModeString)
-		ui->visitMode->addItem(QString::fromStdString(s));
+		ui->visitMode->addItem(QString::fromUtf8(s.data(), s.size()));
 	
 	for(const auto & s : Rewardable::SelectModeString)
-		ui->selectMode->addItem(QString::fromStdString(s));
+		ui->selectMode->addItem(QString::fromUtf8(s.data(), s.size()));
 	
 	for(auto s : {"AUTO", "MODAL", "INFO"})
 		ui->windowMode->addItem(QString::fromStdString(s));
@@ -51,9 +55,11 @@ RewardsWidget::RewardsWidget(CMap & m, CRewardableObject & p, QWidget *parent) :
 	ui->lResources->setRowCount(GameConstants::RESOURCE_QUANTITY - 1);
 	for(int i = 0; i < GameConstants::RESOURCE_QUANTITY - 1; ++i)
 	{
+		MetaString str;
+		str.appendName(GameResID(i));
 		for(auto * w : {ui->rResources, ui->lResources})
 		{
-			auto * item = new QTableWidgetItem(QString::fromStdString(GameConstants::RESOURCE_NAMES[i]));
+			auto * item = new QTableWidgetItem(QString::fromStdString(str.toString()));
 			item->setData(Qt::UserRole, QVariant::fromValue(i));
 			w->setItem(i, 0, item);
 			auto * spinBox = new QSpinBox;
@@ -159,7 +165,9 @@ RewardsWidget::RewardsWidget(CMap & m, CRewardableObject & p, QWidget *parent) :
 	//fill players
 	for(auto color = PlayerColor(0); color < PlayerColor::PLAYER_LIMIT; ++color)
 	{
-		auto * item = new QListWidgetItem(QString::fromStdString(GameConstants::PLAYER_COLOR_NAMES[color.getNum()]));
+		MetaString str;
+		str.appendName(color);
+		auto * item = new QListWidgetItem(QString::fromStdString(str.toString()));
 		item->setData(Qt::UserRole, QVariant::fromValue(color.getNum()));
 		item->setFlags(item->flags() | Qt::ItemIsUserCheckable);
 		item->setCheckState(Qt::Unchecked);
@@ -455,7 +463,7 @@ void RewardsWidget::loadCurrentVisitInfo(int index)
 	}
 	for(auto & i : vinfo.reward.creatures)
 	{
-		int index = i.type->getIndex();
+		int index = i.getType()->getIndex();
 		ui->rCreatureId->setCurrentIndex(index);
 		ui->rCreatureAmount->setValue(i.count);
 		onCreatureAdd(ui->rCreatures, ui->rCreatureId, ui->rCreatureAmount);
@@ -523,7 +531,7 @@ void RewardsWidget::loadCurrentVisitInfo(int index)
 	}
 	for(auto & i : vinfo.limiter.creatures)
 	{
-		int index = i.type->getIndex();
+		int index = i.getType()->getIndex();
 		ui->lCreatureId->setCurrentIndex(index);
 		ui->lCreatureAmount->setValue(i.count);
 		onCreatureAdd(ui->lCreatures, ui->lCreatureId, ui->lCreatureAmount);
@@ -636,10 +644,12 @@ void RewardsWidget::on_visitInfoList_itemSelectionChanged()
 	if(ui->visitInfoList->currentItem() == nullptr)
 	{
 		ui->eventInfoGroup->hide();
+		ui->removeVisitInfo->setEnabled(false);
 		return;
 	}
 	
 	ui->eventInfoGroup->show();
+	ui->removeVisitInfo->setEnabled(true);
 }
 
 void RewardsWidget::on_visitInfoList_currentItemChanged(QListWidgetItem * current, QListWidgetItem * previous)
@@ -732,6 +742,7 @@ void RewardsDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, c
 	if(auto * ed = qobject_cast<RewardsWidget *>(editor))
 	{
 		ed->commitChanges();
+		updateModelData(model, index);
 	}
 	else
 	{
@@ -747,3 +758,66 @@ QWidget * RewardsDelegate::createEditor(QWidget *parent, const QStyleOptionViewI
 {
 	return new RewardsWidget(map, object, parent);
 }
+
+void RewardsDelegate::updateModelData(QAbstractItemModel * model, const QModelIndex & index) const
+{
+	QStringList textList(QObject::tr("Rewards:"));
+	for (const auto & vinfo : object.configuration.info)
+	{
+		textList += QObject::tr("Reward Message: %1").arg(QString::fromStdString(vinfo.message.toString()));
+		textList += QObject::tr("Hero Level: %1").arg(vinfo.reward.heroLevel);
+		textList += QObject::tr("Hero Experience: %1").arg(vinfo.reward.heroExperience);
+		textList += QObject::tr("Mana Diff: %1").arg(vinfo.reward.manaDiff);
+		textList += QObject::tr("Mana Percentage: %1").arg(vinfo.reward.manaPercentage);
+		textList += QObject::tr("Move Points: %1").arg(vinfo.reward.movePoints);
+		textList += QObject::tr("Move Percentage: %1").arg(vinfo.reward.movePercentage);
+		textList += QObject::tr("Primary Skills: %1/%2/%3/%4").arg(vinfo.reward.primary[0]).arg(vinfo.reward.primary[1]).arg(vinfo.reward.primary[2]).arg(vinfo.reward.primary[3]);
+		QStringList resourcesList;
+		for(GameResID resource = GameResID::WOOD; resource < GameResID::COUNT ; resource++)
+		{
+			if(resource == GameResID::MITHRIL)
+				continue; // translated as "Abandoned"?
+			if(vinfo.reward.resources[resource] == 0)
+				continue;
+			MetaString str;
+			str.appendName(resource);
+			resourcesList += QString("%1: %2").arg(QString::fromStdString(str.toString())).arg(vinfo.reward.resources[resource]);
+		}
+		textList += QObject::tr("Resources: %1").arg(resourcesList.join(", "));
+		QStringList artifactsList;
+		for (auto artifact : vinfo.reward.artifacts)
+		{
+			artifactsList += QString::fromStdString(VLC->artifacts()->getById(artifact)->getNameTranslated());
+		}
+		textList += QObject::tr("Artifacts: %1").arg(artifactsList.join(", "));
+		QStringList spellsList;
+		for (auto spell : vinfo.reward.spells)
+		{
+			spellsList += QString::fromStdString(VLC->spells()->getById(spell)->getNameTranslated());
+		}
+		textList += QObject::tr("Spells: %1").arg(spellsList.join(", "));
+		QStringList secondarySkillsList;
+		for(auto & [skill, skillLevel] : vinfo.reward.secondary)
+		{
+			secondarySkillsList += QString("%1 (%2)").arg(QString::fromStdString(VLC->skills()->getById(skill)->getNameTranslated())).arg(skillLevel);
+		}
+		textList += QObject::tr("Secondary Skills: %1").arg(secondarySkillsList.join(", "));
+		QStringList creaturesList;
+		for (auto & creature : vinfo.reward.creatures)
+		{
+			creaturesList += QString("%1 %2").arg(creature.count).arg(QString::fromStdString(creature.getType()->getNamePluralTranslated()));
+		}
+		textList += QObject::tr("Creatures: %1").arg(creaturesList.join(", "));
+		if (vinfo.reward.spellCast.first != SpellID::NONE)
+		{
+			textList += QObject::tr("Spell Cast: %1 (%2)").arg(QString::fromStdString(VLC->spells()->getById(vinfo.reward.spellCast.first)->getNameTranslated())).arg(vinfo.reward.spellCast.second);
+		}
+		QStringList bonusesList;
+		for (auto & bonus : vinfo.reward.bonuses)
+		{
+			bonusesList += QString("%1 %2 (%3)").arg(QString::fromStdString(vstd::findKey(bonusDurationMap, bonus.duration))).arg(QString::fromStdString(vstd::findKey(bonusNameMap, bonus.type))).arg(bonus.val);
+		}
+		textList += QObject::tr("Bonuses: %1").arg(bonusesList.join(", "));
+	}
+	setModelTextData(model, index, textList);
+}
