@@ -7,12 +7,16 @@
  * Full text of license available in license.txt file, in main folder
  *
  */
-#include <QApplication>
 #include "StdInc.h"
+#include <QApplication>
 #include "mainwindow.h"
 
 int main(int argc, char * argv[])
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+	QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
+
 	QApplication vcmieditor(argc, argv);
 	MainWindow mainWindow;
 	return vcmieditor.exec();
