@@ -16,13 +16,17 @@
 #include <QFile>
 #include <QMessageBox>
 #include <QFileInfo>
+#include <QDialog>
+#include <QListWidget>
 
 #include "../lib/VCMIDirs.h"
 #include "../lib/VCMI_Lib.h"
 #include "../lib/logging/CBasicLogConfigurator.h"
 #include "../lib/CConfigHandler.h"
 #include "../lib/filesystem/Filesystem.h"
+#include "../lib/filesystem/CMemoryBuffer.h"
 #include "../lib/GameConstants.h"
+#include "../lib/campaign/CampaignHandler.h"
 #include "../lib/mapObjectConstructors/AObjectTypeHandler.h"
 #include "../lib/mapObjectConstructors/CObjectClassesHandler.h"
 #include "../lib/mapObjects/ObjectTemplate.h"
@@ -30,6 +34,7 @@
 #include "../lib/mapping/CMap.h"
 #include "../lib/mapping/CMapEditManager.h"
 #include "../lib/mapping/MapFormat.h"
+#include "../lib/mapping/MapFormatJson.h"
 #include "../lib/modding/ModIncompatibility.h"
 #include "../lib/RoadHandler.h"
 #include "../lib/RiverHandler.h"
@@ -133,17 +138,33 @@ void MainWindow::parseCommandLine(ExtractionOptions & extractionOptions)
 void MainWindow::loadTranslation()
 {
 #ifdef ENABLE_QT_TRANSLATIONS
-	const std::string translationFile = settings["general"]["language"].String() + ".qm";
-	logGlobal->info("Loading translation '%s'", translationFile);
+	const std::string translationFile = settings["general"]["language"].String()+ ".qm";
+	QString translationFileResourcePath = QString{":/translation/%1"}.arg(translationFile.c_str());
 
-	if (!translator.load(QString{":/translation/%1"}.arg(translationFile.c_str())))
+	logGlobal->info("Loading translation %s", translationFile);
+
+	if(!QFile::exists(translationFileResourcePath))
 	{
-		logGlobal->error("Failed to load translation");
+		logGlobal->debug("Translation file %s does not exist", translationFileResourcePath.toStdString());
+		return;
+	}
+
+	if (!translator.load(translationFileResourcePath))
+	{
+		logGlobal->error("Failed to load translation file %s", translationFileResourcePath.toStdString());
+		return;
+	}
+
+	if(translationFile == "english.qm")
+	{
+		// translator doesn't need to be installed for English
 		return;
 	}
 
 	if (!qApp->installTranslator(&translator))
-		logGlobal->error("Failed to install translator");
+	{
+		logGlobal->error("Failed to install translator for translation file %s", translationFileResourcePath.toStdString());
+	}
 #endif
 }
 
@@ -166,6 +187,7 @@ MainWindow::MainWindow(QWidget* parent) :
 	console = new CConsoleHandler();
 	logConfig = new CBasicLogConfigurator(logPath, console);
 	logConfig->configureDefault();
+	logGlobal->info("Starting map editor of '%s'", GameConstants::VCMI_VERSION);
 	logGlobal->info("The log file will be saved to %s", logPath);
 
 	//init
@@ -205,6 +227,8 @@ MainWindow::MainWindow(QWidget* parent) :
 	ui->toolFill->setIcon(QIcon{":/icons/tool-fill.png"});
 	ui->toolSelect->setIcon(QIcon{":/icons/tool-select.png"});
 	ui->actionOpen->setIcon(QIcon{":/icons/document-open.png"});
+	ui->actionOpenRecent->setIcon(QIcon{":/icons/document-open-recent.png"});
+	ui->menuOpenRecent->setIcon(QIcon{":/icons/document-open-recent.png"});
 	ui->actionSave->setIcon(QIcon{":/icons/document-save.png"});
 	ui->actionNew->setIcon(QIcon{":/icons/document-new.png"});
 	ui->actionLevel->setIcon(QIcon{":/icons/toggle-underground.png"});
@@ -248,6 +272,8 @@ MainWindow::MainWindow(QWidget* parent) :
 	scenePreview = new QGraphicsScene(this);
 	ui->objectPreview->setScene(scenePreview);
 
+	connect(ui->actionOpenRecentMore, &QAction::triggered, this, &MainWindow::on_actionOpenRecent_triggered);
+
 	//loading objects
 	loadObjectsTree();
 	
@@ -301,7 +327,7 @@ void MainWindow::setStatusMessage(const QString & status)
 
 void MainWindow::setTitle()
 {
-	QString title = QString("%1%2 - %3 (v%4)").arg(filename, unsaved ? "*" : "", VCMI_EDITOR_NAME, VCMI_EDITOR_VERSION);
+	QString title = QString("%1%2 - %3 (%4)").arg(filename, unsaved ? "*" : "", VCMI_EDITOR_NAME, GameConstants::VCMI_VERSION.c_str());
 	setWindowTitle(title);
 }
 
@@ -375,6 +401,27 @@ std::unique_ptr<CMap> MainWindow::openMapInternal(const QString & filenameSelect
 		throw std::runtime_error("Corrupted map");
 }
 
+std::shared_ptr<CampaignState> MainWindow::openCampaignInternal(const QString & filenameSelect)
+{
+	QFileInfo fi(filenameSelect);
+	std::string fname = fi.fileName().toStdString();
+	std::string fdir = fi.dir().path().toStdString();
+	
+	ResourcePath resId("MAPEDITOR/" + fname, EResType::CAMPAIGN);
+	
+	//addFilesystem takes care about memory deallocation if case of failure, no memory leak here
+	auto * mapEditorFilesystem = new CFilesystemLoader("MAPEDITOR/", fdir, 0);
+	CResourceHandler::removeFilesystem("local", "mapEditor");
+	CResourceHandler::addFilesystem("local", "mapEditor", mapEditorFilesystem);
+	
+	if(!CResourceHandler::get("mapEditor")->existsResource(resId))
+		throw std::runtime_error("Cannot open campaign from this folder");
+	if(auto campaign = CampaignHandler::getCampaign(resId.getName()))
+		return campaign;
+	else
+		throw std::runtime_error("Corrupted campaign");
+}
+
 bool MainWindow::openMap(const QString & filenameSelect)
 {
 	try
@@ -384,20 +431,32 @@ bool MainWindow::openMap(const QString & filenameSelect)
 	catch(const ModIncompatibility & e)
 	{
 		assert(e.whatExcessive().empty());
-		QMessageBox::warning(this, "Mods are required", QString::fromStdString(e.whatMissing()));
+		QMessageBox::warning(this, tr("Mods are required"), QString::fromStdString(e.whatMissing()));
 		return false;
 	}
 	catch(const std::exception & e)
 	{
-		QMessageBox::critical(this, "Failed to open map", tr(e.what()));
+		QMessageBox::critical(this, tr("Failed to open map"), tr(e.what()));
 		return false;
 	}
 	
 	filename = filenameSelect;
 	initializeMap(controller.map()->version != EMapFormat::VCMI);
+
+	updateRecentMenu(filenameSelect);
+
 	return true;
 }
 
+void MainWindow::updateRecentMenu(const QString & filenameSelect) {
+	QSettings s(Ui::teamName, Ui::appName);
+	QStringList recentFiles = s.value(recentlyOpenedFilesSetting).toStringList();
+	recentFiles.removeAll(filenameSelect);
+	recentFiles.prepend(filenameSelect);
+	constexpr int maxRecentFiles = 10;
+	s.setValue(recentlyOpenedFilesSetting, QStringList(recentFiles.mid(0, maxRecentFiles)));
+}
+
 void MainWindow::on_actionOpen_triggered()
 {
 	if(!getAnswerAboutUnsavedChanges())
@@ -412,6 +471,90 @@ void MainWindow::on_actionOpen_triggered()
 	openMap(filenameSelect);
 }
 
+void MainWindow::on_actionOpenRecent_triggered()
+{
+	QSettings s(Ui::teamName, Ui::appName);
+	QStringList recentFiles = s.value(recentlyOpenedFilesSetting).toStringList();
+
+	class RecentFileDialog : public QDialog
+	{
+
+	public:
+		RecentFileDialog(const QStringList& recentFiles, QWidget *parent)
+			: QDialog(parent), layout(new QVBoxLayout(this)), listWidget(new QListWidget(this))
+		{
+			setMinimumWidth(600);
+
+			connect(listWidget, &QListWidget::itemActivated, this, [this](QListWidgetItem *item)
+			{
+				accept();
+			});
+
+			for (const QString &file : recentFiles)
+			{
+				QListWidgetItem *item = new QListWidgetItem(file);
+				listWidget->addItem(item);
+			}
+
+			// Select most recent items by default.
+			// This enables a "CTRL+R => Enter"-workflow instead of "CTRL+R => 'mouse click on first item'"
+			if(listWidget->count() > 0)
+			{
+				listWidget->item(0)->setSelected(true);
+			}
+
+			layout->setSizeConstraint(QLayout::SetMaximumSize);
+			layout->addWidget(listWidget);
+		}
+
+		QString getSelectedFilePath() const
+		{
+			return listWidget->currentItem()->text();
+		}
+
+	private:
+		QVBoxLayout * layout;
+		QListWidget * listWidget;
+	};
+
+	RecentFileDialog d(recentFiles, this);
+	d.setWindowTitle(tr("Recently Opened Files"));
+	if(d.exec() == QDialog::Accepted && getAnswerAboutUnsavedChanges())
+	{
+		openMap(d.getSelectedFilePath());
+	}
+}
+
+void MainWindow::on_menuOpenRecent_aboutToShow()
+{
+	// Clear all actions except "More...", lest the list will grow with each
+	// showing of the list
+	for (QAction* action : ui->menuOpenRecent->actions()) {
+		if (action != ui->actionOpenRecentMore) {
+			ui->menuOpenRecent->removeAction(action);
+		}
+	}
+
+	QSettings s(Ui::teamName, Ui::appName);
+	QStringList recentFiles = s.value(recentlyOpenedFilesSetting).toStringList();
+
+	// Dynamically populate menuOpenRecent with one action per file.
+	for (const QString & file : recentFiles) {
+		QAction *action = new QAction(file, this);
+		ui->menuOpenRecent->insertAction(ui->actionOpenRecentMore, action);
+		connect(action, &QAction::triggered, this, [this, file]() {
+			if(!getAnswerAboutUnsavedChanges())
+				return;
+			openMap(file);
+		});
+	}
+
+	// Finally add a separator between recent entries and "More..."
+	if(recentFiles.size() > 0) {
+		ui->menuOpenRecent->insertSeparator(ui->actionOpenRecentMore);
+	}
+}
+
 void MainWindow::saveMap()
 {
 	if(!controller.map())
@@ -428,14 +571,27 @@ void MainWindow::saveMap()
 	
 	if(!issues.empty())
 	{
+		auto mapValidationTitle = tr("Map validation");
 		if(critical)
-			QMessageBox::warning(this, "Map validation", "Map has critical problems and most probably will not be playable. Open Validator from the Map menu to see issues found");
+			QMessageBox::warning(this, mapValidationTitle, tr("Map has critical problems and most probably will not be playable. Open Validator from the Map menu to see issues found"));
 		else
-			QMessageBox::information(this, "Map validation", "Map has some errors. Open Validator from the Map menu to see issues found");
+			QMessageBox::information(this, mapValidationTitle, tr("Map has some errors. Open Validator from the Map menu to see issues found"));
 	}
 	
 	Translations::cleanupRemovedItems(*controller.map());
 
+	for(auto obj : controller.map()->objects)
+	{
+		if(obj->ID == Obj::HERO_PLACEHOLDER)
+		{
+			auto hero = dynamic_cast<CGHeroPlaceholder *>(obj.get());
+			if(hero->heroType.has_value())
+			{
+				controller.map()->reservedCampaignHeroes.insert(hero->heroType.value());
+			}
+		}
+	}
+
 	CMapService mapService;
 	try
 	{
@@ -443,7 +599,7 @@ void MainWindow::saveMap()
 	}
 	catch(const std::exception & e)
 	{
-		QMessageBox::critical(this, "Failed to save map", e.what());
+		QMessageBox::critical(this, tr("Failed to save map"), e.what());
 		return;
 	}
 	
@@ -505,7 +661,7 @@ void MainWindow::roadOrRiverButtonClicked(ui8 type, bool isRoad)
 	controller.commitRoadOrRiverChange(mapLevel, type, isRoad);
 }
 
-void MainWindow::addGroupIntoCatalog(const std::string & groupName, bool staticOnly)
+void MainWindow::addGroupIntoCatalog(const QString & groupName, bool staticOnly)
 {
 	auto knownObjects = VLC->objtypeh->knownObjects();
 	for(auto ID : knownObjects)
@@ -517,13 +673,13 @@ void MainWindow::addGroupIntoCatalog(const std::string & groupName, bool staticO
 	}
 }
 
-void MainWindow::addGroupIntoCatalog(const std::string & groupName, bool useCustomName, bool staticOnly, int ID)
+void MainWindow::addGroupIntoCatalog(const QString & groupName, bool useCustomName, bool staticOnly, int ID)
 {
 	QStandardItem * itemGroup = nullptr;
-	auto itms = objectsModel.findItems(QString::fromStdString(groupName));
+	auto itms = objectsModel.findItems(groupName);
 	if(itms.empty())
 	{
-		itemGroup = new QStandardItem(QString::fromStdString(groupName));
+		itemGroup = new QStandardItem(groupName);
 		objectsModel.appendRow(itemGroup);
 	}
 	else
@@ -612,7 +768,7 @@ void MainWindow::loadObjectsTree()
 	{
 		auto *b = new QPushButton(QString::fromStdString(terrain->getNameTranslated()));
 		ui->terrainLayout->addWidget(b);
-		connect(b, &QPushButton::clicked, this, [this, terrain]{ terrainButtonClicked(terrain->getId()); });
+		connect(b, &QPushButton::clicked, this, [this, terrainID=terrain->getId()]{ terrainButtonClicked(terrainID); });
 
 		//filter
 		QString displayName = QString::fromStdString(terrain->getNameTranslated());
@@ -627,7 +783,7 @@ void MainWindow::loadObjectsTree()
 	{
 		auto *b = new QPushButton(QString::fromStdString(road->getNameTranslated()));
 		ui->roadLayout->addWidget(b);
-		connect(b, &QPushButton::clicked, this, [this, road]{ roadOrRiverButtonClicked(road->getIndex(), true); });
+		connect(b, &QPushButton::clicked, this, [this, roadID=road->getIndex()]{ roadOrRiverButtonClicked(roadID, true); });
 	}
 	//add spacer to keep terrain button on the top
 	ui->roadLayout->addItem(new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding));
@@ -636,7 +792,7 @@ void MainWindow::loadObjectsTree()
 	{
 		auto *b = new QPushButton(QString::fromStdString(river->getNameTranslated()));
 		ui->riverLayout->addWidget(b);
-		connect(b, &QPushButton::clicked, this, [this, river]{ roadOrRiverButtonClicked(river->getIndex(), false); });
+		connect(b, &QPushButton::clicked, this, [this, riverID=river->getIndex()]{ roadOrRiverButtonClicked(riverID, false); });
 	}
 	//add spacer to keep terrain button on the top
 	ui->riverLayout->addItem(new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding));
@@ -655,142 +811,162 @@ void MainWindow::loadObjectsTree()
 	ui->treeView->setSelectionMode(QAbstractItemView::SingleSelection);
 	connect(ui->treeView->selectionModel(), SIGNAL(currentChanged(const QModelIndex &, const QModelIndex &)), this, SLOT(treeViewSelected(const QModelIndex &, const QModelIndex &)));
 
+	//groups
+	enum GroupCat { TOWNS, OBJECTS, HEROES, ARTIFACTS, RESOURCES, BANKS, DWELLINGS, GROUNDS, TELEPORTS, MINES, TRIGGERS, MONSTERS, QUESTS, WOG_OBJECTS, OBSTACLES, OTHER };
+	QMap<GroupCat, QString> groups = {
+		{ TOWNS,       tr("Towns")       },
+		{ OBJECTS,     tr("Objects")     },
+		{ HEROES,      tr("Heroes")      },
+		{ ARTIFACTS,   tr("Artifacts")   },
+		{ RESOURCES,   tr("Resources")   },
+		{ BANKS,       tr("Banks")       },
+		{ DWELLINGS,   tr("Dwellings")   },
+		{ GROUNDS,     tr("Grounds")     },
+		{ TELEPORTS,   tr("Teleports")   },
+		{ MINES,       tr("Mines")       },
+		{ TRIGGERS,    tr("Triggers")    },
+		{ MONSTERS,    tr("Monsters")    },
+		{ QUESTS,      tr("Quests")      },
+		{ WOG_OBJECTS, tr("Wog Objects") },
+		{ OBSTACLES,   tr("Obstacles")   },
+		{ OTHER,       tr("Other")       },
+	};
 
 	//adding objects
-	addGroupIntoCatalog("TOWNS", false, false, Obj::TOWN);
-	addGroupIntoCatalog("TOWNS", false, false, Obj::RANDOM_TOWN);
-	addGroupIntoCatalog("TOWNS", true, false, Obj::SHIPYARD);
-	addGroupIntoCatalog("TOWNS", true, false, Obj::GARRISON);
-	addGroupIntoCatalog("TOWNS", true, false, Obj::GARRISON2);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::ARENA);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::BUOY);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::CARTOGRAPHER);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::SWAN_POND);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::COVER_OF_DARKNESS);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::CORPSE);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::FAERIE_RING);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::FOUNTAIN_OF_FORTUNE);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::FOUNTAIN_OF_YOUTH);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::GARDEN_OF_REVELATION);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::HILL_FORT);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::IDOL_OF_FORTUNE);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::LIBRARY_OF_ENLIGHTENMENT);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::LIGHTHOUSE);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::SCHOOL_OF_MAGIC);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::MAGIC_SPRING);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::MAGIC_WELL);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::MERCENARY_CAMP);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::MERMAID);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::MYSTICAL_GARDEN);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::OASIS);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::LEAN_TO);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::OBELISK);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::REDWOOD_OBSERVATORY);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::PILLAR_OF_FIRE);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::STAR_AXIS);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::RALLY_FLAG);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::WATERING_HOLE);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::SCHOLAR);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::SHRINE_OF_MAGIC_INCANTATION);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::SHRINE_OF_MAGIC_GESTURE);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::SHRINE_OF_MAGIC_THOUGHT);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::SIRENS);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::STABLES);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::TAVERN);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::TEMPLE);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::DEN_OF_THIEVES);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::LEARNING_STONE);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::TREE_OF_KNOWLEDGE);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::WAGON);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::SCHOOL_OF_WAR);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::WAR_MACHINE_FACTORY);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::WARRIORS_TOMB);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::WITCH_HUT);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::SANCTUARY);
-	addGroupIntoCatalog("OBJECTS", true, false, Obj::MARLETTO_TOWER);
-	addGroupIntoCatalog("HEROES", true, false, Obj::PRISON);
-	addGroupIntoCatalog("HEROES", false, false, Obj::HERO);
-	addGroupIntoCatalog("HEROES", false, false, Obj::RANDOM_HERO);
-	addGroupIntoCatalog("HEROES", false, false, Obj::HERO_PLACEHOLDER);
-	addGroupIntoCatalog("HEROES", false, false, Obj::BOAT);
-	addGroupIntoCatalog("ARTIFACTS", true, false, Obj::ARTIFACT);
-	addGroupIntoCatalog("ARTIFACTS", false, false, Obj::RANDOM_ART);
-	addGroupIntoCatalog("ARTIFACTS", false, false, Obj::RANDOM_TREASURE_ART);
-	addGroupIntoCatalog("ARTIFACTS", false, false, Obj::RANDOM_MINOR_ART);
-	addGroupIntoCatalog("ARTIFACTS", false, false, Obj::RANDOM_MAJOR_ART);
-	addGroupIntoCatalog("ARTIFACTS", false, false, Obj::RANDOM_RELIC_ART);
-	addGroupIntoCatalog("ARTIFACTS", true, false, Obj::SPELL_SCROLL);
-	addGroupIntoCatalog("ARTIFACTS", true, false, Obj::PANDORAS_BOX);
-	addGroupIntoCatalog("RESOURCES", true, false, Obj::RANDOM_RESOURCE);
-	addGroupIntoCatalog("RESOURCES", false, false, Obj::RESOURCE);
-	addGroupIntoCatalog("RESOURCES", true, false, Obj::SEA_CHEST);
-	addGroupIntoCatalog("RESOURCES", true, false, Obj::TREASURE_CHEST);
-	addGroupIntoCatalog("RESOURCES", true, false, Obj::CAMPFIRE);
-	addGroupIntoCatalog("RESOURCES", true, false, Obj::SHIPWRECK_SURVIVOR);
-	addGroupIntoCatalog("RESOURCES", true, false, Obj::FLOTSAM);
-	addGroupIntoCatalog("BANKS", true, false, Obj::CREATURE_BANK);
-	addGroupIntoCatalog("BANKS", true, false, Obj::DRAGON_UTOPIA);
-	addGroupIntoCatalog("BANKS", true, false, Obj::CRYPT);
-	addGroupIntoCatalog("BANKS", true, false, Obj::DERELICT_SHIP);
-	addGroupIntoCatalog("BANKS", true, false, Obj::PYRAMID);
-	addGroupIntoCatalog("BANKS", true, false, Obj::SHIPWRECK);
-	addGroupIntoCatalog("DWELLINGS", true, false, Obj::CREATURE_GENERATOR1);
-	addGroupIntoCatalog("DWELLINGS", true, false, Obj::CREATURE_GENERATOR2);
-	addGroupIntoCatalog("DWELLINGS", true, false, Obj::CREATURE_GENERATOR3);
-	addGroupIntoCatalog("DWELLINGS", true, false, Obj::CREATURE_GENERATOR4);
-	addGroupIntoCatalog("DWELLINGS", true, false, Obj::REFUGEE_CAMP);
-	addGroupIntoCatalog("DWELLINGS", false, false, Obj::RANDOM_DWELLING);
-	addGroupIntoCatalog("DWELLINGS", false, false, Obj::RANDOM_DWELLING_LVL);
-	addGroupIntoCatalog("DWELLINGS", false, false, Obj::RANDOM_DWELLING_FACTION);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::CURSED_GROUND1);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::MAGIC_PLAINS1);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::CLOVER_FIELD);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::CURSED_GROUND2);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::EVIL_FOG);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::FAVORABLE_WINDS);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::FIERY_FIELDS);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::HOLY_GROUNDS);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::LUCID_POOLS);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::MAGIC_CLOUDS);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::MAGIC_PLAINS2);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::ROCKLANDS);
-	addGroupIntoCatalog("GROUNDS", true, false, Obj::HOLE);
-	addGroupIntoCatalog("TELEPORTS", true, false, Obj::MONOLITH_ONE_WAY_ENTRANCE);
-	addGroupIntoCatalog("TELEPORTS", true, false, Obj::MONOLITH_ONE_WAY_EXIT);
-	addGroupIntoCatalog("TELEPORTS", true, false, Obj::MONOLITH_TWO_WAY);
-	addGroupIntoCatalog("TELEPORTS", true, false, Obj::SUBTERRANEAN_GATE);
-	addGroupIntoCatalog("TELEPORTS", true, false, Obj::WHIRLPOOL);
-	addGroupIntoCatalog("MINES", true, false, Obj::MINE);
-	addGroupIntoCatalog("MINES", false, false, Obj::ABANDONED_MINE);
-	addGroupIntoCatalog("MINES", true, false, Obj::WINDMILL);
-	addGroupIntoCatalog("MINES", true, false, Obj::WATER_WHEEL);
-	addGroupIntoCatalog("TRIGGERS", true, false, Obj::EVENT);
-	addGroupIntoCatalog("TRIGGERS", true, false, Obj::GRAIL);
-	addGroupIntoCatalog("TRIGGERS", true, false, Obj::SIGN);
-	addGroupIntoCatalog("TRIGGERS", true, false, Obj::OCEAN_BOTTLE);
-	addGroupIntoCatalog("MONSTERS", false, false, Obj::MONSTER);
-	addGroupIntoCatalog("MONSTERS", true, false, Obj::RANDOM_MONSTER);
-	addGroupIntoCatalog("MONSTERS", true, false, Obj::RANDOM_MONSTER_L1);
-	addGroupIntoCatalog("MONSTERS", true, false, Obj::RANDOM_MONSTER_L2);
-	addGroupIntoCatalog("MONSTERS", true, false, Obj::RANDOM_MONSTER_L3);
-	addGroupIntoCatalog("MONSTERS", true, false, Obj::RANDOM_MONSTER_L4);
-	addGroupIntoCatalog("MONSTERS", true, false, Obj::RANDOM_MONSTER_L5);
-	addGroupIntoCatalog("MONSTERS", true, false, Obj::RANDOM_MONSTER_L6);
-	addGroupIntoCatalog("MONSTERS", true, false, Obj::RANDOM_MONSTER_L7);
-	addGroupIntoCatalog("QUESTS", true, false, Obj::SEER_HUT);
-	addGroupIntoCatalog("QUESTS", true, false, Obj::BORDER_GATE);
-	addGroupIntoCatalog("QUESTS", true, false, Obj::QUEST_GUARD);
-	addGroupIntoCatalog("QUESTS", true, false, Obj::HUT_OF_MAGI);
-	addGroupIntoCatalog("QUESTS", true, false, Obj::EYE_OF_MAGI);
-	addGroupIntoCatalog("QUESTS", true, false, Obj::BORDERGUARD);
-	addGroupIntoCatalog("QUESTS", true, false, Obj::KEYMASTER);
-	addGroupIntoCatalog("wog object", true, false, Obj::WOG_OBJECT);
-	addGroupIntoCatalog("OBSTACLES", true);
-	addGroupIntoCatalog("OTHER", false);
+	addGroupIntoCatalog(groups[TOWNS], false, false, Obj::TOWN);
+	addGroupIntoCatalog(groups[TOWNS], false, false, Obj::RANDOM_TOWN);
+	addGroupIntoCatalog(groups[TOWNS], true, false, Obj::SHIPYARD);
+	addGroupIntoCatalog(groups[TOWNS], true, false, Obj::GARRISON);
+	addGroupIntoCatalog(groups[TOWNS], true, false, Obj::GARRISON2);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::ARENA);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::BUOY);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::CARTOGRAPHER);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::SWAN_POND);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::COVER_OF_DARKNESS);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::CORPSE);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::FAERIE_RING);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::FOUNTAIN_OF_FORTUNE);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::FOUNTAIN_OF_YOUTH);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::GARDEN_OF_REVELATION);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::HILL_FORT);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::IDOL_OF_FORTUNE);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::LIBRARY_OF_ENLIGHTENMENT);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::LIGHTHOUSE);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::SCHOOL_OF_MAGIC);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::MAGIC_SPRING);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::MAGIC_WELL);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::MERCENARY_CAMP);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::MERMAID);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::MYSTICAL_GARDEN);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::OASIS);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::LEAN_TO);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::OBELISK);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::REDWOOD_OBSERVATORY);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::PILLAR_OF_FIRE);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::STAR_AXIS);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::RALLY_FLAG);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::WATERING_HOLE);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::SCHOLAR);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::SHRINE_OF_MAGIC_INCANTATION);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::SHRINE_OF_MAGIC_GESTURE);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::SHRINE_OF_MAGIC_THOUGHT);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::SIRENS);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::STABLES);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::TAVERN);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::TEMPLE);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::DEN_OF_THIEVES);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::LEARNING_STONE);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::TREE_OF_KNOWLEDGE);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::WAGON);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::SCHOOL_OF_WAR);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::WAR_MACHINE_FACTORY);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::WARRIORS_TOMB);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::WITCH_HUT);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::SANCTUARY);
+	addGroupIntoCatalog(groups[OBJECTS], true, false, Obj::MARLETTO_TOWER);
+	addGroupIntoCatalog(groups[HEROES], true, false, Obj::PRISON);
+	addGroupIntoCatalog(groups[HEROES], false, false, Obj::HERO);
+	addGroupIntoCatalog(groups[HEROES], false, false, Obj::RANDOM_HERO);
+	addGroupIntoCatalog(groups[HEROES], false, false, Obj::HERO_PLACEHOLDER);
+	addGroupIntoCatalog(groups[HEROES], false, false, Obj::BOAT);
+	addGroupIntoCatalog(groups[ARTIFACTS], true, false, Obj::ARTIFACT);
+	addGroupIntoCatalog(groups[ARTIFACTS], false, false, Obj::RANDOM_ART);
+	addGroupIntoCatalog(groups[ARTIFACTS], false, false, Obj::RANDOM_TREASURE_ART);
+	addGroupIntoCatalog(groups[ARTIFACTS], false, false, Obj::RANDOM_MINOR_ART);
+	addGroupIntoCatalog(groups[ARTIFACTS], false, false, Obj::RANDOM_MAJOR_ART);
+	addGroupIntoCatalog(groups[ARTIFACTS], false, false, Obj::RANDOM_RELIC_ART);
+	addGroupIntoCatalog(groups[ARTIFACTS], true, false, Obj::SPELL_SCROLL);
+	addGroupIntoCatalog(groups[ARTIFACTS], true, false, Obj::PANDORAS_BOX);
+	addGroupIntoCatalog(groups[RESOURCES], true, false, Obj::RANDOM_RESOURCE);
+	addGroupIntoCatalog(groups[RESOURCES], false, false, Obj::RESOURCE);
+	addGroupIntoCatalog(groups[RESOURCES], true, false, Obj::SEA_CHEST);
+	addGroupIntoCatalog(groups[RESOURCES], true, false, Obj::TREASURE_CHEST);
+	addGroupIntoCatalog(groups[RESOURCES], true, false, Obj::CAMPFIRE);
+	addGroupIntoCatalog(groups[RESOURCES], true, false, Obj::SHIPWRECK_SURVIVOR);
+	addGroupIntoCatalog(groups[RESOURCES], true, false, Obj::FLOTSAM);
+	addGroupIntoCatalog(groups[BANKS], true, false, Obj::CREATURE_BANK);
+	addGroupIntoCatalog(groups[BANKS], true, false, Obj::DRAGON_UTOPIA);
+	addGroupIntoCatalog(groups[BANKS], true, false, Obj::CRYPT);
+	addGroupIntoCatalog(groups[BANKS], true, false, Obj::DERELICT_SHIP);
+	addGroupIntoCatalog(groups[BANKS], true, false, Obj::PYRAMID);
+	addGroupIntoCatalog(groups[BANKS], true, false, Obj::SHIPWRECK);
+	addGroupIntoCatalog(groups[DWELLINGS], true, false, Obj::CREATURE_GENERATOR1);
+	addGroupIntoCatalog(groups[DWELLINGS], true, false, Obj::CREATURE_GENERATOR2);
+	addGroupIntoCatalog(groups[DWELLINGS], true, false, Obj::CREATURE_GENERATOR3);
+	addGroupIntoCatalog(groups[DWELLINGS], true, false, Obj::CREATURE_GENERATOR4);
+	addGroupIntoCatalog(groups[DWELLINGS], true, false, Obj::REFUGEE_CAMP);
+	addGroupIntoCatalog(groups[DWELLINGS], false, false, Obj::RANDOM_DWELLING);
+	addGroupIntoCatalog(groups[DWELLINGS], false, false, Obj::RANDOM_DWELLING_LVL);
+	addGroupIntoCatalog(groups[DWELLINGS], false, false, Obj::RANDOM_DWELLING_FACTION);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::CURSED_GROUND1);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::MAGIC_PLAINS1);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::CLOVER_FIELD);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::CURSED_GROUND2);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::EVIL_FOG);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::FAVORABLE_WINDS);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::FIERY_FIELDS);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::HOLY_GROUNDS);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::LUCID_POOLS);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::MAGIC_CLOUDS);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::MAGIC_PLAINS2);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::ROCKLANDS);
+	addGroupIntoCatalog(groups[GROUNDS], true, false, Obj::HOLE);
+	addGroupIntoCatalog(groups[TELEPORTS], true, false, Obj::MONOLITH_ONE_WAY_ENTRANCE);
+	addGroupIntoCatalog(groups[TELEPORTS], true, false, Obj::MONOLITH_ONE_WAY_EXIT);
+	addGroupIntoCatalog(groups[TELEPORTS], true, false, Obj::MONOLITH_TWO_WAY);
+	addGroupIntoCatalog(groups[TELEPORTS], true, false, Obj::SUBTERRANEAN_GATE);
+	addGroupIntoCatalog(groups[TELEPORTS], true, false, Obj::WHIRLPOOL);
+	addGroupIntoCatalog(groups[MINES], true, false, Obj::MINE);
+	addGroupIntoCatalog(groups[MINES], false, false, Obj::ABANDONED_MINE);
+	addGroupIntoCatalog(groups[MINES], true, false, Obj::WINDMILL);
+	addGroupIntoCatalog(groups[MINES], true, false, Obj::WATER_WHEEL);
+	addGroupIntoCatalog(groups[TRIGGERS], true, false, Obj::EVENT);
+	addGroupIntoCatalog(groups[TRIGGERS], true, false, Obj::GRAIL);
+	addGroupIntoCatalog(groups[TRIGGERS], true, false, Obj::SIGN);
+	addGroupIntoCatalog(groups[TRIGGERS], true, false, Obj::OCEAN_BOTTLE);
+	addGroupIntoCatalog(groups[MONSTERS], false, false, Obj::MONSTER);
+	addGroupIntoCatalog(groups[MONSTERS], true, false, Obj::RANDOM_MONSTER);
+	addGroupIntoCatalog(groups[MONSTERS], true, false, Obj::RANDOM_MONSTER_L1);
+	addGroupIntoCatalog(groups[MONSTERS], true, false, Obj::RANDOM_MONSTER_L2);
+	addGroupIntoCatalog(groups[MONSTERS], true, false, Obj::RANDOM_MONSTER_L3);
+	addGroupIntoCatalog(groups[MONSTERS], true, false, Obj::RANDOM_MONSTER_L4);
+	addGroupIntoCatalog(groups[MONSTERS], true, false, Obj::RANDOM_MONSTER_L5);
+	addGroupIntoCatalog(groups[MONSTERS], true, false, Obj::RANDOM_MONSTER_L6);
+	addGroupIntoCatalog(groups[MONSTERS], true, false, Obj::RANDOM_MONSTER_L7);
+	addGroupIntoCatalog(groups[QUESTS], true, false, Obj::SEER_HUT);
+	addGroupIntoCatalog(groups[QUESTS], true, false, Obj::BORDER_GATE);
+	addGroupIntoCatalog(groups[QUESTS], true, false, Obj::QUEST_GUARD);
+	addGroupIntoCatalog(groups[QUESTS], true, false, Obj::HUT_OF_MAGI);
+	addGroupIntoCatalog(groups[QUESTS], true, false, Obj::EYE_OF_MAGI);
+	addGroupIntoCatalog(groups[QUESTS], true, false, Obj::BORDERGUARD);
+	addGroupIntoCatalog(groups[QUESTS], true, false, Obj::KEYMASTER);
+	addGroupIntoCatalog(groups[WOG_OBJECTS], true, false, Obj::WOG_OBJECT);
+	addGroupIntoCatalog(groups[OBSTACLES], true);
+	addGroupIntoCatalog(groups[OTHER], false);
 	}
 	catch(const std::exception &)
 	{
-		QMessageBox::critical(this, "Mods loading problem", "Critical error during Mods loading. Disable invalid mods and restart.");
+		QMessageBox::critical(this, tr("Mods loading problem"), tr("Critical error during Mods loading. Disable invalid mods and restart."));
 	}
 }
 
@@ -803,10 +979,12 @@ void MainWindow::on_actionLevel_triggered()
 		ui->minimapView->setScene(controller.miniScene(mapLevel));
 		if (mapLevel == 0)
 		{
+			ui->actionLevel->setText(tr("View underground"));
 			ui->actionLevel->setToolTip(tr("View underground"));
 		}
 		else
 		{
+			ui->actionLevel->setText(tr("View surface"));
 			ui->actionLevel->setToolTip(tr("View surface"));
 		}
 	}
@@ -1094,7 +1272,7 @@ void MainWindow::on_actionUpdate_appearance_triggered()
 			continue;
 		}
 		
-		auto * terrain = controller.map()->getTile(obj->visitablePos()).terType;
+		auto * terrain = controller.map()->getTile(obj->visitablePos()).getTerrain();
 		
 		if(handler->isStaticObject())
 		{
@@ -1221,6 +1399,53 @@ void MainWindow::on_actionh3m_converter_triggered()
 	}
 }
 
+void MainWindow::on_actionh3c_converter_triggered()
+{
+	auto campaignFile = QFileDialog::getOpenFileName(this, tr("Select campaign to convert"),
+		QString::fromStdString(VCMIDirs::get().userDataPath().make_preferred().string()),
+		tr("HoMM3 campaigns (*.h3c)"));
+	if(campaignFile.isEmpty())
+		return;
+	
+	auto campaignFileDest = QFileDialog::getSaveFileName(this, tr("Select destination file"),
+		QString::fromStdString(VCMIDirs::get().userDataPath().make_preferred().string()),
+		tr("VCMI campaigns (*.vcmp)"));
+	if(campaignFileDest.isEmpty())
+		return;
+	
+	QFileInfo fileInfo(campaignFileDest);
+	if(fileInfo.suffix().toLower() != "vcmp")
+		campaignFileDest += ".vcmp";
+	auto campaign = openCampaignInternal(campaignFile);
+
+	auto jsonCampaign = CampaignHandler::writeHeaderToJson(*campaign);
+	
+	std::shared_ptr<CIOApi> io(new CDefaultIOApi());
+	auto saver = std::make_shared<CZipSaver>(io, campaignFileDest.toStdString());
+	for(auto & scenario : campaign->allScenarios())
+	{
+		CMapService mapService;
+		auto map = campaign->getMap(scenario, nullptr);
+		controller.repairMap(map.get());
+		CMemoryBuffer serializeBuffer;
+		{
+			CMapSaverJson jsonSaver(&serializeBuffer);
+			jsonSaver.saveMap(map);
+		}
+
+		auto mapName = boost::algorithm::to_lower_copy(campaign->scenario(scenario).mapName);
+		mapName = boost::replace_all_copy(mapName, ".h3m", std::string("")) + ".vmap";
+
+		auto stream = saver->addFile(mapName);
+		stream->write(reinterpret_cast<const ui8 *>(serializeBuffer.getBuffer().data()), serializeBuffer.getSize());
+
+		jsonCampaign["scenarios"].Vector().push_back(CampaignHandler::writeScenarioToJson(campaign->scenario(scenario)));
+		jsonCampaign["scenarios"].Vector().back()["map"].String() = mapName;
+	}
+
+	auto jsonCampaignStr = jsonCampaign.toString();
+	saver->addFile("header.json")->write(reinterpret_cast<const ui8 *>(jsonCampaignStr.data()), jsonCampaignStr.length());
+}
 
 void MainWindow::on_actionLock_triggered()
 {
