@@ -11,6 +11,7 @@ class ObjectBrowserProxyModel;
 
 VCMI_LIB_NAMESPACE_BEGIN
 class CMap;
+class CampaignState;
 class CGObjectInstance;
 VCMI_LIB_NAMESPACE_END
 
@@ -28,12 +29,14 @@ class MainWindow : public QMainWindow
 	const QString mainWindowSizeSetting = "MainWindow/Size";
 	const QString mainWindowPositionSetting = "MainWindow/Position";
 	const QString lastDirectorySetting = "MainWindow/Directory";
+	const QString recentlyOpenedFilesSetting = "MainWindow/RecentlyOpenedFiles";
 
 #ifdef ENABLE_QT_TRANSLATIONS
 	QTranslator translator;
 #endif
 	
 	std::unique_ptr<CMap> openMapInternal(const QString &);
+	std::shared_ptr<CampaignState> openCampaignInternal(const QString &);
 
 public:
     explicit MainWindow(QWidget *parent = nullptr);
@@ -58,6 +61,10 @@ public:
 
 private slots:
 	void on_actionOpen_triggered();
+	
+	void on_actionOpenRecent_triggered();
+
+	void on_menuOpenRecent_aboutToShow();
 
 	void on_actionSave_as_triggered();
 
@@ -113,6 +120,8 @@ private slots:
 	
 	void on_actionh3m_converter_triggered();
 
+	void on_actionh3c_converter_triggered();
+
 	void on_actionLock_triggered();
 
 	void on_actionUnlock_triggered();
@@ -153,8 +162,8 @@ public slots:
 
 private:
 	void preparePreview(const QModelIndex & index);
-	void addGroupIntoCatalog(const std::string & groupName, bool staticOnly);
-	void addGroupIntoCatalog(const std::string & groupName, bool useCustomName, bool staticOnly, int ID);
+	void addGroupIntoCatalog(const QString & groupName, bool staticOnly);
+	void addGroupIntoCatalog(const QString & groupName, bool useCustomName, bool staticOnly, int ID);
 	
 	QAction * getActionPlayer(const PlayerColor &);
 
@@ -170,6 +179,8 @@ private:
 
 	void parseCommandLine(ExtractionOptions & extractionOptions);
 
+	void updateRecentMenu(const QString & filenameSelect);
+
 private:
     Ui::MainWindow * ui;
 	ObjectBrowserProxyModel * objectBrowser = nullptr;
