@@ -15,6 +15,13 @@ namespace Ui {
 class EventSettings;
 }
 
+QString toQString(const PlayerColor & player);
+QVariant toVariant(const TResources & resources);
+QVariant toVariant(const std::set<PlayerColor> & players);
+
+TResources resourcesFromVariant(const QVariant & v);
+std::set<PlayerColor> playersFromVariant(const QVariant & v);
+
 class EventSettings : public AbstractSettings
 {
 	Q_OBJECT
