@@ -11,6 +11,8 @@
 
 #include <QDialog>
 
+#include "mapcontroller.h"
+
 namespace Ui {
 class TimedEvent;
 }
@@ -20,18 +22,23 @@ class TimedEvent : public QDialog
 	Q_OBJECT
 
 public:
-	explicit TimedEvent(QListWidgetItem *, QWidget *parent = nullptr);
+	explicit TimedEvent(MapController & map, QListWidgetItem *, QWidget * parent = nullptr);
 	~TimedEvent();
 
 private slots:
 
 	void on_TimedEvent_finished(int result);
 
+	void on_addObjectToDelete_clicked();
+	void on_removeObjectToDelete_clicked();
+	void onObjectPicked(const CGObjectInstance * obj);
+	void insertObjectToDelete(const CGObjectInstance * obj);
 	void on_pushButton_clicked();
 
-	void on_resources_itemDoubleClicked(QTableWidgetItem *item);
+	void on_resources_itemDoubleClicked(QTableWidgetItem * item);
 
 private:
-	Ui::TimedEvent *ui;
+	MapController & controller;
+	Ui::TimedEvent * ui;
 	QListWidgetItem * target;
 };
