@@ -11,11 +11,13 @@
 #include "victoryconditions.h"
 #include "ui_victoryconditions.h"
 #include "../mapcontroller.h"
-#include "../lib/CGeneralTextHandler.h"
-#include "../lib/constants/StringConstants.h"
-#include "../../lib/mapObjects/CGCreature.h"
 
-#include "../inspector/townbulidingswidget.h" //to convert BuildingID to string
+#include "../../lib/constants/StringConstants.h"
+#include "../../lib/entities/faction/CTownHandler.h"
+#include "../../lib/mapObjects/CGCreature.h"
+#include "../../lib/texts/CGeneralTextHandler.h"
+
+#include "../inspector/townbuildingswidget.h" //to convert BuildingID to string
 
 VictoryConditions::VictoryConditions(QWidget *parent) :
 	AbstractSettings(parent),
@@ -46,7 +48,7 @@ void VictoryConditions::initialize(MapController & c)
 
 	for(auto & s : conditionStringsWin)
 	{
-		ui->victoryComboBox->addItem(QString::fromStdString(s));
+		ui->victoryComboBox->addItem(tr(s.c_str()));
 	}
 	ui->standardVictoryCheck->setChecked(false);
 	ui->onlyForHumansCheck->setChecked(false);
@@ -405,7 +407,9 @@ void VictoryConditions::on_victoryComboBox_currentIndexChanged(int index)
 			{
 				for(int resType = 0; resType < GameConstants::RESOURCE_QUANTITY; ++resType)
 				{
-					auto resName = QString::fromStdString(GameConstants::RESOURCE_NAMES[resType]);
+					MetaString str;
+					str.appendName(GameResID(resType));
+					auto resName = QString::fromStdString(str.toString());
 					victoryTypeWidget->addItem(resName, QVariant::fromValue(resType));
 				}
 			}
@@ -425,7 +429,7 @@ void VictoryConditions::on_victoryComboBox_currentIndexChanged(int index)
 
 			victorySelectWidget = new QComboBox;
 			ui->victoryParamsLayout->addWidget(victorySelectWidget);
-			victorySelectWidget->addItem("Any town", QVariant::fromValue(-1));
+			victorySelectWidget->addItem(tr("Any town"), QVariant::fromValue(-1));
 			for(int i : getObjectIndexes<const CGTownInstance>(*controller->map()))
 				victorySelectWidget->addItem(getTownName(*controller->map(), i).c_str(), QVariant::fromValue(i));
 			
