@@ -16,6 +16,7 @@
 #include "../lib/mapObjectConstructors/AObjectTypeHandler.h"
 #include "../lib/mapObjectConstructors/CObjectClassesHandler.h"
 #include "../lib/mapping/CMap.h"
+#include "../lib/VCMI_Lib.h"
 
 
 MinimapView::MinimapView(QWidget * parent):
@@ -46,7 +47,7 @@ void MinimapView::mouseMoveEvent(QMouseEvent *mouseEvent)
 	
 	auto pos = mapToScene(mouseEvent->pos());
 	pos *= 32;
-	emit cameraPositionChanged(pos);
+	cameraPositionChanged(pos);
 }
 
 void MinimapView::mousePressEvent(QMouseEvent* event)
@@ -89,7 +90,7 @@ void MapView::mouseMoveEvent(QMouseEvent *mouseEvent)
 	if(tile == tilePrev) //do not redraw
 		return;
 
-	emit currentCoordinates(tile.x, tile.y);
+	currentCoordinates(tile.x, tile.y);
 
 	switch(selectionTool)
 	{
@@ -363,7 +364,7 @@ void MapView::mousePressEvent(QMouseEvent *event)
 						else if(controller->map()->getTile(tile).riverType
 						   && controller->map()->getTile(tile).riverType != controller->map()->getTile(tilen).riverType)
 							continue;
-						else if(controller->map()->getTile(tile).terType != controller->map()->getTile(tilen).terType)
+						else if(controller->map()->getTile(tile).terrainType != controller->map()->getTile(tilen).terrainType)
 							continue;
 					}
 					if(event->button() == Qt::LeftButton && sc->selectionTerrainView.selection().count(tilen))
@@ -562,7 +563,7 @@ void MapView::mouseReleaseEvent(QMouseEvent *event)
 		auto selection = sc->selectionObjectsView.getSelection();
 		if(selection.size() == 1)
 		{
-			emit openObjectProperties(*selection.begin(), tab);
+			openObjectProperties(*selection.begin(), tab);
 		}
 		break;
 	}
@@ -617,7 +618,7 @@ void MapView::dropEvent(QDropEvent * event)
 		{
 			auto * obj = sc->selectionObjectsView.newObject;
 			controller->commitObjectCreate(sc->level);
-			emit openObjectProperties(obj, false);
+			openObjectProperties(obj, false);
 		}
 		else
 		{
@@ -735,13 +736,13 @@ void MapScene::updateViews()
 void MapScene::terrainSelected(bool anythingSelected)
 {
 	isTerrainSelected = anythingSelected;
-	emit selected(isTerrainSelected || isObjectSelected);
+	selected(isTerrainSelected || isObjectSelected);
 }
 
 void MapScene::objectSelected(bool anythingSelected)
 {
 	isObjectSelected = anythingSelected;
-	emit selected(isTerrainSelected || isObjectSelected);
+	selected(isTerrainSelected || isObjectSelected);
 }
 
 MinimapScene::MinimapScene(int lvl):
