@@ -99,11 +99,13 @@ int BattleCbProxy::getUnitByPos(lua_State * L)
 	if(!S.tryGet(1, object))
 		return S.retVoid();
 
-	BattleHex hex;
+	si16 hexVal;
 
-	if(!S.tryGet(2, hex.hex))
+	if(!S.tryGet(2, hexVal))
 		return S.retNil();
 
+	BattleHex hex(hexVal);
+
 	bool onlyAlive;
 
 	if(!S.tryGet(3, onlyAlive))
