@@ -19,7 +19,7 @@ using namespace ::testing;
 class CArtifactTest : public Test
 {
 public:
-	MOCK_METHOD3(registarCb, void(int32_t, const std::string &, const std::string &));
+	MOCK_METHOD4(registarCb, void(int32_t, int32_t, const std::string &, const std::string &));
 
 protected:
 	std::shared_ptr<CArtifact> subject;
@@ -32,14 +32,15 @@ protected:
 
 TEST_F(CArtifactTest, RegistersIcons)
 {
-	subject->iconIndex = 4242;
-	subject->image = "Test1";
-	subject->large = "Test2";
+        subject-> setImage(4242, "Test1", "Test2");
 
-	auto cb = std::bind(&CArtifactTest::registarCb, this, _1, _2, _3);
+	auto cb = [this](auto && PH1, auto && PH2, auto && PH3, auto && PH4) 
+	{
+		registarCb(std::forward<decltype(PH1)>(PH1), std::forward<decltype(PH2)>(PH2), std::forward<decltype(PH3)>(PH3), std::forward<decltype(PH4)>(PH4));
+	};
 
-	EXPECT_CALL(*this, registarCb(Eq(4242), "ARTIFACT", "Test1"));
-	EXPECT_CALL(*this, registarCb(Eq(4242), "ARTIFACTLARGE", "Test2"));
+	EXPECT_CALL(*this, registarCb(Eq(4242), Eq(0), "ARTIFACT", "Test1"));
+	EXPECT_CALL(*this, registarCb(Eq(4242), Eq(0), "ARTIFACTLARGE", "Test2"));
 
 	subject->registerIcons(cb);
 }
