@@ -27,7 +27,12 @@ void GameCallbackMock::setGameState(CGameState * gameState)
 	gs = gameState;
 }
 
-void GameCallbackMock::sendAndApply(CPackForClient * pack)
+void GameCallbackMock::sendAndApply(CPackForClient & pack)
 {
 	upperCallback->apply(pack);
 }
+
+vstd::RNG & GameCallbackMock::getRandomGenerator()
+{
+	throw std::runtime_error("Not implemented!");
+}
