@@ -29,9 +29,9 @@ class MapServiceMock : public IMapService
 public:
 	MapServiceMock(const std::string & path, MapListener * mapListener_);
 
-	std::unique_ptr<CMap> loadMap(const ResourcePath & name) const override;
+	std::unique_ptr<CMap> loadMap(const ResourcePath & name, IGameCallback * cb) const override;
 	std::unique_ptr<CMapHeader> loadMapHeader(const ResourcePath & name) const override;
-	std::unique_ptr<CMap> loadMap(const ui8 * buffer, int size, const std::string & name, const std::string & modName, const std::string & encoding) const override;
+	std::unique_ptr<CMap> loadMap(const ui8 * buffer, int size, const std::string & name, const std::string & modName, const std::string & encoding, IGameCallback * cb) const override;
 	std::unique_ptr<CMapHeader> loadMapHeader(const ui8 * buffer, int size, const std::string & name, const std::string & modName, const std::string & encoding) const override;
 
 	void saveMap(const std::unique_ptr<CMap> & map, boost::filesystem::path fullPath) const override;
