@@ -10,8 +10,7 @@
 
 #pragma once
 
-#include "../../lib/NetPacks.h"
-#include "../../lib/CGameState.h"
+#include "../../lib/gameState/CGameState.h"
 
 namespace test
 {
